/*
 * This source file is part of the Atlantis Little Helper program.
 * Copyright (C) 2001 Maxim Shariy.
 *
 * Atlantis Little Helper is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Atlantis Little Helper is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Atlantis Little Helper; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>
#include <string.h>
#include "files.h"
#include "consts.h"
#include "cstr.h"
#include "collection.h"
#include "cfgfile.h"

#include "objs.h"
#include "data.h"
#include "atlaparser.h"
#include "errs.h"
#include "consts_ah.h" // not very good, but will do for now
#include <time.h>
#include <math.h>

#define IN          "in "
#define CONTAINS    "contains "
#define SILVER      "silver"

#define FLAG_HDR    "$Flag"

#define YES         "Yes"
#define ORDER_CMNT  ";*** "


const char * Monthes[] = {"Jan", "Feb", "Mar",  "Apr", "May", "Jun",  "Jul", "Aug", "Sep",  "Oct", "Nov", "Dec"};

const char * EOL_MS    = "\r\n";
const char * EOL_UNIX  = "\n";
const char * EOL_SCR   = EOL_UNIX;
const char * EOL_FILE  = EOL_UNIX;

const char * STRUCT_UNIT_START = "-+*";

const char * Directions[] = {"North", "Northeast", "Southeast", "South", "Southwest", "Northwest",
                             "N",     "NE",        "SE",        "S",     "SW",        "NW"       };
//enum       eDirection     { North=0, Northeast,   Southeast,   South,   Southwest,   Northwest };
int          ExitFlags [] = { 0x01,    0x02,        0x04,        0x08,    0x10,        0x20      };
int          EntryFlags[] = { 0x08,    0x10,        0x20,        0x01,    0x02,        0x04      };


REPORT_ORDER_ERR_FUNC     ReportOrderErrLoc   = NULL;
GET_STUDY_COST_FUNC       StudyCostLoc        = NULL;
GET_STRUCTURE_ATTR_FUNC   GetStructAttrLoc    = NULL;
GET_CONF_STRING_FUNC      GetConfStringLoc    = NULL;
GET_ORDER_ID_FUNC         GetOrderIdLoc       = NULL;
IS_TRADE_ITEM_FUNC        IsTradeItemLoc      = NULL;

static const char * BUG        = " - it's a bug!";
static const char * NOSETUNIT  = " - Can not set property for unit ";
static const char * NOSET      = " - Can not set unit property ";
static const char * NOTNUMERIC = " - Property is not numeric for unit ";
static const char * NOTSTRING  = " - Property type is not string for unit ";

const char * ExitEndHeader[]  = { HDR_FACTION          ,
                                  HDR_FACTION_STATUS   ,
                                  HDR_ERRORS           ,
                                  HDR_EVENTS           ,
                                  HDR_SILVER           ,
                                  HDR_BATTLES          ,
                                  HDR_ATTACKERS        ,
                                  HDR_ATTITUDES        ,
                                  HDR_SKILLS           ,
                                  HDR_ITEMS            ,
                                  HDR_OBJECTS
                                };
int ExitEndHeaderLen[] =        { sizeof(HDR_FACTION       ) - 1,
                                  sizeof(HDR_FACTION_STATUS) - 1,
                                  sizeof(HDR_ERRORS        ) - 1,
                                  sizeof(HDR_EVENTS        ) - 1,
                                  sizeof(HDR_SILVER        ) - 1,
                                  sizeof(HDR_BATTLES       ) - 1,
                                  sizeof(HDR_ATTACKERS     ) - 1,
                                  sizeof(HDR_ATTITUDES     ) - 1,
                                  sizeof(HDR_SKILLS        ) - 1,
                                  sizeof(HDR_ITEMS         ) - 1,
                                  sizeof(HDR_OBJECTS       ) - 1
                                };


const char * BattleEndHeader[]= { HDR_ERRORS           ,
                                  HDR_EVENTS           ,
                                  HDR_SILVER           ,
                                  HDR_ATTITUDES        ,
                                  HDR_SKILLS           ,
                                  HDR_ITEMS            ,
                                  HDR_OBJECTS          ,
                                  HDR_FACTION          ,
                                  HDR_FACTION_STATUS   ,
                                  HDR_SILVER           
                                };
int BattleEndHeaderLen[] =      { sizeof(HDR_ERRORS        ) - 1,
                                  sizeof(HDR_EVENTS        ) - 1,
                                  sizeof(HDR_SILVER        ) - 1,
                                  sizeof(HDR_ATTITUDES     ) - 1,
                                  sizeof(HDR_SKILLS        ) - 1,
                                  sizeof(HDR_ITEMS         ) - 1,
                                  sizeof(HDR_OBJECTS       ) - 1,
                                  sizeof(HDR_FACTION       ) - 1,
                                  sizeof(HDR_FACTION_STATUS) - 1,
                                  sizeof(HDR_SILVER        ) - 1
                                };

                                
                                
//----------------------------------------------------------------------

BOOL IsInteger(const char * s)
{
    int n = 0;

    if ((!s) || (!*s))
        return FALSE;

    while (*s)
    {
        if ( ('-'==*s) && (n>0) )
            return FALSE;
        else
            if ( (*s<'0') || (*s>'9') )
                return FALSE;

        n++;
        s++;
    }
    return TRUE;
}

//======================================================================

CAtlaParser::CAtlaParser()
{
    int x = 2;
    x = 1/(x-2);
    //assert(0);
}

//----------------------------------------------------------------------

CAtlaParser::CAtlaParser(REPORT_ORDER_ERR_FUNC     OrderErrFunc,
                         GET_STUDY_COST_FUNC       StudyCostFunc,
                         GET_STRUCTURE_ATTR_FUNC   GetStructAttrFunc,
                         RESOLVE_ALIAS_FUNC        ResolveAliasFunc,
                         GET_ITEM_WEIGHTS          GetItemWeightsFunc,
                         GET_CONF_STRING_FUNC      GetConfStringFunc,
                         GET_ORDER_ID_FUNC         GetOrderIdFunc,
                         IS_TRADE_ITEM_FUNC        IsTradeItemFunc
                         )
            :m_sOrderErrors(256)
{
    m_CrntFactionId   = 0;
    m_ParseErr        = ERR_NOTHING;
    m_nCurLine        = 0;
    m_GatesCount      = 0;
    m_YearMon         = 0;
    m_CurYearMon      = 0;
    m_pSource         = NULL;
    m_pCurLand        = NULL;
    m_pCurStruct      = NULL;
    m_NextStructId    = 1;
    m_OrdersLoaded    = FALSE;
    m_MaxSkillDays    = 450;
    m_JoiningRep      = FALSE;
    m_IsHistory       = FALSE;

    ReportOrderErrLoc = OrderErrFunc;
    StudyCostLoc      = StudyCostFunc;
    GetStructAttrLoc  = GetStructAttrFunc;
    ResolveAliasLoc   = ResolveAliasFunc;
    GetItemWeightsLoc = GetItemWeightsFunc;
    GetConfStringLoc  = GetConfStringFunc;
    GetOrderIdLoc     = GetOrderIdFunc;
    IsTradeItemLoc    = IsTradeItemFunc;

    m_Events.Name     = "Events";
    m_Errors.Name     = "Errors";
}

//----------------------------------------------------------------------

CAtlaParser::~CAtlaParser()
{
    Clear();
}




//----------------------------------------------------------------------

void CAtlaParser::Clear()
{
    m_Factions.FreeAll();


    m_YearMon       = 0;
    m_CurYearMon    = 0;
    m_JoiningRep    = FALSE;
    m_IsHistory     = FALSE;
    m_Planes.DeleteAll();
    m_PlanesNamed.FreeAll();  // this one must go before units since it checks contents of CLand::Units collection!!!
    m_Units.FreeAll();

    m_CrntFactionId = 0;
    m_CrntFactionPwd.Empty();
    m_OurFactions.FreeAll();
    m_TaxLandStrs.FreeAll();
    m_TradeLandStrs.FreeAll();
    m_BattleLandStrs.FreeAll();
    m_UnitPropertyNames.FreeAll();
    m_UnitPropertyTypes.FreeAll();
    m_TradeUnitIds.DeleteAll();
    m_Skills.FreeAll();
    m_Items.FreeAll();
    m_Objects.FreeAll();
    m_Battles.FreeAll();
    m_Gates.FreeAll();
    m_Events.Description.Empty();
    m_Errors.Description.Empty();
}

//----------------------------------------------------------------------

BOOL CAtlaParser::ReadNextLine(CStr & s)
{
    BOOL Ok=FALSE;

    if (m_pSource)
    {
        Ok = m_pSource->GetNextLine(s);
        if (Ok)
        {
            m_nCurLine++;
            s.TrimRight(TRIM_ALL);
            s << EOL_SCR;
        }
    }
    return Ok;
};

//----------------------------------------------------------------------

void CAtlaParser::PutLineBack (CStr & s)
{
    m_nCurLine--;
    if (m_pSource)
        m_pSource->QueueString(s.GetData(), s.GetLength());
};

//----------------------------------------------------------------------

int CAtlaParser::ParseFactionInfo(BOOL GetNo)
{
    int          err    = ERR_OK;
    CStr         Line(128);
    CStr         FNo;
    CStr         Str;
    int          LineNo = 0;
    const char * p;
    const char * s;
    unsigned int i;
    CFaction   * pMyFaction;
    long         yearmon = 0;

    while ((ERR_OK==err) && ReadNextLine(Line))
    {
        m_FactionInfo << Line;

        Line.TrimRight(TRIM_ALL);

        if (Line.IsEmpty())
            break;  // stop at empty line


        if (GetNo)
            switch (LineNo)
            {
            case 0:  // faction number
                s = Line.GetData();
                p = strchr(s, '(');
                if (p)
                {
                    pMyFaction = new CFaction;
                    pMyFaction->Description << Line.GetData() << EOL_SCR;
                    pMyFaction->Name.SetStr(s, (p-s));
                    s = p+1;
                    p = strchr(s, ')');
                    FNo.SetStr(s, (p-s));
                    m_CrntFactionId = atol(FNo.GetData());
                    pMyFaction->Id  = m_CrntFactionId;
                    if (!m_Factions.Insert(pMyFaction))
                        delete pMyFaction;
                    m_OurFactions.Insert((void*)m_CrntFactionId);
                }
                break;
            case 1:  // date  December, Year 2
                p = Str.GetToken(Line.GetData(), ',');
                for (i=0; i<sizeof(Monthes)/sizeof(char*); i++)
                    if (0==Str.FindSubStr(Monthes[i]))
                    {
                        yearmon = i+1;
                        break;
                    }
                p = SkipSpaces(p);
                //while (p && *p<=' ')
                //    p++;
                p = Str.GetToken(p, ' '); // 'Year'
                p = Str.GetToken(p, ' ');
                yearmon += 100*atol(Str.GetData());

                if (m_JoiningRep && m_YearMon != yearmon)
                {
                    err = ERR_INV_TURN;
                    if (m_CrntFactionId > 0)
                        for (i=0; i<(unsigned int)m_OurFactions.Count(); i++)
                            if ((long)m_OurFactions.At(i)==m_CrntFactionId)
                            {
                                m_OurFactions.AtDelete(i);
                                break;
                            }
                }
                else
                {
                    m_YearMon    = yearmon;
                    m_CurYearMon = yearmon; // file being currently loaded may not contain year/month info
                }
                break;
            }

        LineNo++;
    }


    return err;
}

//----------------------------------------------------------------------

int  CAtlaParser::SetLandFlag(const char * p, long flag)
{
    CLand * pLand = GetLand(p);

    if (pLand)
        pLand->Flags |= flag;

    return 0;
}

//----------------------------------------------------------------------

int  CAtlaParser::SetLandFlag(long LandId, long flag)
{
    CLand * pLand;

    pLand = GetLand(LandId);
    if (pLand)
        pLand->Flags |= flag;

    return 0;
}

//----------------------------------------------------------------------

int  CAtlaParser::ApplyLandFlags()
{

    int          i, idx;
    const char * s;
    CBaseObject  Dummy;
    CUnit      * pUnit;

    for (i=0; i<m_TaxLandStrs.Count(); i++)
    {
        s = (const char*)m_TaxLandStrs.At(i);
        SetLandFlag(s, LAND_TAX);
    }

    for (i=0; i<m_TradeLandStrs.Count(); i++)
    {
        s = (const char*)m_TradeLandStrs.At(i);
        SetLandFlag(s, LAND_TRADE);
    }

    for (i=0; i<m_BattleLandStrs.Count(); i++)
    {
        s = (const char*)m_BattleLandStrs.At(i);
        SetLandFlag(s, LAND_BATTLE);
    }

    for (i=0; i<m_TradeUnitIds.Count(); i++)
    {
        Dummy.Id = (long)m_TradeUnitIds.At(i);
        if (m_Units.Search(&Dummy, idx))
        {
            pUnit = (CUnit*)m_Units.At(idx);
            SetLandFlag(pUnit->LandId, LAND_TRADE);
        }
    }

    m_TaxLandStrs.FreeAll();
    m_TradeLandStrs.FreeAll();
    m_BattleLandStrs.FreeAll();
    m_TradeUnitIds.DeleteAll();


    return 0;
}

//----------------------------------------------------------------------

//Gogie (129): Rides from plain (7,37) in Partry to plain (6,38) in
//  Partry.
//Unit (967): Collects $200 in taxes in swamp (42,4) in Isyssir.

//Santa Maria [103] sails from swamp (4,32) in Lervir to ocean (3,31) in
//  Atlantis Ocean.
//Santa Maria [103] sails from ocean (3,31) in Atlantis Ocean to ocean
//  (2,30) in Atlantis Ocean.


//Speedy (1356): Rides from swamp (7,35) in Moffat to plain (7,37) in
//  Partry.
//Speedy (1356): Rides from plain (7,37) in Partry to plain (7,39) in
//  Partry.

//Magoga (892): Walks from plain (10,34) in Grue to plain (9,35) in
//  Grue.

//Choppers (1101): Produces 13 wood [WOOD] in swamp (7,35) in Moffat.

int CAtlaParser::ParseOneEvent(CStr & EventLine, BOOL IsEvent)
{
    CUnit      * pUnit = NULL;
    const char * p;
    CStr         Buf(64);
    CStr         Buf2(64);
    CStr         Name;
    long         x;
    char         ch;
    int          idx;
    BOOL         Taken = FALSE;

    if (EventLine.IsEmpty())
        return 0;
    EventLine << EOL_SCR;


    p = Name.GetToken(EventLine.GetData(), "([", ch, TRIM_ALL);
    switch (ch)
    {
    case '(':   // it is unit
        p = Buf.GetInteger(p);
        if (*p != ')')
            break;  // it's not a unit!

        x = atol(Buf.GetData());
        if (x>0)
        {
            pUnit = MakeUnit(x);
            if (pUnit->Name.IsEmpty())
                pUnit->Name = Name;
        }
        if (pUnit)
        {
            if (IsEvent)
                pUnit->Events << EventLine;
            else
                pUnit->Errors << EventLine;
            Taken = TRUE;
        }

        if (IsEvent)
        {
            // Try to get something of the unit event...
            while (p && (*p>' ') )
                p++;
            p = SkipSpaces(p);

            p = SkipSpaces(Buf.GetToken(p, ' ', TRIM_ALL));
            if ( (0==stricmp("walks"   , Buf.GetData())) ||
                 (0==stricmp("rides"   , Buf.GetData())) ||
                 (0==stricmp("flies"   , Buf.GetData())) )
            {
                CLand * pLand1 = NULL;
                CLand * pLand2 = NULL;
                
                p = SkipSpaces(Buf.GetToken(p, ' ', TRIM_ALL));
                if (0==stricmp("from"  , Buf.GetData()))
                {
                    CStr S;
                    
                    Buf.Empty();
                    Buf2.Empty();
                    while (p)
                    {
                        p = SkipSpaces(S.GetToken(p, " \n", ch, TRIM_ALL));
                        if (0==stricmp(S.GetData(), "to"))
                            break;
                        Buf << S << ' ';
                    }
                    if (0==stricmp(S.GetData(), "to"))
                    {
                        while (p)
                        {
                            p = SkipSpaces(S.GetToken(p, " \n", ch, TRIM_ALL));
                            Buf2 << S << ' ';
                        }
                    }

                    ParseTerrain(NULL, 0, Buf, FALSE, &pLand1);

                    // It is nice to parse 'to' terrain as well, since the unit 
                    // can be killed at the destination on sight...
                    if (!Buf2.IsEmpty())
                        ParseTerrain(NULL, 0, Buf2, FALSE, &pLand2);
                    
                    // check for links between planes
                    if (pLand1 && pLand2  &&  pLand1->pPlane && pLand2->pPlane  && pLand1->pPlane != pLand2->pPlane)
                    {
                        pLand1->SetProperty(PRP_LAND_LINK, eLong, (void*)pLand2->Id, eBoth);
                        pLand2->SetProperty(PRP_LAND_LINK, eLong, (void*)pLand1->Id, eBoth);
                        m_LandsToBeLinked.Insert(pLand1);
                        m_LandsToBeLinked.Insert(pLand2);
                    }
                }
            }

            else if ( p && ('$'==*p) && ( (0==stricmp("collects", Buf.GetData())) ||
                                          (0==stricmp("pillages", Buf.GetData())) )
                    )
            {
                p = Buf.GetToken(p, '(', TRIM_ALL);
                p = Buf.GetToken(p, ')', TRIM_ALL);
                if (!m_TaxLandStrs.Search((void*)Buf.GetData(), idx))
                    m_TaxLandStrs.Insert(strdup(Buf.GetData()));
            }
            else if (0==stricmp("produces", Buf.GetData()))
            {
                p = Buf.GetToken(p, '(', TRIM_ALL);
                p = Buf.GetToken(p, ')', TRIM_ALL);
                if (!m_TradeLandStrs.Search((void*)Buf.GetData(), idx))
                    m_TradeLandStrs.Insert(strdup(Buf.GetData()));
            }

            // Performs work, is a trade activity

            // TBD: maybe implement buying and selling trade goods later
            else if (0==stricmp("performs", Buf.GetData()))
            {
                p = SkipSpaces(Buf.GetToken(p, ' ', TRIM_ALL));
                if (0==stricmp("work", Buf.GetData()))
                {
                    p = Buf.GetToken(EventLine.GetData(), '(', TRIM_ALL);
                    p = Buf.GetToken(p                  , ')', TRIM_ALL);
                    x = atol(Buf.GetData());
                    m_TradeUnitIds.Insert((void*)x);

                }
            }

        }
        break;

    case '[':   // ship, probably
        if (IsEvent)
        {
            p = Buf.GetToken(p, ']', TRIM_ALL);
            while (p && (*p>' ') )
                p++;
            p = SkipSpaces(p);
            p = SkipSpaces(Buf.GetToken(p, ' ', TRIM_ALL));
            if (0==stricmp("sails"   , Buf.GetData()))
            {
                p = SkipSpaces(Buf.GetToken(p, ' ', TRIM_ALL));
                if (0==stricmp("from"  , Buf.GetData()))
                {
                    Buf = p;
                    x = Buf.FindSubStr(" to ");
                    if (x>0)
                        Buf.DelSubStr(x, Buf.GetLength()-x);
                    ParseTerrain(NULL, 0, Buf, FALSE, NULL);
                }
            }
        }
        break;
    }

//    if (!Taken)
//        if (IsEvent)
//            m_Events.Description << EventLine;
//        else
//            m_Errors.Description << EventLine;

    if (IsEvent)
    {
        if (!Taken)
            m_Events.Description << EventLine;
    }
    else
        m_Errors.Description << EventLine;

    return 0;
}


int CAtlaParser::ParseEvents(BOOL IsEvents)
{
    int          err   = ERR_OK;
    CStr         Line(128);
    CStr         OneEvent(128);


    while ((ERR_OK==err) && ReadNextLine(Line))
    {
        Line.TrimRight(TRIM_ALL);

        if (Line.IsEmpty())
        {
            ParseOneEvent(OneEvent, IsEvents);
            break;  // stop at empty line
        }

        // comment/error may take more than one line.
        // Dot at the end is not reliable!
        if (strchr(Line.GetData(), ':') ||
            strchr(Line.GetData(), '[') ||
            (!OneEvent.IsEmpty() && ('.'==OneEvent.GetData()[OneEvent.GetLength()-1]))
           )
        {
            // That is hopefully a new event
            ParseOneEvent(OneEvent, IsEvents);
            OneEvent.Empty();
        }
        if (!OneEvent.IsEmpty())
            OneEvent << EOL_SCR;
        OneEvent << Line;

    }


    return err;
}

//----------------------------------------------------------------------

int CAtlaParser::ParseErrors()
{
    return ParseEvents(FALSE);
}

//----------------------------------------------------------------------

int CAtlaParser::ParseUnclSilver(CStr & Line)
{
    const char * p;
    const char * s;
    CStr         N;
    CFaction   * pFaction;

    m_FactionInfo << Line;

    Line.TrimRight(TRIM_ALL);
    s = Line.GetData() + sizeof(HDR_SILVER)-1;
    p = strchr(s, '.');

    if (p)
        N.SetStr(s, p-s);
    else
        N.SetStr(s);
    N.TrimLeft();

    N.TrimRight(TRIM_ALL);

    pFaction = GetFaction(m_CrntFactionId);
    if (pFaction)
        pFaction->UnclaimedSilver = atol(N.GetData());
    
    return ERR_OK;
}

//----------------------------------------------------------------------

/*
Declared Attitudes (default Neutral):
Hostile : none.
Unfriendly : none.
Neutral : none.
Friendly : none.
Ally : none.
*/
int CAtlaParser::ParseAttitudes(CStr & Line)
{
    m_FactionInfo << Line;
    while (ReadNextLine(Line))
    {
        m_FactionInfo << Line;

        Line.TrimRight(TRIM_ALL);
        if (Line.IsEmpty())
            break;  // stop at empty line
    }

    return ERR_OK;
}

//----------------------------------------------------------------------

void CAtlaParser::CheckExit(CPlane * pPlane, int Direction, CLand * pLandSrc, CLand * pLandExit)
{
    int x1,y1,x2,y2, z;

    if (0==pPlane->Width)
        switch (Direction%6)
        {
        case Northeast:

        case Southeast:
            LandIdToCoord(pLandSrc ->Id, x1, y1, z);
            LandIdToCoord(pLandExit->Id, x2, y2, z);
            if (x2<x1)
            {
                pPlane->WestEdge   = x2;
                pPlane->EastEdge   = x1;
                pPlane->Width      = pPlane->EastEdge - pPlane->WestEdge + 1;

                pPlane->EdgeSrcId  = pLandSrc ->Id;
                pPlane->EdgeExitId = pLandExit->Id;
                pPlane->EdgeDir    = Direction%6;
            }
            break;

        case Northwest:
        case Southwest:
            LandIdToCoord(pLandSrc ->Id, x1, y1, z);
            LandIdToCoord(pLandExit->Id, x2, y2, z);
            if (x2>x1)
            {
                pPlane->WestEdge   = x1;
                pPlane->EastEdge   = x2;
                pPlane->Width      = pPlane->EastEdge - pPlane->WestEdge + 1;

                pPlane->EdgeSrcId  = pLandSrc ->Id;
                pPlane->EdgeExitId = pLandExit->Id;
                pPlane->EdgeDir    = Direction%6;
            }
            break;
        }
}


//----------------------------------------------------------------------

int CAtlaParser::AnalyzeTerrain(CLand * pLand)
{
/*
plain (5,39) in Partry, contains Drimnin [city], 3217 peasants (high
  elves), $22519.
------------------------------------------------------------
  It was winter last month; it will be winter next month.
  Wages: $17 (Max: $10937).
  Wanted: 144 grain [GRAI] at $20, 108 livestock [LIVE] at $21, 33
    longbows [LBOW] at $126, 28 plate armor [PARM] at $374, 10 caviar
    [CAVI] at $158, 12 cotton [COTT] at $149.
  For Sale: 35 horses [HORS] at $62, 28 wagons [WAGO] at $157, 17
    pearls [PEAR] at $71, 16 wool [WOOL] at $76, 643 high elves [HELF]
    at $68, 128 leaders [LEAD] at $136.
  Entertainment available: $1125.
  Products: 40 grain [GRAI], 33 horses [HORS].
*/
    enum         {eMain, eSale, eWanted, eProduct, eNone} SectType;
    CStr         Section(64);
    CStr         Struct (64);
    CStr         S1     (32);
    CStr         S2     (32);
    CStr         N1     (32);
    CStr         N2     (32);
    CStr         Buf    (32);
    long         n1;
    long         n2;
    const char * src;
    const char * str;
    const char * srcold;
    const char * p;
    char         ch;
    CProduct   * pProd;
    int          delpos = 0;
    int          dellen = 0;
    int          idx;

    SectType = eMain;
    srcold   = pLand->Description.GetData();
    src      = Section.GetToken(srcold, '.', TRIM_ALL);
    while (!Section.IsEmpty())
    {
        str = Section.GetData();
        while (str)
        {
            str = Struct.GetToken(str, ":,", ch, TRIM_ALL);
            p   = Struct.GetData();
            switch(ch)
            {
            case ':': // that's a section name!
                pLand->Flags|=LAND_VISITED; // Just simple presense in the report is not enough!
                if      (0==stricmp("For Sale", p))
                    SectType = eSale;
                else if (0==stricmp("Wanted"  , p))
                    SectType = eWanted;
                else if (0==stricmp("Products", p))
                {
                    SectType = eProduct;
                    delpos   = srcold - pLand->Description.GetData();
                    dellen   = src - srcold;
                }
                break;

            case ',':
            case  0 :
                switch (SectType)
                {
                case eMain:    // recognize contains, $ and peasants
                    if ('$'==*p)
                    {
                        N1.GetInteger(++p);
                        pLand->Taxable = atol(N1.GetData());
                    }
                    else
                    {
                        p  = SkipSpaces(S1.GetToken(p, ' ', TRIM_ALL));
                        p  = S2.GetToken(p, ' ', TRIM_ALL);
                        n1 = atol(S1.GetData());
                        if (n1>0)
                        {
                            if (0==stricmp("peasants", S2.GetData()))
                                pLand->Peasants = n1;
                        }
                        else
                            if (0==stricmp("contains", S1.GetData()))
                            {
                                //pLand->CityName = S2;
                                //p = S2.GetToken(p, '[');
                                //p = pLand->CityType.GetToken(p, ']');

                                //There may be a space in the city name!
                                p = S1.GetToken(Struct.GetData(), ' ', TRIM_ALL);

                                p = pLand->CityName.GetToken(p, '[', TRIM_ALL);
                                p = pLand->CityType.GetToken(p, ']', TRIM_ALL);
                            }
                    }
                    break;

                case eSale:
                case eWanted:  // 35 horses [HORS] at $62
                               // N1 S1     [S2]   at $N2

                    p = SkipSpaces(N1.GetToken(p, ' ', TRIM_ALL));
                    p = S1.GetToken(p, '[', TRIM_ALL);
                    p = S2.GetToken(p, ']', TRIM_ALL);
                    p = N2.GetToken(p, '$', TRIM_ALL);
                    N2= p;
                    n2= atol(N2.GetData());

                    if ((!S2.IsEmpty()) && (n2>0) )
                    {
                        Buf = S2;
                        if (eSale == SectType)
                            Buf << PRP_SALE_AMOUNT_POSTFIX;
                        else
                            Buf<< PRP_WANTED_AMOUNT_POSTFIX;
                        pLand->SetProperty(Buf.GetData(), eLong, (void*)atol(N1.GetData()), eBoth);

                        Buf = S2;
                        if (eSale == SectType)
                            Buf << PRP_SALE_PRICE_POSTFIX;
                        else
                            Buf<< PRP_WANTED_PRICE_POSTFIX;
                        pLand->SetProperty(Buf.GetData(), eLong, (void*)n2, eBoth);
                    }
                    break;

                case eProduct: // Products: 40 grain [GRAI], 33 horses [HORS].
                               //           N1 S1    [S2]
                    p = SkipSpaces(N1.GetToken(p, ' ', TRIM_ALL));
                    n1= atol(N1.GetData());
                    if (0==n1)
                        if (0==stricmp(N1.GetData(), "none"))
                        n1 = -1;

                    if (n1)
                    {
                        pProd = new CProduct;
                        pProd->Amount = n1;
                        p = pProd->LongName.GetToken(p, '[', TRIM_ALL);
                        p = pProd->ShortName.GetToken(p, ']', TRIM_ALL);
                        if (pLand->Products.Search(pProd, idx))
                            pLand->Products.AtFree(idx);
                        pLand->Products.Insert(pProd);
                    }

                    break;

                default:
                    break;

                }
                break;
            }
        }
        srcold   = src;
        src      = Section.GetToken(src, '.');
        SectType = eNone;
    }

    if (dellen)
        pLand->Description.DelSubStr(delpos, dellen);

    return 0;
}

//----------------------------------------------------------------------

CPlane * CAtlaParser::MakePlane(const char * planename)
{
    static CBaseObject Dummy;  // we do not want it to be constructed/destructed all the way
                               // should be ok with multiple instances - single-threaded
    int      i;
    CPlane * pPlane;

    Dummy.Name = planename;
    if (m_PlanesNamed.Search(&Dummy, i))
        pPlane = (CPlane*)m_PlanesNamed.At(i);
    else
    {
        pPlane       = new CPlane;
        pPlane->Id   = m_Planes.Count();
        pPlane->Name = planename;
        m_PlanesNamed.Insert(pPlane);
        m_Planes.Insert(pPlane);

    }

    return pPlane;
}


//----------------------------------------------------------------------

int CAtlaParser::ParseTerrain(CLand * pMotherLand, int ExitDir, CStr & FirstLine, BOOL FullMode, CLand ** ppParsedLand)
{
// FirstLine looks somewhat like:
//    swamp (48,52[,somewhere]) in Aghleam, 118 peasants (tribesmen), $354.

    int                  err = ERR_OK;
    const char         * p;
    CStr                 Name;
    CStr                 S(64);
    long                 x, y;
    CLand              * pLand = NULL;
    CStr                 CurLine(128);
    CStr                 PlaneName(32);
    CStr                 LandName(32);
    int                  i;
    int                  idxland;
    char                 ch;
    CPlane             * pPlane;
    CStruct            * pStruct;
    BOOL                 DoBreak;
    CBaseObject        * pGate;
    CBaseObject          Dummy;
    int                  no;
    int                  idx;
    CStr                 TempDescr(64); // Land description is collected in here. It can replace already existing description

    if (FirstLine.IsEmpty())
        goto Exit;

    p = SkipSpaces(Name.GetToken(FirstLine.GetData(), " \n\t", ch, TRIM_ALL));
    if (!p || *p != '(')   // it must be '('
        goto Exit;

    // ok, now goes (xxx,yyy[,somewhere]) bla-bla-bla
    p++;
    p = S.GetToken(p, ',');
    if (!IsInteger(S.GetData()))
        goto Exit;
    x = atol(S.GetData());

    // yyy[,somewhere]) bla-bla-bla
    p = S.GetToken(p, ",)", ch);
    if (!IsInteger(S.GetData()))
        goto Exit;
    y = atol(S.GetData());
    if (','==ch)
        // we have a plane name
        p = PlaneName.GetToken(p, ')');
    else
        PlaneName = DEFAULT_PLANE;

    if (!p)
        goto Exit;

    p = SkipSpaces(S.GetToken(SkipSpaces(p), ' ', TRIM_ALL));
    if (0!=stricmp(S.GetData(),"in"))
        goto Exit;
    LandName.GetToken(p, ",.", ch, TRIM_ALL);

    pPlane = MakePlane(PlaneName.GetData());

    Dummy.Id = LandCoordToId(x,y, pPlane->Id);
    if (pPlane->Lands.Search(&Dummy, idxland))
    {
        pLand = (CLand*)pPlane->Lands.At(idxland);
    }
    else
    {
        pLand               = new CLand;
        pLand->Id           = LandCoordToId(x,y, pPlane->Id);
        pLand->Name         = LandName;
        pLand->TerrainType  = Name; //.SetStr(Name, NameEnd-Name);
        pLand->pPlane       = pPlane;
        pLand->Taxable      = 0;
        pPlane->Lands.Insert(pLand);
    }
    TempDescr = FirstLine;

    if (pMotherLand && (NULL==strchr(FirstLine.GetData(), '.')) )
    {
        // Exit description may take more then one line!
        while (ReadNextLine(CurLine))
        {
            // the line must not start from -+* and must not contain : .
            const char * s = SkipSpaces(CurLine.GetData());
            if (!s || !*s)
                break;
            
            if (strchr(STRUCT_UNIT_START, *s) || strchr(s, ':'))
            {
                PutLineBack(CurLine);
                break;
            }

            CurLine.TrimRight(TRIM_ALL);
            TempDescr << CurLine << EOL_SCR;

            if (strchr(CurLine.GetData(), '.'))
                break;
        }
    }

    if (pMotherLand && (pPlane==pMotherLand->pPlane))
        CheckExit(pPlane, ExitDir, pMotherLand, pLand);


    // It is only scan for exits from the land
    if (!FullMode)
    {
        TempDescr.TrimRight(TRIM_ALL);
        pLand->Description.TrimRight(TRIM_ALL);
        if (TempDescr.GetLength() > pLand->Description.GetLength())
            pLand->Description = TempDescr;
        AnalyzeTerrain(pLand);
        goto Exit;
    }
    
    // Structures can be destroyed, so remove those coming from history
    for (i=pLand->Structs.Count()-1; i>=0; i--)
    {
        pStruct = (CStruct*)pLand->Structs.At(i);
        if (0==(pStruct->Attr & SA_HIDDEN) &&   // keep the gates!
            0==(pStruct->Attr & SA_SHAFT ) )    // keep the shafts!
            pLand->Structs.AtFree(i);
    }
        
    // now  goes extended land description terminated by exits list
    // And check for other headers just in case!
    DoBreak = FALSE;
    while (ReadNextLine(CurLine))
    {
        CurLine.TrimRight(TRIM_ALL);
        p = SkipSpaces(CurLine.GetData());
        if (0==stricmp("Exits:", p))
            break;
        for (i=0; i<(int)sizeof(ExitEndHeader)/(int)sizeof(const char *); i++)
            if (0==strnicmp(p, ExitEndHeader[i], ExitEndHeaderLen[i] ))
            {
                CurLine << EOL_FILE;
                PutLineBack(CurLine);
                DoBreak = TRUE;
                break;
            }
        if (DoBreak)
            break;

        // remove the atlaclient's turn mark
        no=0;
        while (p && *p && '-'==*p)
        {
            no++;
            p++;
        }
        if (no>20 && ';'==*p)
        {
            no = strlen(p++);
            CurLine.DelSubStr(CurLine.GetLength()-no, no);
            pLand->AtlaclientsLastTurnNo = atol(p);
        }
            
        TempDescr << CurLine << EOL_SCR;
    }

    // When should we replace old description with the new one?
    // My guess is - everytime for the full parsing!    
    pLand->Description = TempDescr;
    pLand->Description.TrimRight(TRIM_ALL);
    AnalyzeTerrain(pLand);


    if (!m_IsHistory && m_CurYearMon>0)
        pLand->Flags |= LAND_IS_CURRENT;

    m_pCurLand   = pLand;
    m_pCurStruct = NULL;

    // now is a list of exits and gates terminated by unit or structure
    pLand->ExitBits = 0;
    while (ReadNextLine(CurLine))
    {
        if (0==SafeCmp(EOL_SCR, CurLine.GetData()))
            continue;
        DoBreak = TRUE;
        p       = S.GetToken(CurLine.GetData(), ":(", ch);
        switch (ch)
        {
        case ':':  // is it an exit?

            if (0==stricmp(S.GetData(), FLAG_HDR))

            {
                pLand->FlagText[0] = SkipSpaces(p);
                pLand->FlagText[0].TrimRight(TRIM_ALL);
            }
            for (i=0; i<(int)sizeof(Directions)/(int)sizeof(const char*); i++)
                if (0==stricmp(S.GetData(), Directions[i]))
                {
                    // yes!
                    pPlane->ExitsCount++;
                    S = p;
                    S.TrimLeft();
                    ParseTerrain(pLand, i, S, FALSE, NULL);
                    pLand->ExitBits |= ExitFlags[i];
                    DoBreak = FALSE;
                    break;
                }
            break;
        case '(': // is it a gate?
            // There is a Gate here (Gate 18 of 35).
            if (0==stricmp(S.GetData(), "There is a Gate here"))
            {
                p  = SkipSpaces(S.GetToken(p, ' '));
                p  = S.GetToken(p, ' ');  // S = 18
                no = atol(S.GetData());
                pStruct     = new CStruct;
                pStruct->Id = -no;  // negative, so it does not clash with structures!
                pStruct->Description = CurLine.GetData();
                pStruct->Kind        = STRUCT_GATE;
                pStruct->Attr        = GetStructAttrLoc(pStruct->Kind.GetData());
                pLand->AddNewStruct(pStruct);

                p = SkipSpaces(p);
                p = SkipSpaces(S.GetToken(p, ' '));  // S = of
                p = S.GetToken(p, ' ');  // S = 35
                m_GatesCount = atol(S.GetData());
                DoBreak = FALSE;


                pGate = new CBaseObject;
                pGate->Id = no;
                if (m_Gates.Search(pGate, idx))
                {
                    delete pGate;
                    pGate = (CBaseObject*)m_Gates.At(idx);
                }
                else
                    idx = -1;

                pGate->Description.Format("Gate % 4d. ", no);
                pGate->Description << pLand->TerrainType << " (" << x << "," << y;
                if (0!=stricmp(PlaneName.GetData(), DEFAULT_PLANE))
                    pGate->Description << "," << PlaneName;
                pGate->Description << ")";

                pGate->Name        = pGate->Description;

                if (idx<0)
                    m_Gates.Insert(pGate);
            }
            break;
        }

        if (DoBreak)
        {
            PutLineBack(CurLine);
            goto Exit;

        }
    }




    // list of units....
    // will be obtained separately!

Exit:
    if (ppParsedLand)
        *ppParsedLand = pLand;

    return err;
}

//----------------------------------------------------------------------

CUnit * CAtlaParser::MakeUnit(long Id)
{
    CBaseObject  Dummy;
    int          idx;
    CUnit      * pUnit;

    Dummy.Id = Id;
    if (m_Units.Search(&Dummy, idx))
    {
        pUnit    = (CUnit*)m_Units.At(idx);
    }
    else
    {
        pUnit = new CUnit;
        pUnit->Id = Id;
        m_Units.Insert(pUnit);
    }

    return pUnit;
}

//----------------------------------------------------------------------

/*
void CAtlaParser::MakeUnitAndGetOld(long Id, CUnit *& pUnit, CUnit *& pUnitOld, int & idxold)
{
    CBaseObject  Dummy;

    Dummy.Id = Id;
    if (m_Units.Search(&Dummy, idxold))
        pUnitOld = (CUnit*)m_Units.At(idxold);
    else
        pUnitOld = NULL;
    
    pUnit = new CUnit;
    pUnit->Id = Id;
}
*/

//----------------------------------------------------------------------

/*
  Taxmen (767), on guard, Yellow Pants (34), revealing faction,
  taxing, 84 high elves [HELF], vodka [VODK], spear [SPEA], 2578
  silver [SILV]. Skills: combat [COMB] 2 (90).
*/

int CAtlaParser::ParseUnit(CStr & FirstLine)
{
#define DOT          '.'
#define COMMA        ','
#define STRUCT_LIMIT ",."
#define SECT_ITEMS   "Items"
#define SECT_SKILLS  "Skills"

    CFaction    * pFaction = NULL;
    CUnit       * pUnit    = NULL;
//    CUnit       * pUnitOld = NULL;
//    int           idxold;
    CStr          CurLine (128);
    CStr          UnitText(128);
    CStr          UnitPrefix;
    CStr          Line    (128);
    CStr          FactName;
    CStr          Section ( 32);
    CStr          Buf     (128);
    CStr          S1      ( 32);
    CStr          S2      ( 32);
    CStr          N1      ( 32);
    CStr          N2      ( 32);
    CStr          CurName ( 32);
    long          n1;
    const char  * src;
    const char  * p;
    char          Delimiter;
    char          LastDelimiter = DOT;
    char          ch;
    int           err = ERR_OK;
    BOOL          SkillsFound = FALSE;

    FirstLine.TrimRight(TRIM_ALL);
    p = FirstLine.GetData();
    
    // Get the prefix - leading spaces and the first word
    while (p && *p && *p<=' ')
        UnitPrefix << *p++;
    while (p && *p && *p>' ')
        UnitPrefix << *p++;
    while (p && *p && *p<=' ')
        UnitPrefix << *p++;
    
    UnitText = p;
    UnitText << EOL_SCR;

    while (ReadNextLine(CurLine))
    {
//        CurLine.TrimLeft(TRIM_ALL);
        p = SkipSpaces(CurLine.GetData());
        
        // maybe terminated by an empty string
//        if ( CurLine.IsEmpty() )
        if (!p || !*p)
            break;

            
        // or a next unit may just follow...
        if ( (0==strnicmp(p, HDR_UNIT_ALIEN        , sizeof(HDR_UNIT_ALIEN)-1 )) ||
             (0==strnicmp(p, HDR_UNIT_OWN          , sizeof(HDR_UNIT_OWN  )-1 )) ||
             (0==strnicmp(p, HDR_STRUCTURE         , sizeof(HDR_STRUCTURE )-1 ))
           )
        {
            PutLineBack(CurLine);
            break;
        }

        CurLine.TrimRight(TRIM_ALL);
        UnitText.AddStr(CurLine.GetData(), CurLine.GetLength());

        UnitText.AddStr(EOL_SCR);
    }

    // now analize the description
    src = UnitText.GetData();
    while (src && *src) 
    {
        src  = SkipSpaces(Line.GetToken(src, STRUCT_LIMIT, Delimiter, TRIM_ALL));

        if (DOT==LastDelimiter)
        {
            // New section. Find out the section name.
            p = Section.GetToken(Line.GetData(), ':');
            if (p)
            {
                if (SkillsFound || 0==stricmp(SECT_SKILLS, Section.GetData()))

                {
                    // remove section name from the structure
                    p = SkipSpaces(p);
                    //while (*p && (*p <= ' '))
                    //    p++;
                    Line.DelSubStr(0, p-Line.GetData());
                }
                else
                {
                    // Some moron put a colon in the unit name!
                    Section = SECT_ITEMS;
                }
            }
            else
            {
                // section has no name. Is it items?
                if (!SkillsFound) //(NULL==pUnit)
                    Section = SECT_ITEMS;
                else
                {
                    // we did not think about that!!!
                    // Nah, it's just some moron put a dot in the description

                    //Buf.Format("Unit description - expected \"Something: \" after \'.\' at line %d\n", m_nCurLine);
                    //LOG_ERR(ERR_DESIGN, Buf.GetData());
                    //Section.Empty();
                }
            }
        }

/*
  Taxmen (767), on guard, Yellow Pants (34), revealing faction,
  taxing, 84 high elves [HELF], vodka [VODK], spear [SPEA], 2578
  silver [SILV]. Skills: combat [COMB] 2 (90).
*/
        // we will analize the first section (SECT_ITEMS) and SECT_SKILLS
        if      (0==stricmp(SECT_ITEMS, Section.GetData()))
        {
            if (0==stricmp(Line.GetData(), "taxing") && pUnit)
                pUnit->Flags |= UNIT_AUTOTAXING;
            
            // recognize patterns:
            // S1 (N1)
            // N1 S1 [S2]
            // S1 [S2]

            // '[' can be present in the unit or faction name, so give '(' precedence.
            p = Buf.GetToken(Line.GetData(), "(", ch);
            if (!p)
                p = Buf.GetToken(Line.GetData(), "[", ch);
            switch (ch)
            {
            case '(':
                S1 = Buf;
                p  = N1.GetToken(p, ')');
                n1 = atol(N1.GetData());
                if (p && (n1>0))
                {
                    if (NULL == pUnit)
                    {
                        // it is unit name
                        pUnit = MakeUnit(n1);
                        //MakeUnitAndGetOld(n1, pUnit, pUnitOld, idxold); // we will decide later which one we keep
                        
                        if (m_pCurLand)
                            m_pCurLand->AddUnit(pUnit);
                        if (0 == strnicmp(SkipSpaces(UnitPrefix.GetData()), HDR_UNIT_OWN, sizeof(HDR_UNIT_OWN)-1) ||
                            UnitPrefix.GetLength() + UnitText.GetLength() > pUnit->Description.GetLength())
                        {
                            pUnit->Description = UnitPrefix;
                            pUnit->Description << UnitText;
                            pUnit->Name = CurName;
                            pUnit->Name << S1;
                        }
                        if (m_pCurStruct)
                        {
                            SetUnitProperty(pUnit, PRP_STRUCT_ID,   eLong,    (void*)m_pCurStruct->Id,      eBoth);
                            SetUnitProperty(pUnit, PRP_STRUCT_NAME, eCharPtr, m_pCurStruct->Name.GetData(), eBoth);
                            if (0==m_pCurStruct->OwnerUnitId)
                            {
                                m_pCurStruct->OwnerUnitId = pUnit->Id;
                                SetUnitProperty(pUnit, PRP_STRUCT_OWNER, eCharPtr, YES, eBoth);
                            }
                        }
                    }
                    else 
                    {
                        if (0 == pUnit->FactionId)
                        {
                            // it is Faction name
                            pFaction = GetFaction(n1);
                            if (!pFaction)
                            {
                                pFaction       = new CFaction;
                                pFaction->Name << CurName << S1;
                                pFaction->Id   = n1;
                                m_Factions.Insert(pFaction);
                            }
                            pUnit->FactionId= pFaction->Id;
                            pUnit->pFaction = pFaction;
                        }
//                        else
//                            pFaction = pUnit->pFaction;
                        pUnit->IsOurs = pUnit->IsOurs ||
                                        (m_CrntFactionId > 0 && pUnit->FactionId == m_CrntFactionId);
                    }
//                        else
//                        {
//                            // there must be nothing else!
//                            Buf.Format("Unit description - unexpected \"Name (number)\" combination at line %d\n", m_nCurLine);
//                            LOG_ERR(ERR_DESIGN, Buf.GetData());
//                        }
                }
                CurName.Empty();
                break;


            case '[':
                p = N1.GetInteger(Line.GetData());
                if (N1.IsEmpty())
                    n1 = 1;
                else
                    n1 = atol(N1.GetData());

                p = S1.GetToken(p, '[', TRIM_ALL);
                p = S2.GetToken(p, ']', TRIM_ALL);

                if (!p || S2.IsEmpty() || !pUnit)
                {
                    // there is no shortname
                    Buf.Format("Unit description - flag/item error at line %d\n", m_nCurLine);
                    LOG_ERR(ERR_DESIGN, Buf.GetData());
                }
                else
                    SetUnitProperty(pUnit, S2.GetData(), eLong, (void*)n1, eBoth);

                CurName.Empty();
                break;

            default: // a dot in the name
                if (','!=Delimiter)
                    CurName << Buf << Delimiter;
                else
                    CurName.Empty();
            }
        }
        
        
        
        else if (0==stricmp(SECT_SKILLS, Section.GetData()))
        {
            SkillsFound = TRUE;

            // recognize patterns:
            // S1 [S2] N1 (N2)
            p = S1.GetToken(Line.GetData(), '[', TRIM_ALL);
            if (p)
                p = S2.GetToken(p, ']', TRIM_ALL);
            if (p)
                p = N1.GetToken(p, '(', TRIM_ALL);
            if (p)
                p = N2.GetToken(p, ')', TRIM_ALL);

            if (0!=stricmp("none", S1.GetData()))
                if (!p || !pUnit)
                {
                    Buf.Format("Unit description - skills error at line %d\n", m_nCurLine);
                    LOG_ERR(ERR_DESIGN, Buf.GetData());
                }
                else
                {
                    Buf = S2;
                    Buf << PRP_SKILLS_POSTFIX; // That is a skill!
                    SetUnitProperty(pUnit, Buf.GetData(), eLong, (void*)atol(N1.GetData()), eBoth);

                    Buf = S2;
                    Buf << PRP_SKILL_DAYS_POSTFIX;
                    SetUnitProperty(pUnit, Buf.GetData(), eLong, (void*)atol(N2.GetData()), eBoth);
                }

        }

        LastDelimiter = Delimiter;
    }


    return err;
}



//----------------------------------------------------------------------


int CAtlaParser::ParseStructure(CStr & FirstLine)
{
    //+ Ruin [1] : Ruin, closed to player units.
    //+ Ship [100] : Longboat, needs 10.
    //+ Forager [101] : Longboat.
    //+ Shaft [1] : Shaft, contains an inner location.
    
    // Description is terminated by a unit or an empty line

    CStr         S;
    CStr         Name;
    const char * p;
    long         id;
    CStruct    * pStruct;
    char         ch;
    CStr         CurLine(64);
    CStr         TmpDescr(64);
    CStr         Kind;

    if (!m_pCurLand)
        goto Exit;

    p  = Name.GetToken(FirstLine.GetData(), '[');
    p  = S.GetToken(p, ']');
    id = atol(S.GetData());
    if (0==id)
        goto Exit;
    p  = S.GetToken(p, ':');
    p  = Kind.GetToken(p, ",.;", ch);

    Name << " [" << id << "]";
    
    TmpDescr = FirstLine;
    TmpDescr.TrimRight(TRIM_ALL);
    while (ReadNextLine(CurLine))
    {
        const char * s = SkipSpaces(CurLine.GetData());
        if (!s || !*s)
            break;
            
        if (strchr(STRUCT_UNIT_START, *s))
        {
            PutLineBack(CurLine);
            break;
        }

        CurLine.TrimRight(TRIM_ALL);
        TmpDescr << EOL_SCR << CurLine;
    }

    pStruct              = new CStruct;
    pStruct->Id          = id; 
    pStruct->Name        = &Name.GetData()[sizeof(HDR_STRUCTURE)-1];
    pStruct->Description = TmpDescr;
    pStruct->Kind        = Kind;
    pStruct->Attr        = GetStructAttrLoc(pStruct->Kind.GetData());
    if (pStruct->Attr & (SA_ROAD_N | SA_ROAD_NE | SA_ROAD_SE | SA_ROAD_S | SA_ROAD_SW | SA_ROAD_NW ))
        if (pStruct->Description.FindSubStr("needs") > 0 || pStruct->Description.FindSubStr("decay") > 0 )
            pStruct->Attr |= SA_ROAD_BAD;
        
    m_pCurStruct         = m_pCurLand->AddNewStruct(pStruct);

    
Exit:
    return ERR_OK;
}

//----------------------------------------------------------------------

void CAtlaParser::SetShaftLinks()
{
    CLand      * pLand;
    CLand      * pLandDest;
    CStruct    * pStruct;
    int          i,j,n;
    EValueType   type;
    const void * value;
    CStr         S, T;
    const char * p;
    
    for (i=0; i<m_LandsToBeLinked.Count(); i++)
    {
        pLand = (CLand*)m_LandsToBeLinked.At(i);
        
        if (!pLand->GetProperty(PRP_LAND_LINK, type, value, eOriginal) || eLong!=type)
            continue;
            
        for (j=0; j<pLand->Structs.Count(); j++)
        {
            pStruct = (CStruct*)pLand->Structs.At(j);
//            if (0==stricmp(pStruct->Kind.GetData(), "Shaft") )
            if (pStruct->Attr & SA_SHAFT  ) 
            {
                pLandDest = GetLand((long)value);
                if (pLandDest && pStruct->Description.FindSubStr("links")<0)
                {
                    ComposeLandStrCoord(pLandDest, S);
                    if ('.' ==  pStruct->Description.GetData()[ pStruct->Description.GetLength()-1])
                         pStruct->Description.DelCh( pStruct->Description.GetLength()-1);
                        
                    p = strrchr( pStruct->Description.GetData(), '\n');    
                    n = p ? ( pStruct->Description.GetData()-p) : 0;
        
                    T.Empty();
                    T << "; links to (" << S << ").";  
                    if (n+T.GetLength() > 64)
                         pStruct->Description << EOL_SCR << "  ";
                     pStruct->Description << T;
                }
                break;
            }
        }
    }    
    
    m_LandsToBeLinked.DeleteAll();    
}

//----------------------------------------------------------------------

void CAtlaParser::StoreBattle(CStr & s)
{
    CStr          S1, S2, S3;
    CStr          N1, N2, N3;
    CBattle     * pBattle;
    const char  * p;
    int           i;
    BOOL          Ok = FALSE;

    s.TrimRight(TRIM_ALL);
    if (s.IsEmpty())
        return;

    // Captain (15166) is assassinated in forest (83,129) in Imyld!
    p = s.GetData();
    p = SkipSpaces(N1.GetToken(p, '(', TRIM_ALL));
    p = SkipSpaces(N1.GetToken(p, ')', TRIM_ALL));
    p = SkipSpaces(S1.GetToken(p, ' ', TRIM_ALL));
    p = SkipSpaces(S2.GetToken(p, ' ', TRIM_ALL));

    if ( IsInteger(N1.GetData()) &&
         0 == stricmp("is"          , S1.GetData()) &&
         0 == stricmp("assassinated", S2.GetData()) 
       )
    {
        p  = S3.GetToken(p, '(', TRIM_ALL);
        p  = N3.GetToken(p, ',', TRIM_ALL);
        Ok = TRUE;
    }
    else
    {
        //Xbowmen (591) attacks City Guard (24) in swamp (13,33) in Salen!
        p  = s.GetData();
        p  = S1.GetToken(p, '(', TRIM_ALL);
        p  = N1.GetToken(p, ')', TRIM_ALL);
        p  = S2.GetToken(p, '(', TRIM_ALL);
        p  = N2.GetToken(p, ')', TRIM_ALL);
        p  = S3.GetToken(p, '(', TRIM_ALL);
        p  = N3.GetToken(p, ',', TRIM_ALL);
        Ok = (  IsInteger(N1.GetData()) &&
                IsInteger(N2.GetData()) &&
                IsInteger(N3.GetData()) &&
                (0==strnicmp(S3.GetData(), "in", 2)) );
    }

    if (Ok)
    {
        p = S3.GetToken(p, ')', TRIM_ALL);
        N3 << "," << S3;

        if (!m_BattleLandStrs.Search((void*)N3.GetData(), i))
            m_BattleLandStrs.Insert(strdup(N3.GetData()));

        pBattle              = new CBattle;
        pBattle->LandStrId   = N3;
        pBattle->Description = s;
        pBattle->Name.SetStr(s.GetData(), p-s.GetData());
        if (!m_Battles.Insert(pBattle))
            delete pBattle;
    }
}

//----------------------------------------------------------------------

// It suxx to build parsing on empty strings, but I can not see any clear sign of a battle end

int CAtlaParser::ParseBattles()
{
    CStr         CurLine(64);
    CStr         Battle(128);
    CStr         Block1(64), Block2(64);
    const char * p;
    int          i;
    CStr         S1, S2, N1;

    while (ReadNextLine(CurLine))
    {
        CurLine.TrimRight(TRIM_ALL);

        if (CurLine.IsEmpty())
        {
            // Captain (15166) is assassinated in forest (83,129) in Imyld!
            p = SkipSpaces(Block1.GetData());
            p = SkipSpaces(N1.GetToken(p, '(', TRIM_ALL));
            p = SkipSpaces(N1.GetToken(p, ')', TRIM_ALL));
            p = SkipSpaces(S1.GetToken(p, ' ', TRIM_ALL));
            p = SkipSpaces(S2.GetToken(p, ' ', TRIM_ALL));


            if (IsInteger(N1.GetData()) &&
                0 == stricmp("is"          , S1.GetData()) &&
                0 == stricmp("assassinated", S2.GetData())
                )
            {
                // It is an assassination
                Battle << Block2 << EOL_SCR;
                StoreBattle(Battle);
                Battle.Empty();
                Block2 = Block1;
                Block1.Empty();
            }
            else
            {
                Battle << Block2 << EOL_SCR;
                Block2 = Block1;
                Block1.Empty();
            }
        }
        else
        {
            if (0==strnicmp(CurLine.GetData(), HDR_ATTACKERS, sizeof(HDR_ATTACKERS)-1))
            {
                // Ho-ho! New battle starting!  Finish the old one first.
                StoreBattle(Battle);
                Battle.Empty();
                Battle << Block2 << EOL_SCR;
                Block2 = Block1;
                Block1.Empty();
            }
            else
            {
                // Is the battle list over yet?
                p = SkipSpaces(CurLine.GetData());
                for (i=0; i<(int)sizeof(BattleEndHeader)/(int)sizeof(const char *); i++)
                    if (0==strnicmp(p, BattleEndHeader[i], BattleEndHeaderLen[i] ))
                    {
                        CurLine << EOL_FILE;
                        PutLineBack(CurLine);
                        Battle << Block2 << Block1 << EOL_SCR;
                        StoreBattle(Battle);
                        Battle.Empty();
                        return 0;
                    }
            }
            Block1 << CurLine << EOL_SCR;
        }
    }

    return 0;
}


//----------------------------------------------------------------------

//Skill reports:
//
//armorer [ARMO] 5: A unit with this skill may PRODUCE mithril armor
//  from one unit of mithril. Mithril armor provides a 9/10 chance of
//  surviving a successful attack in battle from a normal weapon and a
//  2/3 chance of surviving an attack from a good weapon.  Mithril armor
//  weighs one unit. Production of mithril armor can be increased by
//  using hammers.
//
//entertainment [ENTE] 1: A unit with this skill may use the ENTERTAIN
//  order to generate funds. The amount of silver gained will be 20 per
//  man, times the level of the entertainers. This amount is limited by
//  the region that the unit is in.

int CAtlaParser::ParseSkills()
{
    CStr             CurLine(64);

    CStr             OneSkill(128);
    CStr             S;
    const char     * p;
    CShortNamedObj * pSkill;

    while (ReadNextLine(CurLine))
    {
        CurLine.TrimRight(TRIM_ALL);
        if (CurLine.IsEmpty())
        {
            if (!OneSkill.IsEmpty())
            {
                pSkill = new CShortNamedObj;

                p = OneSkill.GetData();
                p = S.GetToken(p, '[');     pSkill->Name        = S;
                p = S.GetToken(p, ']');     pSkill->ShortName   = S;
                p = S.GetToken(p, ':');     pSkill->Level       = atol(S.GetData());

                pSkill->Description = OneSkill;

                pSkill->Name.GetToken(OneSkill.GetData(), ':');

                m_Skills.Insert(pSkill);
                OneSkill.Empty();
            }
        }
        else
        {
            // check for invalid format in the first line
            if (OneSkill.IsEmpty())
            {
                p = S.GetToken(CurLine.GetData(), '[');
                if (!p)
                    break;
                p = S.GetToken(p, ']');
                if (!p)
                    break;
                p = S.GetToken(p, ':');
                if (!p)
                    break;
                if (atol(S.GetData()) <= 0)
                    break;
            }

            OneSkill << CurLine << EOL_SCR;
        }
    }
    CurLine << EOL_FILE;
    PutLineBack(CurLine);

    return 0;
}


//----------------------------------------------------------------------

//ironwood [IRWD], weight 10. Units with lumberjack [LUMB] 3 may PRODUCE
//  this item at a rate of 1 per man-month.
//
//mithril [MITH], weight 10. Units with mining [MINI] 3 may PRODUCE this
//  item at a rate of 1 per man-month.
//

int CAtlaParser::ParseItems()
{
    CStr             CurLine(64);
    CStr             OneItem(128);
    CStr             S;
    const char     * p;
    CShortNamedObj * pItem;

    while (ReadNextLine(CurLine))
    {
        CurLine.TrimRight(TRIM_ALL);
        if (CurLine.IsEmpty())
        {
            if (!OneItem.IsEmpty())
            {
                pItem = new CShortNamedObj;

                p = OneItem.GetData();
                p = S.GetToken(p, '[');     pItem->Name        = S;
                p = S.GetToken(p, ']');     pItem->ShortName   = S;

                pItem->Description = OneItem;
                pItem->Name.GetToken(OneItem.GetData(), ',');

                m_Items.Insert(pItem);
                OneItem.Empty();
            }
        }
        else
        {
            // check for invalid format in the first line
            if (OneItem.IsEmpty())
            {
                p = S.GetToken(CurLine.GetData(), '[');
                if (!p)
                    break;
                p = S.GetToken(p, ']');
                if (!p)
                    break;
                p = S.GetToken(p, ',');
                if (!p)
                    break;
                if (!S.IsEmpty())
                    break;
            }

            OneItem << CurLine << EOL_SCR;
        }
    }
    CurLine << EOL_FILE;
    PutLineBack(CurLine);

    return 0;
}

//----------------------------------------------------------------------

//Object reports:
//
//Longboat: This is a ship. Units may enter this structure. This ship
//  requires 5 total levels of sailing skill to sail. This structure is
//  built using shipbuilding [SHIP] 1 and requires 25 wood to build.
//
//Clipper: This is a ship. Units may enter this structure. This ship
//  requires 10 total levels of sailing skill to sail. This structure is
//  built using shipbuilding [SHIP] 1 and requires 50 wood to build.


int CAtlaParser::ParseObjects()
{
    CStr             CurLine(64);
    CStr             OneItem(128);
    CStr             S;
    const char     * p;
    CShortNamedObj * pItem;
    char             ch;

    while (ReadNextLine(CurLine))
    {
        CurLine.TrimRight(TRIM_ALL);
        if (CurLine.IsEmpty())
        {
            if (!OneItem.IsEmpty())
            {
                pItem = new CShortNamedObj;

                p = OneItem.GetData();
                p = S.GetToken(p, ':');     
                pItem->Name        = S;
                pItem->ShortName   = S;
                pItem->Description = OneItem;

                m_Objects.Insert(pItem);
                OneItem.Empty();
            }
        }
        else
        {
            // check for invalid format in the first line
            if (OneItem.IsEmpty())
            {
                p = SkipSpaces(S.GetToken(CurLine.GetData(), ':'));
                if (!p)
                    break;
                p = SkipSpaces(S.GetToken(p, " \t", ch));
                if (0!=stricmp(S.GetData(), "This"))
                    break;
                p = SkipSpaces(S.GetToken(p, " \t", ch));
                if (0!=stricmp(S.GetData(), "is"))
                    break;
            }

            OneItem << CurLine << EOL_SCR;
        }
    }
    CurLine << EOL_FILE;
    PutLineBack(CurLine);

    return 0;
}


//----------------------------------------------------------------------

int CAtlaParser::ParseLines()
{
    int        err = ERR_OK;
    CStr       CurLine(64);
    const char * p;

    while ((ERR_OK==err) && ReadNextLine(CurLine))
    {
        // recognize header line and then call specific handler

        p = SkipSpaces(CurLine.GetData());

        if      (0==strnicmp(p, HDR_FACTION           , sizeof(HDR_FACTION)-1 ))
            err = ParseFactionInfo(TRUE);


        else if (0==strnicmp(p, HDR_FACTION_STATUS    , sizeof(HDR_FACTION_STATUS)-1 ))
            err = ParseFactionInfo(FALSE);

        else if (0==strnicmp(p, HDR_EVENTS            , sizeof(HDR_EVENTS)-1 ))
            err = ParseEvents();

        else if (0==strnicmp(p, HDR_ERRORS            , sizeof(HDR_ERRORS)-1 ))
            err = ParseErrors();

        else if (0==strnicmp(p, HDR_SILVER            , sizeof(HDR_SILVER)-1 ))
            err = ParseUnclSilver(CurLine);

        else if (0==strnicmp(p, HDR_ATTITUDES         , sizeof(HDR_ATTITUDES)-1 ))
            err = ParseAttitudes(CurLine);

        else if (0==strnicmp(p, HDR_UNIT_OWN          , sizeof(HDR_UNIT_OWN)-1 ))
            err = ParseUnit(CurLine);

        else if (0==strnicmp(p, HDR_UNIT_ALIEN        , sizeof(HDR_UNIT_ALIEN)-1 ))
            err = ParseUnit(CurLine);

        else if (0==strnicmp(p, HDR_STRUCTURE         , sizeof(HDR_STRUCTURE)-1 ))
            err = ParseStructure(CurLine);

        else if (0==strnicmp(p, HDR_BATTLES           , sizeof(HDR_BATTLES)-1 ))
            err = ParseBattles();

        else if (0==strnicmp(p, HDR_SKILLS            , sizeof(HDR_SKILLS)-1 ))
            err = ParseSkills();

        else if (0==strnicmp(p, HDR_ITEMS             , sizeof(HDR_ITEMS)-1 ))
            err = ParseItems();

        else if (0==strnicmp(p, HDR_OBJECTS           , sizeof(HDR_OBJECTS)-1 ))
            err = ParseObjects();

        else if (0==strnicmp(p, HDR_ORDER_TEMPLATE    , sizeof(HDR_ORDER_TEMPLATE)-1 ))
            LoadOrders(*m_pSource, m_CrntFactionId, FALSE); // That's a non-return point.
                                                            // Hope there is nothing else after the order template.



        else

            err = ParseTerrain(NULL, 0, CurLine, TRUE, NULL);

    }

    return err;
}

//----------------------------------------------------------------------

int CAtlaParser::ParseRep(const char * FNameIn, BOOL Join, BOOL IsHistory)
{
    m_pCurLand   = NULL;
    m_pCurStruct = NULL;
    m_ParseErr   = ERR_NOTHING;
    m_pSource    = new CFileReader;
    m_JoiningRep = Join;
    m_IsHistory  = IsHistory;
    m_CrntFactionId = 0;
    m_CrntFactionPwd.Empty();
    m_IsHistory  = IsHistory;

    if (Join)
        m_FactionInfo << EOL_SCR << EOL_SCR << "-----------------------------------------------" 
                      << EOL_SCR << EOL_SCR;
    
    if (!Join)
        m_YearMon = 0;
    m_CurYearMon  = 0;
        
    if (!m_pSource->Open(FNameIn))
    {
        m_ParseErr = ERR_FOPEN;
        goto Done;
    }

    m_MaxSkillDays = atol(GetConfStringLoc(SZ_SECT_COMMON, SZ_KEY_MAX_SKILL_DAYS));

    m_ParseErr = ParseLines();
    ApplyLandFlags();
    SetExitFlags();
    SetShaftLinks();

Done:
    m_pSource->Close();
    delete m_pSource;
    m_pSource = NULL;
    return m_ParseErr;
}


//----------------------------------------------------------------------

void CAtlaParser::SetExitFlags()
{

    CLand       * pLand;
    CLand       * pLandSrc;
    int           nl, np, i;
    int           x, y, z;
    int           x0, y0;
    CPlane      * pPlane;
    BOOL          SetAllExits; // if reading 1.0.0 history file

    for (np=0; np<m_Planes.Count(); np++)
    {
        pPlane = (CPlane*)m_Planes.At(np);
        SetAllExits = ( (pPlane->ExitsCount<2) && (pPlane->Lands.Count()>2) );
        for (nl=0; nl<pPlane->Lands.Count(); nl++)
        {
            pLand = (CLand*)pPlane->Lands.At(nl);
            if (SetAllExits)
                pLand->ExitBits = 0xFF;
            else
                if ( (0==(pLand->Flags&LAND_VISITED)) || (pLand->Flags&LAND_SET_EXITS)  )
                {
                    pLand->ExitBits = 0;
                    LandIdToCoord(pLand->Id, x0, y0, z);
                    for (i=0; i<6; i++)
                    {
                        x = x0;
                        y = y0;

                        switch (i)
                        {
                        case North     : y -= 2;     break;
                        case Northeast : y--; x++;   break;
                        case Southeast : y++; x++;   break;
                        case South     : y += 2;     break;
                        case Southwest : y++; x--;   break;
                        case Northwest : y--; x--;   break;
                        }

                        pLandSrc = GetLand(x, y, np, TRUE);
                        if ( pLandSrc && (pLandSrc->Flags&LAND_VISITED) )
                        {
                            if (pLandSrc->ExitBits&EntryFlags[i])
                                pLand->ExitBits |= ExitFlags[i];
                        }
                        else
                            pLand->ExitBits |= ExitFlags[i]; // assume there is an exit if nothing is known
                    }
                }
        }
    }
}

//----------------------------------------------------------------------


void AddTabbed(CStr & Dest, const char * Src, int Offs)
{
#define SPC "  "
    int  i;
    int  n;
    CStr Spc;

    for (i=0; i<Offs; i++)
        Spc << SPC;
    n = Spc.GetLength();
    Dest << Spc;

    while (Src && *Src)
    {
        if ( (n>=64) && (' '==*Src) )
        {
            Dest << EOL_SCR << Spc << SPC;
            n = Spc.GetLength() + sizeof(SPC) - 1;
        }
        Dest << *Src;
        Src++;
        n++;
    }
    Dest << EOL_SCR;
}

//----------------------------------------------------------------------

/*
[Hex 0,2,Overworld]
Turn=31
Terrain=Plain
Confirmed=True
Exits=15
Province=Ellon
Peasants=PLAI,658
Tax Base=2632
Wages=14
Max Wages=1842
Entertainment=131
For Sale=PLAI,131,56;LEAD,26,112
For Sale=IRON,-1,75;WOOD,-1,75;STON,-1,75;FUR,-1,75;HERB,-1,75;HORS,-1,75;SWOR,-1,150;XBOW,-1,150;LBOW,-1,150;CARM,-1,150;PARM,-1,625;WAGO,-1,250;TELF,-1,64;LEAD,-1,128
Wanted=GRAI,112,18;LIVE,154,27;CARM,30,93;WAGO,31,192;VODK,10,146;DYE,9,174
Products=LIVE,76;HORS,26
Gate=5


City=Tolsta
Town=Dryck
Village=Brodick

*/

int CAtlaParser::ParseCBHex(const char * FirstLine)
{
    CStr                 CurLine(128);
    CStr                 S(128);
    CStr                 Name;
    const char         * Value, * p;
    CLand              * pLand = NULL;
    CBaseObject          Dummy;
    CPlane             * pPlane;
    CStruct            * pStruct;
    int                  i;
    long                 n;
    const char         * item;
    BOOL                 NewLand;


// Hex data
    long         X  = 0;
    long         Y  = 0;
    CStr         Plane;
    CStr         Terrain;
    BOOL         Confirmed = FALSE;
    CStr         Province;
    CStr         Peasants;
    long         nPeasants = 0;
    long         TaxBase   = 0;
    long         Wages     = 0;
    long         MaxWages  = 0;
    long         Entertainment = 0;
    CLongColl    SaleNo;
    CLongColl    SalePrice;
    CBufColl     SaleItem;
    CLongColl    WantedNo;
    CLongColl    WantedPrice;
    CBufColl     WantedItem;
    CLongColl    ProduceNo;
    CBufColl     ProduceItem;
    long         Gate      = 0;
    //int          ExitBits  = 0xFF;
    CStr         CityName;
    CStr         CityType;



    FirstLine =  SkipSpaces(S.GetToken(FirstLine, ','));        X = atol(S.GetData());
    FirstLine =  SkipSpaces(S.GetToken(FirstLine, ','));        Y = atol(S.GetData());
    FirstLine =  Plane.GetToken(FirstLine, ']');


    while (ReadNextLine(CurLine))
    {

        CurLine.TrimRight(TRIM_ALL);
        p = SkipSpaces(CurLine.GetData());

        if ('['==*p)
        {
            CurLine << EOL_SCR;
            PutLineBack(CurLine);
            break;
        }

        Value = SkipSpaces(Name.GetToken(CurLine.GetData(), '='));

        if      (0==stricmp(Name.GetData(),  "Terrain"   ))
            Terrain = Value;

        else if (0==stricmp(Name.GetData(),  "Confirmed"   ))
            Confirmed = (0==strnicmp(Value, "TRUE", 4));

        else if (0==stricmp(Name.GetData(),  "Province"   ))
            Province = Value;

        else if (0==stricmp(Name.GetData(),  "Peasants"   ))
            nPeasants = atol(Peasants.GetToken(Value, ','));

        else if (0==stricmp(Name.GetData(),  "Tax Base"   ))
            TaxBase = atol(Value);

        //else if (0==stricmp(Name.GetData(),  "Exits"   ))
        //    ExitBits = atol(Value);

        else if (0==stricmp(Name.GetData(),  "Wages"   ))

            Wages = atol(Value);

        else if (0==stricmp(Name.GetData(),  "Max Wages"   ))
            MaxWages = atol(Value);

        else if (0==stricmp(Name.GetData(),  "Entertainment"   ))
            Entertainment =  atol(Value);

        else if (0==stricmp(Name.GetData(),  "For Sale"   ))
            while(Value)
            {
                Value = S.GetToken(Value, ',');     SaleItem .Insert(strdup(S.GetData()));
                Value = S.GetToken(Value, ',');     SaleNo   .Insert((void*)atol(S.GetData()));
                Value = S.GetToken(Value, ';');     SalePrice.Insert((void*)atol(S.GetData()));
            }

        else if (0==stricmp(Name.GetData(),  "Wanted"   ))
            while(Value)
            {
                Value = S.GetToken(Value, ',');     WantedItem .Insert(strdup(S.GetData()));
                Value = S.GetToken(Value, ',');     WantedNo   .Insert((void*)atol(S.GetData()));
                Value = S.GetToken(Value, ';');     WantedPrice.Insert((void*)atol(S.GetData()));
            }

        else if (0==stricmp(Name.GetData(),  "Products"   ))
            while(Value)
            {
                Value = S.GetToken(Value, ',');     ProduceItem .Insert(strdup(S.GetData()));
                Value = S.GetToken(Value, ';');     ProduceNo   .Insert((void*)atol(S.GetData()));
            }

        else if (0==stricmp(Name.GetData(),  "Gate"   ))
            Gate = atol(Value);

        else if ( (0==stricmp(Name.GetData(),  "Town"   )) ||
                  (0==stricmp(Name.GetData(),  "City"   )) ||
                  (0==stricmp(Name.GetData(),  "Village")) )
        {
            CityType = Name;
            CityName = Value;
        }
    }


    pPlane = MakePlane(Plane.GetData());


    Dummy.Id = LandCoordToId(X, Y, pPlane->Id);
    if (pPlane->Lands.Search(&Dummy,i))
    {
        pLand = (CLand *)pPlane->Lands.At(i);
        NewLand = FALSE;
    }
    else
    {
        pLand               = new CLand;
        pLand->Id           = LandCoordToId(X, Y, pPlane->Id);
        pLand->TerrainType  = Terrain;
        pLand->pPlane       = pPlane;

        pPlane->Lands.Insert(pLand);
        NewLand = TRUE;
    }
                                                            

    // Add gate
    if (Gate>0)
    {
        pStruct     = new CStruct;
        pStruct->Id = -Gate;  // negative, so it do not clash with structures!
        pStruct->Description << "There is a Gate here (Gate " << Gate << " of " << m_GatesCount << ")";
        pStruct->Kind        = STRUCT_GATE;
        
        pStruct->Attr        = GetStructAttrLoc(pStruct->Kind.GetData());
        pLand->AddNewStruct(pStruct);
    }


    if ( NewLand || (Confirmed && (0==(pLand->Flags&LAND_VISITED)) ) )
    {
        pLand->Description.Empty();
        pLand->Taxable      = TaxBase;
        pLand->Peasants     = nPeasants;
        pLand->CityName     = CityName;
        pLand->CityType     = CityType;
        pLand->Flags       |= LAND_SET_EXITS;
        pLand->ExitBits     = 0xFF;
        if (Confirmed)
            pLand->Flags   |= LAND_VISITED;
        if (Gate>0)
            pLand->Flags   |= LAND_STR_GATE;
        //if (ExitBits!=0xFF)
        //    pLand->ExitBits = ExitBits;

/*
desert (7,23) in Fanders, contains Corran [village], 857 peasants
  (nomads), $2571.
------------------------------------------------------------
  It was monsoon season last month; it will be clear next month.
  Wages: $13 (Max: $2228).
  Wanted: 108 grain [GRAI] at $20, 134 livestock [LIVE] at $21, 78
    fish [FISH] at $20.
  For Sale: 171 nomads [NOMA] at $52, 34 leaders [LEAD] at $104.
  Entertainment available: $128.
  Products: 15 livestock [LIVE], 16 iron [IRON], 13 stone [STON].
        */

//plain (39,7) in Ellon, contains Zapulla [city].

//void AddTabbed(CStr & Dest, const char * Src, int Offs)

        S.Empty();
        S << Terrain << " (" << X << "," << Y;
        if (0!=stricmp(DEFAULT_PLANE, Plane.GetData()))
            S << "," << Plane;
        S << ") in " << Province;
        if (!CityName.IsEmpty())
            S << ", contains " << CityName << " [" << CityType << "]";
        if (!Confirmed)
        {
            S << ".";
            AddTabbed(pLand->Description, S.GetData(), 0);
        }
        else
        {
            S << ", " << nPeasants << " peasants (" << Peasants << ")";
            S << ", $" << TaxBase << ".";
            AddTabbed(pLand->Description, S.GetData(), 0);
            AddTabbed(pLand->Description, "------------------------------------------------------------", 0);

            // weather statement should go here

            S.Empty();
            S << "Wages: $" << Wages << " (Max: $" << MaxWages << ").";
            AddTabbed(pLand->Description, S.GetData(), 1);

            S.Empty();
            S << "Wanted: ";
            for (i=0; i<WantedItem.Count(); i++)
            {
                if (i>0)
                    S << ", ";
                item = (const char *)WantedItem.At(i);
                S << (long)WantedNo.At(i) << " " << item << " [" << item << "] at $" << (long)WantedPrice.At(i);
            }
            if (0==WantedItem.Count())
                S << "none";
            S << ".";
            AddTabbed(pLand->Description, S.GetData(), 1);


            S.Empty();
            S << "For Sale: ";
            for (i=0; i<SaleItem.Count(); i++)
            {
                if (i>0)
                    S << ", ";
                n    = (long)SaleNo.At(i);
                item = (const char *)SaleItem.At(i);
                if (n>=0)
                    S << n;
                else
                    S << "unlimited";
                S << " " << item << " [" << item << "] at $" << (long)SalePrice.At(i);
            }
            if (0==SaleItem.Count())
                S << "none";
            S << ".";
            AddTabbed(pLand->Description, S.GetData(), 1);


            if (Entertainment>0)
            {
                S.Empty();
                S << "Entertainment available: $" << Entertainment << ".";
                AddTabbed(pLand->Description, S.GetData(), 1);
            }

            S.Empty();
            S << "Products: ";
            for (i=0; i<ProduceItem.Count(); i++)
            {
                if (i>0)
                    S << ", ";
                item = (const char *)ProduceItem.At(i);
                S << (long)ProduceNo.At(i) << " " << item << " [" << item << "]";
            }
            if (0==ProduceItem.Count())
                S << "none";
            S << ".";
            AddTabbed(pLand->Description, S.GetData(), 1);
        }

        pLand->Description.TrimRight(TRIM_ALL);
    }


    SaleItem.FreeAll();
    WantedItem.FreeAll();
    ProduceItem.FreeAll();

    return ERR_OK;
}

//----------------------------------------------------------------------

/*

[Structure 1: 0,6,Overworld]
Turn=31
Name=Building
Kind=Tower
Needs=0


[Structure 1: 0,8,Overworld]
Turn=31
Name=Building
Kind=Timber Yard
Needs=0

[Structure 2: 0,8,Overworld]
Turn=31
Name=Building
Kind=Timber Yard
Needs=0

[Structure 120: 32,30,Overworld]
Turn=31
Name=Ship
Kind=Clipper
Needs=41


*/

int CAtlaParser::ParseCBStruct(const char * FirstLine)
{
    CStr                 CurLine(128);
    CStr                 S(128);
    CStr                 Name;
    const char         * Value, * p;
    CLand              * pLand = NULL;
    CBaseObject          Dummy;
    CPlane             * pPlane;
    CStruct            * pStruct;
    int                  i;


// Hex data
    long         No = 0;
    long         X  = 0;
    long         Y  = 0;
    CStr         Plane;
    CStr         StrName;
    CStr         Kind;
    long         Needs = 0;


    FirstLine =  SkipSpaces(S.GetToken(FirstLine, ':'));        No= atol(S.GetData());
    FirstLine =  SkipSpaces(S.GetToken(FirstLine, ','));        X = atol(S.GetData());
    FirstLine =  SkipSpaces(S.GetToken(FirstLine, ','));        Y = atol(S.GetData());
    FirstLine =  Plane.GetToken(FirstLine, ']');


    while (ReadNextLine(CurLine))
    {
        CurLine.TrimRight(TRIM_ALL);
        p = SkipSpaces(CurLine.GetData());

        if ('['==*p)
        {
            CurLine << EOL_SCR;
            PutLineBack(CurLine);
            break;
        }

        Value = SkipSpaces(Name.GetToken(CurLine.GetData(), '='));

        if      (0==stricmp(Name.GetData(),  "Name"   ))
            StrName = Value;
        else if (0==stricmp(Name.GetData(),  "Kind"   ))

            Kind = Value;
        else if (0==stricmp(Name.GetData(),  "Needs"   ))
            Needs = atol(Value);
    }

    pPlane = MakePlane(Plane.GetData());

    Dummy.Id = LandCoordToId(X, Y, pPlane->Id);
    if (pPlane->Lands.Search(&Dummy,i))
    {
        pLand = (CLand *)pPlane->Lands.At(i);

        //+ Ship [100] : Longboat, needs 10.
        pStruct              = new CStruct;
        pStruct->Id          = No; //m_NextStructId++;
        //pStruct->No          = No;
        pStruct->Name        = StrName;
        pStruct->Kind        = Kind;
        pStruct->Description << StrName << " [" << No << "] : " << Kind;
        if (Needs>0)
            pStruct->Description << ", needs " << Needs;
        pStruct->Description << ".";

        pStruct->Attr        = GetStructAttrLoc(pStruct->Kind.GetData());
        pLand->AddNewStruct(pStruct);
    }

    return ERR_OK;
}


//----------------------------------------------------------------------

int CAtlaParser::ParseCBDataFile(const char * FNameIn)
{
    int          err = ERR_OK;
    CStr         CurLine(128);

    CStr         Cmd;
    const char * p;

    m_pSource = new CFileReader;
    if (!m_pSource->Open(FNameIn))
    {
        err = ERR_FOPEN;
        goto Done;
    }

    while ((ERR_OK==err) && ReadNextLine(CurLine))
    {
        p = SkipSpaces(CurLine.GetData());

        if ('['==*p)
        {
            // [Hex 0,2,Overworld]
            // [Structure 2: 0,8,Overworld]
            p = SkipSpaces(Cmd.GetToken(SkipSpaces(p+1), ' '));


            if (0==stricmp(Cmd.GetData(), "Hex"))
                err = ParseCBHex(p);

            if (0==stricmp(Cmd.GetData(), "Structure"))
                err = ParseCBStruct(p);
        }
    }
    SetExitFlags();

Done:
    m_pSource->Close();
    delete m_pSource;
    m_pSource = NULL;

    return err;
}

//----------------------------------------------------------------------

void CAtlaParser::GetUnitList(CCollection * pResultColl, int x, int y, int z)
{

    CLand * pLand;
    int     i;

    pLand = GetLand(x, y, z);
    if (pLand)
        for (i=0; i<pLand->Units.Count(); i++)
            pResultColl->Insert(pLand->Units.At(i));
}

//-------------------------------------------------------------

CFaction * CAtlaParser::GetFaction(int id)
{
    CBaseObject  DummyFaction;
    int          idx;

    DummyFaction.Id = id;
    if (m_Factions.Search(&DummyFaction, idx))
        return (CFaction*)m_Factions.At(idx);
    else
        return NULL;    
}

//-------------------------------------------------------------

CLand * CAtlaParser::GetLand(int x, int y, int nPlane, BOOL AdjustForEdge)
{
    char     dummy[sizeof(CLand)];
    int      idx;
    CPlane * pPlane;

    pPlane = (CPlane*)m_Planes.At(nPlane);
    if (!pPlane)
        return NULL;

    if (AdjustForEdge && (pPlane->Width > 0))
    {
        while (x>pPlane->EastEdge)
            x-=pPlane->Width;
        while (x<pPlane->WestEdge)
            x+=pPlane->Width;
    }

    ((CLand*)&dummy)->Id = LandCoordToId(x,y, pPlane->Id);

    if (pPlane->Lands.Search(&dummy, idx))
        return (CLand*)pPlane->Lands.At(idx);
    else
        return NULL;
}

//-------------------------------------------------------------

CLand * CAtlaParser::GetLand(long LandId)
{
    int x, y, z;

    LandIdToCoord(LandId, x, y, z);
    return GetLand(x, y, z, FALSE);
}


//-------------------------------------------------------------

void CAtlaParser::ComposeLandStrCoord(CLand * pLand, CStr & LandStr)
{
    int      x, y, z;
    CPlane * pPlane;

    LandStr.Empty();
    if (!pLand)
        return;

    LandIdToCoord(pLand->Id, x, y, z);
    pPlane = (CPlane*)m_Planes.At(z);
    if (!pPlane)
        return;

    LandStr << (long)x << "," << (long)y;
    if (0!=SafeCmp(pPlane->Name.GetData(), DEFAULT_PLANE))
        LandStr << "," << pPlane->Name.GetData();
}

//-------------------------------------------------------------

BOOL CAtlaParser::LandStrCoordToId(const char * landcoords, long & id)
{
    CStr                 S;
    long                 x, y;
    CPlane             * pPlane;
    CBaseObject          Dummy;
    int                  i;

    // xxx,yyy[,somewhere]
    landcoords = S.GetToken(landcoords, ',');
    if (!IsInteger(S.GetData()))
        return FALSE;
    x = atol(S.GetData());

    // yyy[,somewhere]
    landcoords = S.GetToken(landcoords, ',');
    if (!IsInteger(S.GetData()))
        return FALSE;
    y = atol(S.GetData());


    if ( (NULL==landcoords) || (0==*landcoords) )
        landcoords = DEFAULT_PLANE;

    Dummy.Name = landcoords;
    if (m_PlanesNamed.Search(&Dummy, i))
        pPlane = (CPlane*)m_PlanesNamed.At(i);
    else
        return FALSE;

    id = LandCoordToId(x, y, pPlane->Id);
    return TRUE;
}



//-------------------------------------------------------------

CLand * CAtlaParser::GetLand(const char * landcoords) //  "48,52[,somewhere]"
{
    long                 id;

    if (LandStrCoordToId(landcoords, id))
        return GetLand(id);
    else
        return NULL;
}


//----------------------------------------------------------------------

#define CHECK_REP_LEN        \
if (S.GetLength() - n > 65)  \
{                            \
    S << eol << "    ";      \
    n = S.GetLength()-4;     \
}                            \

#define APPEND_SPACE_WRAP    \
if (S.GetLength() - n > 65)  \
{                            \
    S << eol << "    ";      \
    n = S.GetLength()-4;     \
}                            \
else                         \
    S << " ";                \
    

void CAtlaParser::ComposeProductsLine(CLand * pLand, const char * eol, CStr & S)
{
    CStr       Line(64);
    int        i;

    CProduct * pProd;
    int        n = S.GetLength();


    if (pLand->Products.Count() > 0)
    {
        S << "  Products:";
        for (i=0; i<pLand->Products.Count(); i++)
        {
            pProd = (CProduct*)pLand->Products.At(i);

            if (pProd->Amount>=0)
            {
                APPEND_SPACE_WRAP S << pProd->Amount;           
                APPEND_SPACE_WRAP S << pProd->LongName;         
                APPEND_SPACE_WRAP S << "["<< pProd->ShortName << "]";
            }
            else
            {
                APPEND_SPACE_WRAP S << " none";
            }

            if (pLand->Products.Count()-1 == i)
                S << ".";
            else
            {
                S << ",";   
            }
        }

        S << eol;
    }
}

//-------------------------------------------------------------

BOOL CAtlaParser::SaveOneHex(CFileWriter & Dest, CLand * pLand, CPlane * pPlane, SAVE_HEX_OPTIONS * pOptions)
{
    CLand            * pLandExit;
    int                i;
    int                x, y, z, x0, y0;
    int                g;
    
    CStr               sLine (128);
    CStr               sExits(128);
    CStruct          * pStruct;
    const char       * p;
    BOOL               IsLinked;
    BOOL               TurnNoMarkerWritten = FALSE;
    CUnit            * pUnit;
    EValueType         type;
    const void       * value;
    
    IsLinked = FALSE;
    sExits.Empty();
    LandIdToCoord(pLand->Id, x0, y0, z);
    for (i=0; i<6; i++)
    {
        if (pLand->ExitBits&ExitFlags[i])
        {
            x = x0;
            y = y0;

            switch (i)
            {
            case North     : y -= 2;     break;
            case Northeast : y--; x++;   break;
            case Southeast : y++; x++;   break;
            case South     : y += 2;     break;
            case Southwest : y++; x--;   break;
            case Northwest : y--; x--;   break;
            }

            pLandExit = GetLand(x, y, z, TRUE);
            if ( pLandExit && (pLandExit->ExitBits&EntryFlags[i]))
            {
                if (pLandExit->Flags&LAND_VISITED)
                    IsLinked = TRUE;

//  Northeast : mountain (20,0) in Lautaro, contains Krod [city].


                LandIdToCoord(pLandExit->Id, x, y, z);
                sExits << "  " << Directions[i] << " : "
                      << pLandExit->TerrainType << " (" << (long)x << "," << (long)y;
                if (0!=stricmp(DEFAULT_PLANE, pPlane->Name.GetData()))
                    sExits << "," << pPlane->Name;
                sExits << ") in " << pLandExit->Name;
                if (!pLandExit->CityName.IsEmpty())
                    sExits << ", contains " << pLandExit->CityName << " [" << pLandExit->CityType << "]";
                sExits  << "." << EOL_FILE;;
            }
        }
    }

    if ( (0==(pLand->Flags&LAND_VISITED)) && IsLinked)
        return FALSE;

        
    p = pLand->Description.GetData();
    while (p)
    {
        p = sLine.GetToken(p, '\n', TRIM_NONE);
        sLine.TrimRight(TRIM_ALL);
        
        if (pOptions->WriteTurnNo > 0 && !TurnNoMarkerWritten && sLine.GetLength() > 20)
        {
            BOOL         SkipIt = FALSE;
            const char * p;
             
            p = sLine.GetData();
            while (*p)   
            {
                if ('-' != *p)
                {
                    SkipIt = TRUE;
                    break;
                }
                p++;
            }
            if (!SkipIt)
            {
                sLine << ";" << pOptions->WriteTurnNo;
                TurnNoMarkerWritten = TRUE;
            }
        }
        
        sLine << EOL_FILE;
        Dest.WriteBuf(sLine.GetData (), sLine.GetLength ());
    }

    sLine.Empty();
    
    if (pOptions->SaveResources)
        ComposeProductsLine(pLand, EOL_FILE, sLine);

    sLine << EOL_FILE << "Exits:" << EOL_FILE ;
    sLine << sExits;
    sLine.TrimRight(TRIM_ALL);
    sLine << EOL_FILE << EOL_FILE;

    Dest.WriteBuf(sLine.GetData (), sLine.GetLength());
    sLine.Empty();

    // Units out of structs, not optimized, does not matter
    if (pOptions->SaveUnits)
        for (i=0; i<pLand->UnitsSeq.Count(); i++)
        {
            pUnit = (CUnit *)pLand->UnitsSeq.At(i);
            if (!IS_NEW_UNIT(pUnit) && !pUnit->GetProperty(PRP_STRUCT_ID, type, value, eOriginal) )
            {
                sLine << pUnit->Description;
                sLine.TrimRight(TRIM_ALL);
                sLine << EOL_FILE;
            }
        }
    
    // Gates and structs
    for (g=0; g<pLand->Structs.Count(); g++)
    {
        pStruct = (CStruct*)pLand->Structs.At(g);
        if (0==(SA_MOBILE & pStruct->Attr) && pOptions->AlwaysSaveImmobStructs // save history
            || pOptions->SaveStructs || pOptions->SaveUnits)
        {
            sLine << EOL_FILE;
            sLine << pStruct->Description.GetData();
            sLine.TrimRight(TRIM_ALL);
            sLine << EOL_FILE;
            
            // Units inside structs, not optimized, does not matter
            if (pOptions->SaveUnits)
                for (i=0; i<pLand->UnitsSeq.Count(); i++)
                {
                    pUnit = (CUnit *)pLand->UnitsSeq.At(i);
                    if (!IS_NEW_UNIT(pUnit) && pUnit->GetProperty(PRP_STRUCT_ID, type, value, eOriginal) && eLong==type && (long)value==pStruct->Id)
                    {
                        sLine << pUnit->Description;
                        sLine.TrimRight(TRIM_ALL);
                        sLine << EOL_FILE;
                    }
                }
            Dest.WriteBuf(sLine.GetData(), sLine.GetLength());
            sLine.Empty();
        }
    }

    sLine << EOL_FILE << EOL_FILE;
    Dest.WriteBuf(sLine.GetData (), sLine.GetLength());
    
    return TRUE;
}

//-------------------------------------------------------------

int  CAtlaParser::SaveOrders(const char * FNameOut, const char * password, BOOL decorate, int factid)
{
    CUnitsByHex   Coll;
    int           i, n, idx;
    CUnit       * pUnit;
    CFileWriter   Dest;
    CStr          S(64), S1(64);
    int           err = ERR_OK;
    const char  * p;
    long          OldLand = -1;
    CLand       * pLand, DummyLand;
    CPlane      * pPlane;
    char          buf[64];
    struct tm     t;
    time_t        now;
    CFaction    * pFaction = NULL;

    if (Dest.Open(FNameOut))
    {
        pFaction =  GetFaction(factid);
                 
        // get our units and sort them
        for (i=0; i<m_Units.Count(); i++)
        {
            pUnit = (CUnit*)m_Units.At(i);
            if (pUnit->FactionId == factid)
                Coll.Insert(pUnit);
        }

        // write orders

        S.Empty();
        S << "#atlantis " << (long)factid << " \"" << password << '\"' << EOL_FILE << EOL_FILE;
        if (pFaction)
        {
            time(&now);
            t = *localtime(&now);
            i = m_YearMon%100-1;
            if ( (i < 12) && (i >=0) )
                sprintf(buf, "%s year %ld,  %s", Monthes[i], m_YearMon/100, asctime(&t));
            else
                sprintf(buf, "%02ld/%02ld,  %s", m_YearMon%100, m_YearMon/100, asctime(&t));
            S << ORDER_CMNT << pFaction->Name << " orders for " << buf  << EOL_FILE << EOL_FILE;
        }
        Dest.WriteBuf(S.GetData(), S.GetLength());

        for (i=0; i<Coll.Count(); i++)
        {
            pUnit = (CUnit*)Coll.At(i);
            if (!pUnit->Orders.IsEmpty() && !IS_NEW_UNIT(pUnit))
            {
                // land comment
                if (decorate && (OldLand!=pUnit->LandId))
                {
                    OldLand      = pUnit->LandId;
                    DummyLand.Id = OldLand;
                    for (n=0; n<m_Planes.Count(); n++)
                    {
                        pPlane = (CPlane*)m_Planes.At(n);
                        if (pPlane->Lands.Search(&DummyLand, idx))
                        {
                            pLand = (CLand*)pPlane->Lands.At(idx);
                            S1.GetToken(pLand->Description.GetData(), '\n');
                            S1.TrimRight(TRIM_ALL);
                            S.Empty();
                            S << EOL_FILE << ORDER_CMNT << S1 << EOL_FILE;
                            Dest.WriteBuf(S.GetData(), S.GetLength());
                            break;
                        }
                    }
                }

                // unit orders
                S.Empty();
                S << EOL_FILE << "unit " << pUnit->Id << EOL_FILE;
                Dest.WriteBuf(S.GetData(), S.GetLength());

                p = pUnit->Orders.GetData();
                while (p && *p)
                {
                    p = S.GetToken(p, '\n', TRIM_NONE);
                    S.TrimRight(TRIM_ALL);
                    S << EOL_FILE;
                    Dest.WriteBuf(S.GetData(), S.GetLength());
                }
            }
        }
        S.Empty();
        S << EOL_FILE << "#end" << EOL_FILE;
        Dest.WriteBuf(S.GetData(), S.GetLength());
        Dest.Close();

    }
    else
        err = ERR_FOPEN;

    Coll.DeleteAll();

    return err;
}

//-------------------------------------------------------------

// count number of men for the specified faction in every hex.
// store as a property

void CAtlaParser::CountMenForTheFaction(int FactionId)
{
    int           nPlane;
    int           nLand;
    int           nUnit;
    CPlane      * pPlane;
    CLand       * pLand;
    CUnit       * pUnit;
    long          nMen, x;
    EValueType    type;

    for (nPlane=0; nPlane<m_Planes.Count(); nPlane++)
    {
        pPlane = (CPlane*)m_Planes.At(nPlane);
        for (nLand=0; nLand<pPlane->Lands.Count(); nLand++)
        {
            pLand = (CLand*)pPlane->Lands.At(nLand);
            if (!pLand->GetProperty(PRP_SEL_FACT_MEN, type, (const void *&)nMen, eNormal) || (eLong!=type))
                nMen=0;
            
            for (nUnit=0; nUnit<pLand->Units.Count(); nUnit++)
            {
                pUnit = (CUnit*)pLand->Units.At(nUnit);
                if (FactionId == pUnit->FactionId)
                {
                    if (pUnit->GetProperty(PRP_MEN, type, (const void *&)x, eNormal) && (eLong==type))
                        nMen+=x;
                }
            }
            pLand->SetProperty(PRP_SEL_FACT_MEN, eLong, (void*)nMen, eBoth);
        }
    }

}

//-------------------------------------------------------------

void CAtlaParser::LoadOrders  (CFileReader & F, int FactionId, BOOL GetComments)
{
    CStr          Line(64), S(64), No(32);
    const char  * p;
    CUnit       * pUnit;
    CUnit         Dummy;
    int           idx;
    char          ch;

    m_CrntFactionPwd.Empty();
    
    for (idx=0; idx<m_Units.Count(); idx++)
    {
        pUnit = (CUnit*)m_Units.At(idx);
        if (pUnit->FactionId == FactionId)
            pUnit->Orders.Empty();
    }
    pUnit = NULL;

    while (F.GetNextLine(Line))
    {
        Line.TrimRight(TRIM_ALL);
        p = SkipSpaces(Line.GetData());

        if (0==SafeCmp(p, "#end"))  // that is the end of report
            break;
        if (!p || !*p || (';'==*p && ('*'==*(p+1) || !GetComments)) )
            continue;

        p = S.GetToken(p, " \t", ch, TRIM_ALL);
        if (0==stricmp(S.GetData(),"unit"))
        {
            p = No.GetToken(p, " \t", ch);
            Dummy.Id = atol(No.GetData());
            if (m_Units.Search(&Dummy, idx))
                pUnit = (CUnit*)m_Units.At(idx);
            else
                pUnit = NULL;
        }
        else
            if (0==stricmp(S.GetData(),"#atlantis"))
            {
                // #atlantis NN "password"
                m_CrntFactionPwd = S.GetToken(p, " \t", ch, TRIM_ALL);
                if (!m_CrntFactionPwd.IsEmpty() && '\"'==m_CrntFactionPwd.GetData()[0])
                {
                    m_CrntFactionPwd.DelCh(0);
                    if (!m_CrntFactionPwd.IsEmpty() && '\"'==m_CrntFactionPwd.GetData()[m_CrntFactionPwd.GetLength()-1])
                        m_CrntFactionPwd.DelCh(m_CrntFactionPwd.GetLength()-1);
                }
            }
            else
                if (pUnit)
                {
                    Line << EOL_SCR;
                    pUnit->Orders.AddStr(Line.GetData(), Line.GetLength());
                }
    }

    RunOrders(NULL);
    m_OrdersLoaded = TRUE;
}

//-------------------------------------------------------------

int  CAtlaParser::LoadOrders  (const char * FNameIn, int & FactionId)
{
    CFileReader   F;
    CStr          Line(64), S(64), No(32);
    CUnit       * pUnit;
    CUnit         Dummy;
    const char  * p;
    char          ch;
    int           idx;

    if (FNameIn && *FNameIn && F.Open(FNameIn))
    {
        // find out what is the faction id        
        FactionId = 0;
        while (F.GetNextLine(Line))
        {
            Line.TrimRight(TRIM_ALL);
            p = SkipSpaces(Line.GetData());
    
            if (0==SafeCmp(p, "#end"))  // that is the end of report
                break;
            if (!p || !*p || ';'==*p || '#'==*p)
                continue;
    
            p = S.GetToken(p, " \t", ch, TRIM_ALL);
            if (0==stricmp(S.GetData(),"unit"))
            {
                p = No.GetToken(p, " \t", ch);
                Dummy.Id = atol(No.GetData());
                if (m_Units.Search(&Dummy, idx))
                {
                    pUnit = (CUnit*)m_Units.At(idx);
                    if (pUnit->FactionId > 0)
                    {
                        FactionId = pUnit->FactionId; // just take the first unit
                        break;
                    }
                }
            }
        }
        
        F.Close();
        F.Open(FNameIn);
        LoadOrders(F, FactionId, TRUE);
        F.Close();
        return ERR_OK;
    }
    return ERR_FOPEN;
}

//-------------------------------------------------------------

void CAtlaParser::OrderErrFinalize()
{
    if (ReportOrderErrLoc && !m_sOrderErrors.IsEmpty())
        ReportOrderErrLoc(m_sOrderErrors.GetData(), m_sOrderErrors.GetLength());

    m_sOrderErrors.Empty();
}

void CAtlaParser::OrderErr(int Severity, int UnitId, const char * Msg)
{
    const char * type;
    CStr         S(32);

    if (0==Severity)
        type = "Error  ";
    else
        type = "Warning";
    S.Format("Unit % 5d %s : %s%s", UnitId, type, Msg, EOL_SCR);

    m_sOrderErrors << S;
}

//-------------------------------------------------------------

#define GEN_ERR(pUnit, msg)                 \
{                                           \
    Line << msg;                            \
    OrderErr(0, pUnit->Id, Line.GetData()); \
    return Changed;                         \
}


BOOL CAtlaParser::ShareSilver(CUnit * pMainUnit)
{
    CLand             * pLand = NULL;
    CUnit             * pUnit;
    int                 idx;
    long                unitmoney;
    long                mainmoney;
    long                n;
    EValueType          type;
    CStr                Line(32);
    BOOL                Changed = FALSE;

    do
    {
        if (!pMainUnit || IS_NEW_UNIT(pMainUnit) || !pMainUnit->IsOurs  )
            break;

        pLand = GetLand(pMainUnit->LandId);
        if (!pLand)
            break;

        RunLandOrders(pLand); // just in case...

        if (!pMainUnit->GetProperty(PRP_SILVER, type, (const void *&)mainmoney, eNormal) )
        {
            mainmoney = 0;
            type      = eLong;
            if (PE_OK!=pMainUnit->SetProperty(PRP_SILVER, type, (const void*)mainmoney, eBoth))
                GEN_ERR(pMainUnit, NOSETUNIT << pMainUnit->Id << BUG);
        }
        else if (eLong!=type)
            GEN_ERR(pMainUnit, NOTNUMERIC << pMainUnit->Id << BUG);

        mainmoney -= pMainUnit->SilvRcvd;

        if (mainmoney<=0)
            break;


        for (idx=0; idx<pLand->Units.Count(); idx++)
        {
            pUnit = (CUnit*)pLand->Units.At(idx);
            if (!pUnit->IsOurs)
                continue;

            if (!pUnit->GetProperty(PRP_SILVER, type, (const void *&)unitmoney, eNormal) )
                continue;
            else if (eLong!=type)
                GEN_ERR(pUnit, NOTNUMERIC << pUnit->Id << BUG);

            if (unitmoney>=0)
                continue;

            n = -unitmoney;
            if (n>mainmoney)
            {
                n         = mainmoney;
                mainmoney = 0;
            }
            else
                mainmoney -= n;

            pMainUnit->Orders.TrimRight(TRIM_ALL);
            if (!pMainUnit->Orders.IsEmpty())
                pMainUnit->Orders << EOL_SCR ;
            if (IS_NEW_UNIT(pUnit))
                pMainUnit->Orders << "GIVE " << pUnit->Name << " " << n << " SILV";
            else
                pMainUnit->Orders << "GIVE " << pUnit->Id   << " " << n << " SILV";
            Changed = TRUE;
            if (mainmoney<=0)
                break;
        }
        if (Changed)
            RunLandOrders(pLand);
    } while (FALSE);

    OrderErrFinalize();
    return Changed;
}

//-------------------------------------------------------------

BOOL CAtlaParser::GenGiveEverything(CUnit * pFrom, long nTo)
{
    CLand             * pLand = NULL;
    int                 no;
    BOOL                Changed = FALSE;
    const char        * propname;
    int                 i;
    BOOL                skipit; 
    EValueType          type;
    long                amount;
    long                amountorg;
    long                x;
    CUnit               Dummy;
    CStr                S;
    const char        * p;

    do
    {
        if (!pFrom || IS_NEW_UNIT(pFrom) || !pFrom->IsOurs  )
            break;
    
        pLand = GetLand(pFrom->LandId);
        if (!pLand)
            break;
    
        Dummy.Id = nTo;    
        if (!pLand->Units.Search(&Dummy, i) )
        {
            S << "Can not find unit " << nTo;
            OrderErr(0, pFrom->Id, S.GetData()); 
            break;
        }   
            
        RunLandOrders(pLand); // just in case...
    
        pFrom->Orders.TrimRight(TRIM_ALL);
        if (!pFrom->Orders.IsEmpty())
            pFrom->Orders << EOL_SCR ;
        
        no = 0;
        do 
        {
            propname = pFrom->GetPropertyName(no++);
            skipit   = FALSE;
            
            if (!propname)
                break;
        
            // ignore standard properties
            for (i=0; i<STD_UNIT_PROPS_COUNT; i++)
                if (0==stricmp(propname, STD_UNIT_PROPS[i]))
                {
                    skipit = TRUE;
                    break;
                }
            if (skipit)
                continue;

            // ignore skills and such    
            for (i=0; i<STD_UNIT_POSTFIXES_COUNT; i++)
            {
                p       = propname;
                while (p)
                {
                    p = strstr(p, STD_UNIT_POSTFIXES[i]);
                    if (p)
                    {
                        p += strlen(STD_UNIT_POSTFIXES[i]);
                        if (!*p)
                        {
                            skipit = TRUE;
                            break;
                        }
                    }
                }
                if (skipit)
                    break;
            }
            if (skipit)
                continue;
                    
    
            // ignore strings    
            if (!pFrom->GetProperty(propname, type, (const void *&)amount, eNormal) || eLong!=type)            
                continue;
            // ignore strings    
            if (!pFrom->GetProperty(propname, type, (const void *&)amountorg, eOriginal) || eLong!=type)            
                continue;
            x = min(amount, amountorg);
            if (x<=0)
                continue;    
    
            pFrom->Orders << "GIVE " << nTo << " " << x << " " << propname << EOL_SCR;
            Changed = TRUE;
        
        } while (propname);
        
        if (Changed)
            RunLandOrders(pLand);
    }
    while (FALSE); 

    OrderErrFinalize();
    return Changed;
}

//-------------------------------------------------------------

BOOL CAtlaParser::GenOrdersTeach(CUnit * pMainUnit)
{
    CLand             * pLand = NULL;
    CUnit             * pUnit;
    int                 idx;
    EValueType          type;
    long                n1, n2;
    CStr                Line(32);
    BOOL                Changed  = FALSE;


    do
    {
        if (!pMainUnit || IS_NEW_UNIT(pMainUnit) || !pMainUnit->IsOurs)
            break;

        pLand = GetLand(pMainUnit->LandId);
        if (!pLand)
            break;

        RunLandOrders(pLand); // just in case...

        for (idx=0; idx<pLand->UnitsSeq.Count(); idx++)
        {
            pUnit = (CUnit*)pLand->UnitsSeq.At(idx);
            if (!pUnit->StudyingSkill.IsEmpty())
            {
                if (!pMainUnit->GetProperty(pUnit->StudyingSkill.GetData(), type, (const void *&)n1, eNormal) )
                    n1 = 0;
                if (!pUnit->GetProperty(pUnit->StudyingSkill.GetData(), type, (const void *&)n2, eNormal) )
                    n2 = 0;

                if ( (n1 > n2) &&            // can teach
                     (pUnit->Teaching <= 20) // student only partially taught by someone else
                   )
                {
                    // count men now

                    if (!pMainUnit->GetProperty(PRP_MEN, type, (const void *&)n1, eNormal)  || (n1<=0))
                        break;
                    if (!pUnit->GetProperty(PRP_MEN, type, (const void *&)n2, eNormal) || (n2<=0))
                        continue;

                    if (n1*(STUDENTS_PER_TEACHER - pMainUnit->Teaching) >= n2)
                    {
                        pMainUnit->Teaching += (double)n2/n1;

                        pMainUnit->Orders.TrimRight(TRIM_ALL);
                        if (!pMainUnit->Orders.IsEmpty())
                            pMainUnit->Orders << EOL_SCR ;
                        if (IS_NEW_UNIT(pUnit))
                            pMainUnit->Orders << "TEACH " << pUnit->Name;
                        else
                            pMainUnit->Orders << "TEACH " << pUnit->Id;
                        Changed = TRUE;
                    }
                }
            }
        }

    } while (FALSE);

    if (Changed)
        RunLandOrders(pLand); // just in case...

    return Changed;
}

//-------------------------------------------------------------

BOOL CAtlaParser::DiscardJunkItems(CUnit * pUnit, const char * junk)
{
    CLand             * pLand = NULL;
    EValueType          type;
    long                value;
    CStr                sJunkItem(32);
    BOOL                Changed = FALSE;

    if (!pUnit || IS_NEW_UNIT(pUnit) || !pUnit->IsOurs )
        return FALSE;

    pLand = GetLand(pUnit->LandId);
    if (!pLand)
        return FALSE;

    RunLandOrders(pLand); // just in case...

    while (junk && *junk)
    {
        junk = sJunkItem.GetToken(junk, ',');
        if (!pUnit->GetProperty(sJunkItem.GetData(), type, (const void *&)value, eNormal) || (eLong!=type))
            continue;

        pUnit->Orders.TrimRight(TRIM_ALL);
        if (!pUnit->Orders.IsEmpty())
            pUnit->Orders << EOL_SCR ;
        pUnit->Orders << "GIVE 0 " << value << " " << sJunkItem;
        Changed = TRUE;
    }
    if (Changed)
        RunLandOrders(pLand);
    return Changed;
}



//-------------------------------------------------------------

BOOL CAtlaParser::DetectSpies(CUnit * pUnit, long lonum, long hinum, long amount)
{

#define DETECT_ONE_SPY                                                         \
{                                                                              \
    pUnit->Orders << "GIVE " << no << " " << amount << " SILV ;ne"  << EOL_SCR;\
    Changed = TRUE;                                                            \
    no++;                                                                      \
}

    CLand             * pLand = NULL;
    BOOL                Changed = FALSE;
    long                no;
    int                 idx = 0;
    CUnit             * pNext;

    if (!pUnit || IS_NEW_UNIT(pUnit) || !pUnit->IsOurs)
        return FALSE;

    pLand = GetLand(pUnit->LandId);
    if (!pLand)
        return FALSE;

    RunLandOrders(pLand); // just in case...

    pUnit->Orders.TrimRight(TRIM_ALL);
    if (!pUnit->Orders.IsEmpty())
        pUnit->Orders << EOL_SCR ;

    no = lonum;
    while (no<=hinum)
    {
        pNext = (CUnit*)m_Units.At(idx);

        if (pNext)
        {
            while (no<pNext->Id)
                DETECT_ONE_SPY
            if (no == pNext->Id)
                no++;
            idx++;
        }
        else
            DETECT_ONE_SPY
    }

    if (Changed)
        RunLandOrders(pLand);

    return Changed;
}


//-------------------------------------------------------------

const char * CAtlaParser::ReadPropertyName(const char * src, CStr & Name)
{
    char ch;
    int  i;
    CStr S;

    Name.Empty();
    src = SkipSpaces(S.GetToken(src, ";\n", ch, TRIM_ALL));

    if (!S.IsEmpty())
    {
        if ('"'==S.GetData()[0])
        {
            if ('"'==S.GetData()[S.GetLength()-1])
            {
                // remove quotes, replace spaces with underscores

                S.DelCh(S.GetLength()-1);
                S.DelCh(0);
                for (i=0; i<S.GetLength(); i++)
                    if (' '==S.GetData()[i])
                        S.SetCh(i, '_');
            }
            else
                return src;
        }
        Name = ResolveAliasLoc(S.GetData());
    }

    return src;
}

//-------------------------------------------------------------

#define SHOW_WARN(msg)                               \
{                                                    \
    if (!skiperror)                                  \
    {                                                \
        ErrorLine.Empty();                           \
        ErrorLine << Line << msg;                    \
        OrderErr(1, pUnit->Id, ErrorLine.GetData()); \
        docontinue = TRUE;                           \
        continue;                                    \
    }                                                \
}


#define SHOW_ERR(msg)                            \
{                                                \
    ErrorLine.Empty();                           \
    ErrorLine << Line << msg;                    \
    OrderErr(0, pUnit->Id, ErrorLine.GetData()); \
    docontinue = TRUE;                           \
    errors = TRUE;                               \
    continue;                                    \
}


void CAtlaParser::RunLandOrders(CLand * pLand, const char * sCheckTeach)
{
    enum
    {      SQ_MIN    ,
           SQ_FORM   ,
           SQ_CLAIM  ,
           SQ_LEAVE  ,
           SQ_ENTER  ,
           SQ_PROMOTE,
           SQ_GIVE   ,
//           SQ_GIVE2  ,
           SQ_SELL   , // Shar1 Extrict SELL/BUY check. Sell is executed before buy
           SQ_BUY    ,
           SQ_STUDY  ,
           SQ_TEACH  ,
           SQ_MOVE   ,
           SQ_MAX
    };

    CBaseObject         Dummy;
    CUnit             * pUnit;
    CUnit             * pUnitNew;
    CUnit             * pUnitMaster;
    CUnit             * pUnit2;
    CStruct           * pStruct;
    CStr                Line(32);
    CStr                Cmd (32);
    CStr                S   (32);
    CStr                S1  (32);
    CStr                N1  (32);
    CStr                N2  (32);
    CStr                LandProp(32);
    CStr                ErrorLine(32);
    CStr                AllProp;
    const char        * p, * p1;
    const char        * src;
    long                n;
    long                n1;
    long                n2;
    long                peritem;
    long                unitmoney;
    long                unitprop;
    int                 mainidx;
    int                 sequence;
    int                 i;
    BOOL                isnew;
    BOOL                errors = FALSE;
    int                 idx;
    EValueType          type;
    EValueType          type1;
    const void        * value;
    const void        * value2;
    int                 X, Y, Z;    // current coordinates for moving unit
    int                 XN=0, YN=0;     // saved current coords while new unit is processed
    char                ch;
    BOOL                docontinue;
    BOOL                our_unit;
    BOOL                TeachCheckGlb = (BOOL)atoi(sCheckTeach?sCheckTeach:GetConfStringLoc(SZ_SECT_COMMON, SZ_KEY_CHECK_TEACH_LVL));

    int               * weights;    // calculate weight change while giving
    const char       ** movenames;
    int                 movecount;
    BOOL                skiperror;
    BOOL                inTurn; // Shar1 Support for TURN/ENDTURN
    long                landprop; // Shar1 Extrict SELL/BUY check


    // Reset land and old units and remove new units
    pLand->ResetNormalProperties();
    pLand->ResetUnits();
    pLand->Flags &= ~(LAND_TAX_NEXT | LAND_TRADE_NEXT);
    

    // Run Orders
    for (sequence=SQ_MIN+1; sequence<SQ_MAX; sequence++)
    {
        if (errors)
            break;

        for (mainidx=0; mainidx<pLand->UnitsSeq.Count(); mainidx++)
        {
            inTurn = FALSE; // Shar1 Support for TURN/ENDTURN
            pUnitMaster = NULL;
            pUnit       = (CUnit*)pLand->UnitsSeq.At(mainidx);
            if (IS_NEW_UNIT(pUnit))
                continue;

            LandIdToCoord(pLand->Id, X, Y, Z);

            src         = pUnit->Orders.GetData();
            while (src && *src)
            {
                long order;
                isnew= FALSE;
                src  = Line.GetToken(src, '\n', TRIM_ALL);

                // trim all the comments
                p = strchr(Line.GetData(), ';');
                if (p)
                {
                    S1 = SkipSpaces(++p);
                    S1.TrimRight(TRIM_ALL);
                    skiperror = (0==stricmp(S1.GetData(),"ne"));
                    Line.DelSubStr(p-Line.GetData()-1, Line.GetLength() - (p-Line.GetData()-1) );
                }
                else
                    skiperror = FALSE;

                p    = SkipSpaces(Cmd.GetToken(Line.GetData(), " \t", ch, TRIM_ALL));
                if ('@'==Cmd.GetData()[0])
                {
                    Cmd.DelCh(0);
                    if (Cmd.IsEmpty()) // I do not know if putting spaces after '@' is legal... anyway
                        p    = SkipSpaces(Cmd.GetToken(p, " \t", ch, TRIM_ALL));
                }

                if (Cmd.IsEmpty())
                    continue;
                if (!GetOrderIdLoc(Cmd.GetData(), order))
                {
                    if (SQ_MIN+1 == sequence)
                        SHOW_WARN(" - unknown order")
                    else
                        continue;
                }
                
                // Shar1 Support for TURN/ENDTURN - Start
                if (inTurn)
                {
                    // I don't know if Atlantis support nested TURN/ENDTURN tags
                    // By now, ALH won't support them. But it can be changed easily,
                    // changing the inTurn into a turnLevel (integer)
                    if (O_TURN == order)
                    {
                        SHOW_WARN(" - Nesting TURN/ENDTURN is not allowed");
                    }
                    else if (O_ENDTURN == order)
                    {
                        inTurn = FALSE;
                    }
                }
                else if (O_TURN == order)
                {
                    inTurn = TRUE;
                }
                else if (O_ENDTURN == order)
                {
                    SHOW_WARN(" - TURN was not called!");
                }                        // Shar1 Support for TURN/ENDTURN - End

//     Give phase is split into two to prevent chain gives, like A -> B -> C
//                                
//                else if ( ( (SQ_GIVE==sequence) || (SQ_GIVE2==sequence) ) &&
//                          (O_GIVE == order) )
//                {
//                    // GIVE               15 UNIT
//                    // GIVE [NEW]         3  15 SILV
//                    // GIVE FACTION X NEW Y  N  Something
//                    //                    n1 n2 S1
//
//                    p = SkipSpaces(N1.GetToken(p, " \t", ch, TRIM_ALL));
//                    if (0==stricmp("FACTION", N1.GetData()))
//                    {
//                        p = SkipSpaces(N1.GetToken(p, " \t", ch, TRIM_ALL));  // X
//                        p = SkipSpaces(N1.GetToken(p, " \t", ch, TRIM_ALL));  // NEW
//                        p = SkipSpaces(N1.GetToken(p, " \t", ch, TRIM_ALL));  // Y
//                        N1= "0";
//                    }
//                    if (0==stricmp("NEW", N1.GetData()))
//                    {
//                        isnew = TRUE;
//                        p = SkipSpaces(N1.GetToken(p, " \t", ch, TRIM_ALL));
//                    }
//
//                    p = SkipSpaces(N2.GetToken(p, " \t", ch, TRIM_ALL));
//
//
//                    //p = SkipSpaces(S1.GetToken(p, " \t", ch, TRIM_ALL));
//                    p = ReadPropertyName(p, S1);
//
//
//                    n1 = atol(N1.GetData());
//                    n2 = atol(N2.GetData());
//                    if (0==stricmp("UNIT", N2.GetData()))  // give unit to another faction
//                        continue;
//
//                    if (S1.IsEmpty() || (n1<0) || ((n2<=0) && (0!=stricmp(N2.GetData(), "ALL"))) )
//                        SHOW_WARN(" - Invalid GIVE command");
//                    if (isnew)
//                        n1 = NEW_UNIT_ID(pLand, n1);
//
//                    if (n1==pUnit->Id)
//                        SHOW_WARN(" - Isn't it stupid?");
//
//                    Dummy.Id = n1;
//                    if ((0==n1) || pLand->Units.Search(&Dummy, idx))
//                    {
//                        pUnit2 = (CUnit*)pLand->Units.At(idx);
//
//                        if ( pUnit->GetProperty(S1.GetData(), type, value, eNormal) && (eLong==type))
//                        {
//                            if (0==stricmp(N2.GetData(), "ALL"))
//                            {
//                                // ok, we can not calculate "ALL" at the SQ_GIVE2 phase, and we can not store it 
//                                // in a single var. So we will store it in a unit's property
//                                AllProp = S1;
//                                AllProp << PRP_GIVE_ALL_POSTFIX;
//                                if (SQ_GIVE==sequence)
//                                {
//                                    if (PE_OK!=pUnit->SetProperty(AllProp.GetData(), type, value, eNormal))
//                                        SHOW_WARN(NOSET << BUG);
//                                }   
//                                else
//                                {
//                                    if ( !pUnit->GetProperty(AllProp.GetData(), type, value, eNormal) || (eLong!=type))
//                                        SHOW_WARN(" - Can not get GIVE_ALL property" << BUG);
//                                    pUnit->DelProperty(AllProp.GetData());
//                                }
//                                    
//                                n2 = (long)value;
//                            }
//                            if (SQ_GIVE==sequence)
//                            {
//                                if (PE_OK!=pUnit->SetProperty(S1.GetData(), type, (const void*)((long)value-n2), eNormal))
//                                    SHOW_WARN(NOSET << BUG);
//                                if (n2 > (long)value)
//                                    SHOW_WARN(" - That is too MANY! Give " << (long)value << " at max.");
//                            }
//                            else
//                            {
//                                if (n1!=0) // otherwise pUnit2 is invalid
//                                {
//                                    if (!pUnit2->GetProperty(S1.GetData(), type, value2, eNormal) )
//                                    {
//                                        value2 = (const void*)0L;
//                                        // set original value to 0!
//                                        if (PE_OK!=pUnit2->SetProperty(S1.GetData(), type, value2, eNormal))
//                                            SHOW_WARN(NOSETUNIT << n1 << BUG);
//                                    }
//                                    else if (eLong!=type)
//                                        SHOW_WARN(NOTNUMERIC << n1 << BUG);
//
//                                    if (PE_OK!=pUnit2->SetProperty(S1.GetData(), type, (const void*)((long)value2+n2), eNormal))
//                                        SHOW_WARN(NOSET << BUG);
//                                    if (0==stricmp(PRP_SILVER, S1.GetData()))
//                                        pUnit2->SilvRcvd += n2;
//                                }
//
//                                // modify weights in the second phase only
//                                if ( GetItemWeightsLoc(S1.GetData(), weights, movenames, movecount) )
//                                {
//
//                                    pUnit ->AddWeight(-n2, weights, movenames, movecount);
//                                    if (n1!=0) // otherwise pUnit2 is invalid
//                                        pUnit2->AddWeight( n2, weights, movenames, movecount);
//                                }
//                            }
//                        }
//                        else
//                            if (SQ_GIVE==sequence)
//                                SHOW_WARN(" - Can not GIVE that!")
//                    }
//                    else
//                        if (SQ_GIVE==sequence)
//                            SHOW_WARN(" - Can not find unit " << n1)
//
//                }

                else if ( (SQ_GIVE == sequence) && (O_GIVE == order) )
                {
                    // GIVE               15 UNIT
                    // GIVE [NEW]         3  15 SILV
                    // GIVE FACTION X NEW Y  N  Something
                    //                    n1 n2 S1

                    p = SkipSpaces(N1.GetToken(p, " \t", ch, TRIM_ALL));
                    if (0==stricmp("FACTION", N1.GetData()))
                    {
                        p = SkipSpaces(N1.GetToken(p, " \t", ch, TRIM_ALL));  // X
                        p = SkipSpaces(N1.GetToken(p, " \t", ch, TRIM_ALL));  // NEW
                        p = SkipSpaces(N1.GetToken(p, " \t", ch, TRIM_ALL));  // Y
                        N1= "0";
                    }
                    if (0==stricmp("NEW", N1.GetData()))
                    {
                        isnew = TRUE;
                        p = SkipSpaces(N1.GetToken(p, " \t", ch, TRIM_ALL));
                    }

                    p = SkipSpaces(N2.GetToken(p, " \t", ch, TRIM_ALL));


                    //p = SkipSpaces(S1.GetToken(p, " \t", ch, TRIM_ALL));
                    p = ReadPropertyName(p, S1);


                    n1 = atol(N1.GetData());
                    n2 = atol(N2.GetData());

                    if (0==stricmp("UNIT", N2.GetData()))  // give unit to another faction
                    {
                        n2 = 1;    // need to pass the following check
                        S1 = "1";
                    }
                    if (S1.IsEmpty() || n1<0 || (n2<=0 && 0!=stricmp(N2.GetData(), "ALL") ) )
                        SHOW_WARN(" - Invalid GIVE command");
                    if (isnew)
                        n1 = NEW_UNIT_ID(pLand, n1);

                    if (n1==pUnit->Id)
                        SHOW_WARN(" - Giving to yourself");

                    Dummy.Id = n1;
                    if ((0==n1) || pLand->Units.Search(&Dummy, idx))
                    {
                        pUnit2 = (CUnit*)pLand->Units.At(idx);

                        if (0==stricmp("UNIT", N2.GetData()))  // give unit to another faction
                        {
                            if (pUnit2 && pUnit->FactionId == pUnit2->FactionId)
                                SHOW_WARN(" - Giving to your own faction");
                            pUnit->Flags |= UNIT_GIVEN;
                            continue;
                        }
                        
                        if ( pUnit->GetProperty(S1.GetData(), type, value, eNormal) && (eLong==type))
                        {
                            if (0==stricmp(N2.GetData(), "ALL"))
                                n2 = (long)value;
                            
                            if (PE_OK!=pUnit->SetProperty(S1.GetData(), type, (const void*)((long)value-n2), eNormal))
                                SHOW_WARN(NOSET << BUG);
                            if (n2 > (long)value)
                                SHOW_WARN(" - That is too MANY! Give " << (long)value << " at max.");

                            if (n1!=0) // otherwise pUnit2 is invalid
                            {
                                if (!pUnit2->GetProperty(S1.GetData(), type, value2, eNormal) )
                                {
                                    value2 = (const void*)0L;
                                    // set original value to 0!
                                    if (PE_OK!=pUnit2->SetProperty(S1.GetData(), type, value2, eNormal))
                                        SHOW_WARN(NOSETUNIT << n1 << BUG);
                                }
                                else if (eLong!=type)
                                    SHOW_WARN(NOTNUMERIC << n1 << BUG);
  
                                if (PE_OK!=pUnit2->SetProperty(S1.GetData(), type, (const void*)((long)value2+n2), eNormal))
                                    SHOW_WARN(NOSET << BUG);
                                if (0==stricmp(PRP_SILVER, S1.GetData()))
                                    pUnit2->SilvRcvd += n2;
                            }

                            if ( GetItemWeightsLoc(S1.GetData(), weights, movenames, movecount) )
                            {
                                pUnit ->AddWeight(-n2, weights, movenames, movecount);
                                if (n1!=0) // otherwise pUnit2 is invalid
                                    pUnit2->AddWeight( n2, weights, movenames, movecount);
                            }
                        }
                        else
                            SHOW_WARN(" - Can not GIVE that!")
                    }
                    else
                        SHOW_WARN(" - Can not find unit " << n1)
                }
                
                else if ( (SQ_BUY==sequence) && (O_BUY == order) )
                {
                    // BUY 33 VIKI
                    //     n1 S1
                    p = SkipSpaces(N1.GetToken(p, " \t", ch, TRIM_ALL));
                    //p = SkipSpaces(S1.GetToken(p, " \t", ch, TRIM_ALL));
                    p = ReadPropertyName(p, S1);
                    n1= atol(N1.GetData());
                    if (S1.IsEmpty() || (n1<=0 && 0!=stricmp(N1.GetData(), "ALL")) )
                        SHOW_WARN(" - Invalid BUY command");

                    //LandProp = ResolveAliasLoc(S1.GetData());
                    LandProp = S1;
                    LandProp << PRP_SALE_PRICE_POSTFIX;
                    if ( pLand->GetProperty(LandProp.GetData(), type, (const void *&)peritem, eNormal) && (eLong==type))
                    {
                        if (0==stricmp(N1.GetData(), "ALL"))
                        {
                            LandProp = S1;
                            LandProp << PRP_SALE_AMOUNT_POSTFIX;
                            if ( !pLand->GetProperty(LandProp.GetData(), type, (const void *&)n1, eNormal) || (eLong!=type))
                                n1=0;
                        }

                        if (!pUnit->GetProperty(S1.GetData(), type, (const void *&)unitprop, eNormal) )
                        {
                            unitprop = 0;
                            if (PE_OK!=pUnit->SetProperty(S1.GetData(), type, (const void *)unitprop, eBoth))
                                SHOW_WARN(NOSETUNIT << pUnit->Id << BUG);
                        }
                        else if (eLong!=type)
                            SHOW_WARN(NOTNUMERIC << pUnit->Id << BUG);

                        if (!pUnit->GetProperty(PRP_SILVER, type, (const void *&)unitmoney, eNormal) )
                        {
                            unitmoney = 0;
                            if (PE_OK!=pUnit->SetProperty(PRP_SILVER, type, (const void*)unitmoney, eBoth))
                                SHOW_WARN(NOSETUNIT << pUnit->Id << BUG);
                        }
                        else if (eLong!=type)
                            SHOW_WARN(NOTNUMERIC << pUnit->Id << BUG);

                        // Shar1 Extrict SELL/BUY check. Start
                        
                        // Checking amount
                        LandProp = S1;
                        LandProp << PRP_SALE_AMOUNT_POSTFIX;
                        if (!pLand->GetProperty(LandProp.GetData(), type, (const void *&) landprop, eNormal) || (eLong!=type))
                            landprop = 0;
                        if (n1 > landprop)
                        {
                            SHOW_WARN(" - That is too MANY! Buy " << (long) landprop << " at max.");
                            n1 = landprop;
                        }
                        unitmoney -= n1*peritem; // This is the old code
                        unitprop  += n1;         // This is the old code
                        landprop  -= n1;

                        if ( (PE_OK!=pUnit->SetProperty(S1.GetData(), type, (const void *)unitprop,  eNormal)) || // This is the old code
                             (PE_OK!=pUnit->SetProperty(PRP_SILVER,   type, (const void *)unitmoney, eNormal)) || // This is ALMOST the old code
                             (PE_OK!=pLand->SetProperty(LandProp.GetData(), type, (const void *)landprop,  eNormal)))
                        // Shar1 End
                            SHOW_WARN(NOSET << BUG);

                        if (IsTradeItemLoc(S1.GetData()))
                            pUnit->Flags |= UNIT_PRODUCING;
                            
                        // adjust weight
                        if (GetItemWeightsLoc(S1.GetData(), weights, movenames, movecount))
                            pUnit ->AddWeight(n1, weights, movenames, movecount);
                    }
                    else
                        SHOW_WARN(" - Can not BUY that!");
                }


                else if ( (SQ_SELL == sequence) && (O_SELL == order) )
// Shar1 Extrict SELL/BUY check. New sell phase                else if ( (SQ_BUY==sequence) && (O_SELL == order) )
                {
                    // BUY 33 VIKI
                    //     n1 S1
                    p = SkipSpaces(N1.GetToken(p, " \t", ch, TRIM_ALL));
                    //p = SkipSpaces(S1.GetToken(p, " \t", ch, TRIM_ALL));
                    p = ReadPropertyName(p, S1);
                    n1= atol(N1.GetData());
                    if (S1.IsEmpty() || (n1<=0 && 0!=stricmp(N1.GetData(), "ALL")) )
                        SHOW_WARN(" - Invalid SELL command");

                    LandProp = S1; //ResolveAliasLoc(S1.GetData());
                    LandProp << PRP_WANTED_PRICE_POSTFIX;
                    if ( !pLand->GetProperty(LandProp.GetData(), type,  (const void *&)peritem,  eNormal) ||
                         !pUnit->GetProperty(S1.GetData(),       type1, (const void *&)unitprop, eNormal) ||
                         (eLong!=type) || (eLong!=type1)
                         )
                        SHOW_WARN(" - Can not SELL that!");
                    if (0==stricmp(N1.GetData(), "ALL"))
                        n1=unitprop;
                    if (!pUnit->GetProperty(PRP_SILVER, type, (const void *&)unitmoney, eNormal) )
                    {
                        unitmoney = 0;
                        if (PE_OK!=pUnit->SetProperty(PRP_SILVER, type, (const void*)unitmoney, eBoth))
                            SHOW_WARN(NOSETUNIT << pUnit->Id << BUG);
                    }
                    else if (eLong!=type)
                        SHOW_WARN(NOTNUMERIC << pUnit->Id << BUG);

                    // Shar1 Extrict SELL/BUY check. Start
                    // Check amount wanted by market
                    LandProp = S1;
                    LandProp << PRP_WANTED_AMOUNT_POSTFIX;
                    if (!pLand->GetProperty(LandProp.GetData(), type,  (const void *&) landprop,  eNormal) || (eLong!=type))
                    {
                        SHOW_WARN(" - Can not SELL that!");
                        landprop = 0;
                        n1 = 0;
                    }
                    if (n1 > landprop)
                    {
                        SHOW_WARN(" - Selling beyond market's capacity! Sell " << (long) landprop << " at max.");
                        n1 = landprop;
                    }
                    // Check amount owned by unit
                    if (n1 > unitprop)
                    {
                        SHOW_WARN(" - That is too MANY! Sell " << (long) unitprop << " at max.");
                        n1 = unitprop;
                    }
                    unitmoney += n1*peritem; // This is the old code
                    unitprop  -= n1; // This is the old code
                    landprop  -= n1;

                    if ( (PE_OK!=pUnit->SetProperty(S1.GetData(), type, (const void *)unitprop,  eNormal)) || // This is the old code
                         (PE_OK!=pUnit->SetProperty(PRP_SILVER,   type, (const void *)unitmoney, eNormal)) || // This is ALMOST the old code
                         (PE_OK!=pLand->SetProperty(LandProp.GetData()  ,   type, (const void *)landprop,  eNormal)))
                    // Shar1. End
                        SHOW_WARN(NOSET << BUG);

                    pUnit->SilvRcvd += n1*peritem;

                    // adjust weight
                    if (GetItemWeightsLoc(S1.GetData(), weights, movenames, movecount))
                        pUnit ->AddWeight(-n1, weights, movenames, movecount);
                }

                else if (O_FORM == order)
                {
                    p        = SkipSpaces(N1.GetToken(p, " \t", ch, TRIM_ALL));
                    n1       = NEW_UNIT_ID(pLand, atol(N1.GetData()));
                    Dummy.Id = n1;

                    if (SQ_FORM==sequence)
                    {
                        if (!IS_NEW_UNIT_ID(n1))
                            SHOW_WARN(" - Invalid new unit number!");
                        if (pLand->Units.Search(&Dummy, idx))
                            SHOW_WARN(" - Unit already exists!");
                        pUnitNew             = new CUnit;
                        pUnitNew->Id         = n1;
                        pUnitNew->FactionId  = pUnit->FactionId;
                        pUnitNew->pFaction   = pUnit->pFaction;
                        pUnitNew->IsOurs     = TRUE;
                        pUnitNew->Name        << "NEW " << N1;
                        pUnitNew->Description << "Created by " << pUnit->Id;

                        if (pUnit->GetProperty(PRP_STRUCT_ID, type, value, eOriginal) )
                            pUnitNew->SetProperty(PRP_STRUCT_ID, type, value, eBoth);
                        if (pUnit->GetProperty(PRP_STRUCT_NAME, type, value, eOriginal) )
                            pUnitNew->SetProperty(PRP_STRUCT_NAME, type, value, eBoth);


                        //if (pLand->AddUnit(pUnitNew))
                        //    mainidx++; // we added new unit in the coll beginning, so all other units were shifted!
                        //else
                        //    SHOW_WARN(" - Can not create new unit! " << BUG);
                        if (!pLand->AddUnit(pUnitNew))
                            SHOW_WARN(" - Can not create new unit! " << BUG);
                    }
                    else
                    {
                        if (!IS_NEW_UNIT_ID(n1))
                            continue;
                        if (pUnitMaster)
                            SHOW_WARN(" - Sorry, we do not support new units creating units :(");
                        pUnitMaster       = pUnit;
                        if (!pLand->Units.Search(&Dummy, idx))
                            SHOW_WARN(" - Can not find new unit! " << BUG);
                        pUnit = (CUnit*)pLand->Units.At(idx);
                        XN = X;
                        YN = Y;
                        LandIdToCoord(pLand->Id, X, Y, Z);
                    }
                }

                else if ( (SQ_FORM!=sequence) && (O_ENDFORM == order) )
                {
                    if (!pUnitMaster)
                        if (SQ_FORM+1 == sequence)
                        {
                            SHOW_WARN(" - FORM was not called!");
                        }
                        else
                            continue;

                    if  (SQ_TEACH==sequence)  // have to call it here or new units teaching will not be calculated
                        OrderProcess_Teach(pUnit);

                    pUnit       = pUnitMaster;
                    pUnitMaster = NULL;
                    X = XN;
                    Y = YN;
                }

                else if ( (SQ_FORM==sequence) && (O_ATTACK == order || 
                                                  O_ASSASSINATE == order ||
                                                  O_STEAL == order ))
                {
                    // just check if the target is valid
                    // and SQ_FORM is good enough, no need to introduce a new phase
                    while (p && *p)
                    {
                        p = SkipSpaces(N1.GetToken(p, " \t", ch, TRIM_ALL));

                        n1 = atol(N1.GetData());
                        Dummy.Id = n1;
                        if (pLand->Units.Search(&Dummy, idx))
                        {
                            pUnit2 = (CUnit*)pLand->Units.At(idx);
                            if (pUnit2->IsOurs)
                                SHOW_WARN(" - Unit " << n1 << " is our, can not " << Cmd << "!");
                        }   
                        else
                            SHOW_WARN(" - Unit " << n1 << " is not here, can not " << Cmd << "!");
                                                
                        if (O_ATTACK != order) // only attack can have multiple targets
                            break;
                    }
                }
                
                
                else if ( (SQ_STUDY==sequence) && (O_STUDY == order) )
                {
                    p = ReadPropertyName(p, S1);

                    n1 = StudyCostLoc(S1.GetData());
                    if (n1<=0)
                        SHOW_WARN(" - Can not study that!");

                    if (!pUnit->GetProperty(PRP_MEN, type, (const void *&)n2, eNormal) || (n2<=0))
                        SHOW_WARN(" - There are no men in the unit!");
                    if (eLong!=type)
                        SHOW_WARN(NOTNUMERIC << pUnit->Id << BUG);

                    S = S1;
                    S << PRP_SKILL_DAYS_POSTFIX;
                    if (pUnit->GetProperty(S.GetData(), type, (const void *&)n, eNormal) && eLong==type && n>=m_MaxSkillDays)
                        SHOW_WARN(" - Already knows the skill!");


                    if (!pUnit->GetProperty(PRP_SILVER, type, (const void *&)unitmoney, eNormal) )
                    {
                        unitmoney = 0;
                        if (PE_OK!=pUnit->SetProperty(PRP_SILVER, eLong, (const void*)unitmoney, eBoth))
                            SHOW_WARN(NOSETUNIT << pUnit->Id << BUG);
                    }
                    else if (eLong!=type)
                        SHOW_WARN(NOTNUMERIC << pUnit->Id << BUG);

                    unitmoney -= n1*n2;

                    if (PE_OK!=pUnit->SetProperty(PRP_SILVER,   eLong, (const void *)unitmoney, eNormal))
                        SHOW_WARN(NOSET << BUG);

                    pUnit->StudyingSkill = S1;
                    pUnit->StudyingSkill << PRP_SKILLS_POSTFIX;
                }

                else if ( (SQ_TEACH==sequence) && (O_TEACH == order) )
                {
                    docontinue = FALSE;
                    while (p && *p)

                    {
                        p = SkipSpaces(N1.GetToken(p, " \t", ch, TRIM_ALL));
                        isnew = (0==stricmp("NEW", N1.GetData()));
                        if (isnew)
                            p = SkipSpaces(N1.GetToken(p, " \t", ch, TRIM_ALL));
                        if (N1.IsEmpty())
                            break;
                        n1 = atol(N1.GetData());
                        if (isnew)
                            n1 = NEW_UNIT_ID(pLand, n1);


                        Dummy.Id = n1;
                        if (pLand->Units.Search(&Dummy, idx))
                        {
                            pUnit2 = (CUnit*)pLand->Units.At(idx);

                            our_unit = pUnit2->IsOurs;

                            if (pUnit2->StudyingSkill.IsEmpty() && our_unit)
                            {
                                SHOW_WARN(" - Unit " << pUnit2->Id << " is not studying");
                            }
                            else
                            {
                                if (our_unit  && TeachCheckGlb)
                                {
                                    // are the levels ok?
                                    if (!pUnit2->GetProperty(pUnit2->StudyingSkill.GetData(), type, (const void *&)n2, eNormal) )
                                        n2 = 0;
                                    if (eLong!=type)
                                        SHOW_WARN(NOTNUMERIC << n1 << BUG);

                                    if ((!pUnit->GetProperty(pUnit2->StudyingSkill.GetData(), type, (const void *&)n1, eNormal) || (n1<=n2)) &&
                                        (!IS_NEW_UNIT(pUnit))
                                       )
                                        SHOW_WARN(" - Can not teach unit " << pUnit2->Id);
                                    if (eLong!=type)
                                        SHOW_WARN(NOTNUMERIC << n1 << BUG);
                                }

                                // levels are ok, use n1 and n2 for men counts
                                if (!pUnit->GetProperty(PRP_MEN, type, (const void *&)n1, eNormal)  || (n1<=0))
                                    SHOW_WARN(" - There are no men in the unit!");

                                if (!pUnit2->GetProperty(PRP_MEN, type, (const void *&)n2, eNormal) || (n2<=0))
                                    SHOW_WARN(" - There are no men in the student unit!");

                                if (!pUnit->pStudents)
                                    pUnit->pStudents = new CBaseCollById;

                                if (!pUnit->pStudents->Insert(pUnit2))
                                    SHOW_WARN(" - Unit " << pUnit2->Id << " is already in the students list");

                            }


                        }
                        else
                            SHOW_WARN(" - Can not find unit " << n1)
                    }
                    if (docontinue)
                        continue;
                }


                else if ( (SQ_CLAIM==sequence) && (O_CLAIM == order) )
                {
                    p        = SkipSpaces(N1.GetToken(p, " \t", ch, TRIM_ALL));

                    n1       = atol(N1.GetData());

                    if (!pUnit->GetProperty(PRP_SILVER, type, (const void *&)unitmoney, eNormal) )
                    {
                        unitmoney = 0;
                        if (PE_OK!=pUnit->SetProperty(PRP_SILVER, eLong, (const void*)unitmoney, eBoth))
                            SHOW_WARN(NOSETUNIT << pUnit->Id << BUG);
                    }
                    else if (eLong!=type)
                        SHOW_WARN(NOTNUMERIC << pUnit->Id << BUG);

                    unitmoney += n1;

                    if (PE_OK!=pUnit->SetProperty(PRP_SILVER,   eLong, (const void *)unitmoney, eNormal))
                        SHOW_WARN(NOSET << BUG);
                }


                else if ( (SQ_LEAVE==sequence) && (O_LEAVE == order) )
                {
                    if (pUnit->GetProperty(PRP_STRUCT_ID, type, (const void *&)n1) && eLong==type)
                    {
                        pStruct  = pLand->GetStructById(n1);
                        if (pStruct && pStruct->OwnerUnitId == pUnit->Id)
                            pStruct->OwnerUnitId = 0;
                    }

                    if ( (PE_OK!=pUnit->SetProperty(PRP_STRUCT_NAME,  eCharPtr, "", eNormal)) ||
                         (PE_OK!=pUnit->SetProperty(PRP_STRUCT_OWNER, eCharPtr, "", eNormal)) ||
                         (PE_OK!=pUnit->SetProperty(PRP_STRUCT_ID,    eLong   ,  0, eNormal)) )
                        SHOW_WARN(NOSETUNIT << pUnit->Id << BUG);
                }


                else if ( (SQ_ENTER==sequence) && (O_ENTER == order) )
                {
                    p        = SkipSpaces(N1.GetToken(p, " \t", ch, TRIM_ALL));
                    n1       = atol(N1.GetData());
                    pStruct  = pLand->GetStructById(n1);
                    if (pStruct)
                    {
                        if (0==pStruct->OwnerUnitId)
                        {
                            pStruct->OwnerUnitId = pUnit->Id;
                            S1 = YES;
                        }
                        else
                            S1.Empty();
                        if ( (PE_OK!=pUnit->SetProperty(PRP_STRUCT_ID,  eLong, 0,                  eNormal)) ||
                             (PE_OK!=pUnit->SetProperty(PRP_STRUCT_ID,  eLong, (void*)pStruct->Id, eNormal)) ||
                             (PE_OK!=pUnit->SetProperty(PRP_STRUCT_NAME,  eCharPtr, "",                      eNormal)) ||
                             (PE_OK!=pUnit->SetProperty(PRP_STRUCT_NAME,  eCharPtr, pStruct->Name.GetData(), eNormal)) ||
                             (PE_OK!=pUnit->SetProperty(PRP_STRUCT_OWNER, eCharPtr, S1.GetData(), eNormal)) )
                            SHOW_WARN(NOSETUNIT << pUnit->Id << BUG);
                    }
                    else
                        SHOW_WARN(" - Invalid structure number " << n1);
                }


                else if ( (SQ_PROMOTE==sequence) && (O_PROMOTE == order) )
                {
                    p = SkipSpaces(N1.GetToken(p, " \t", ch, TRIM_ALL));
                    if (0==stricmp("NEW", N1.GetData()))
                    {
                        isnew = TRUE;
                        p = SkipSpaces(N1.GetToken(p, " \t", ch, TRIM_ALL));
                    }
                    n1 = atol(N1.GetData());
                    if (isnew)
                        n1 = NEW_UNIT_ID(pLand, n1);

                    Dummy.Id = n1;
                    if (pLand->Units.Search(&Dummy, idx))
                    {
                        pUnit2 = (CUnit*)pLand->Units.At(idx);
                        p      = NULL;
                        p1     = NULL;

                        if (!pUnit->GetProperty(PRP_STRUCT_NAME, type, (const void *&)p, eNormal) )
                            SHOW_WARN(" - The unit is not inside a struct");
                        if (eCharPtr!=type)
                            SHOW_WARN(NOTSTRING << pUnit->Id << BUG);

                        if (!pUnit2->GetProperty(PRP_STRUCT_NAME, type, (const void *&)p1, eNormal) )
                            SHOW_WARN(" - The unit is not inside a struct");
                        if (eCharPtr!=type)
                            SHOW_WARN(NOTSTRING << pUnit2->Id << BUG);

                        if (0!=SafeCmp(p, p1))
                            SHOW_WARN(" - Units are not in the same struct");

                        if   (PE_OK!=pUnit ->SetProperty(PRP_STRUCT_OWNER, eCharPtr, "",  eNormal))
                            SHOW_WARN(NOSETUNIT << pUnit->Id << BUG);

                        if ( (PE_OK!=pUnit2->SetProperty(PRP_STRUCT_OWNER, eCharPtr, "",  eNormal)) ||

                             (PE_OK!=pUnit2->SetProperty(PRP_STRUCT_OWNER, eCharPtr, YES, eNormal)) )
                            SHOW_WARN(NOSETUNIT << pUnit2->Id << BUG);
                    }
                    else
                        SHOW_WARN(" - Can not find unit " << n1)
                }


                else if ( (SQ_MOVE==sequence) &&
                          (  (O_MOVE == order) ||
                             (O_SAIL == order) ||
                             (O_ADVANCE == order)
                          )
                        )
                {
                    int ID;

                    while (p)
                    {
                        p        = SkipSpaces(S1.GetToken(p, " \t", ch, TRIM_ALL));
                        for (i=0; i<(int)sizeof(Directions)/(int)sizeof(const char*); i++)
                            if (0==stricmp(S1.GetData(), Directions[i]))
                            {
                                switch (i%6)
                                {
                                case North     : Y -= 2;     break;
                                case Northeast : Y--; X++;   break;
                                case Southeast : Y++; X++;   break;
                                case South     : Y += 2;     break;
                                case Southwest : Y++; X--;   break;
                                case Northwest : Y--; X--;   break;
                                }

                                if (pLand->pPlane->Width > 0)
                                    if (X>pLand->pPlane->EastEdge)
                                        X = pLand->pPlane->WestEdge;
                                    else
                                        if (X<pLand->pPlane->WestEdge)
                                            X = pLand->pPlane->EastEdge;

                                ID = LandCoordToId(X,Y, pLand->pPlane->Id);
                                if (!pUnit->pMovement)
                                    pUnit->pMovement = new CLongColl;
                                pUnit->pMovement->Insert((void*)ID);

                                break;
                            }
                    }
                }

                // autotax flag must be removed in the very first pass, or land flag will be set and never removed                
                else if (SQ_MIN+1==sequence && O_AUTOTAX==order) 
                {
                    p = SkipSpaces(N1.GetToken(p, " \t", ch, TRIM_ALL));
                    n1 = atol(N1.GetData());
                    if (0==n1)
                        pUnit->Flags &= ~UNIT_TAXING;
                    else    
                        pUnit->Flags |=  UNIT_TAXING;
                }
                
                else if (SQ_GIVE ==sequence && O_TAX    ==order)
                {
                    pUnit->Flags |=  UNIT_TAXING;
                }
                
                else if (SQ_MAX-1==sequence && O_BUILD==order)
                {
                    pUnit->Flags |= UNIT_PRODUCING;
                }

                else if (SQ_MAX-1==sequence && O_PRODUCE==order)
                {
                    pUnit->Flags |= UNIT_PRODUCING;
                }

                // copy commands to the new unit
                if (pUnitMaster && (SQ_MAX-1==sequence) && (O_FORM != order) && !errors)
                    pUnit->Orders << Line << EOL_SCR;

            } // commands loop


            if  (SQ_TEACH==sequence)  // we have to collect all the students, then teach
                OrderProcess_Teach(pUnit);

            if ((pUnit->Flags & UNIT_TAXING) && !(pUnit->Flags & UNIT_GIVEN))
                pLand->Flags |= LAND_TAX_NEXT;
            if ((pUnit->Flags & UNIT_PRODUCING) && !(pUnit->Flags & UNIT_GIVEN))
                pLand->Flags |= LAND_TRADE_NEXT;
            
        }   // units loop
    }   // phases loop
    pLand->CalcStructsLoad();
    OrderErrFinalize();
}



//-------------------------------------------------------------

void CAtlaParser::OrderProcess_Teach(CUnit * pUnit)
{
    long          nstud = 0; // students count
    long          n1, n2;
    CUnit       * pUnit2;
    double        teach;
    CStr          ErrorLine(32);
    CStr          Line;
    BOOL          errors = FALSE;
    EValueType    type;
    BOOL          docontinue;
    int           i;


    do
    {
        if ( pUnit->pStudents && (pUnit->pStudents->Count() > 0)  )
        {

            if (!pUnit->StudyingSkill.IsEmpty())
                SHOW_ERR(" - can not teach and study at the same time");

            for (i=0; i<pUnit->pStudents->Count(); i++)
            {

                pUnit2 = (CUnit*)pUnit->pStudents->At(i);
                if (!pUnit2->GetProperty(PRP_MEN, type, (const void *&)n2, eNormal) )
                    n2 = 0; // should not happen
                nstud += n2;
            }
            if (!pUnit->GetProperty(PRP_MEN, type, (const void *&)n1, eNormal) )
                n1 = 0; // unlikely, too

            // n1 - number of teachers
            // nstud - number of students

            if (nstud>0)
            {
                pUnit ->Teaching = (double)nstud/n1;
                teach = (double)n1*STUDENTS_PER_TEACHER/nstud*30;

                for (i=0; i<pUnit->pStudents->Count(); i++)
                {
                    pUnit2           = (CUnit*)pUnit->pStudents->At(i);
                    pUnit2->Teaching += teach;
                }
            }
        }
    }
    while (FALSE);
}

//-------------------------------------------------------------

void CAtlaParser::RunOrders(CLand * pLand, const char * sCheckTeach)
{
    int         i, n;
    CPlane    * pPlane;

    if (!sCheckTeach)
        sCheckTeach = GetConfStringLoc(SZ_SECT_COMMON, SZ_KEY_CHECK_TEACH_LVL);

    if (pLand)
    {
        RunLandOrders(pLand, sCheckTeach);
    }
    else  // run orders for all lands
    {
        for (n=0; n<m_Planes.Count(); n++)
        {
            pPlane = (CPlane*)m_Planes.At(n);
            for (i=0; i<pPlane->Lands.Count(); i++)
            {
                pLand = (CLand*)pPlane->Lands.At(i);
                if (pLand)
                    RunLandOrders(pLand, sCheckTeach);
            }
        }
    }
}

//-------------------------------------------------------------

BOOL CAtlaParser::ApplyDefaultOrders(BOOL EmptyOnly)
{
    int           i;
    CUnit       * pUnit;
    CStr          NewOrder(32);
    CStr          OldOrder(32);
    const char  * pNew;
    const char  * pOld;
    BOOL          Exists;
    BOOL          Changed = FALSE;

    for (i=0; i<m_Units.Count(); i++)
    {
        pUnit = (CUnit*)m_Units.At(i);
        if (pUnit->IsOurs)
        {
            if (EmptyOnly)
            {
                pUnit->Orders.TrimRight(TRIM_ALL);
                if (pUnit->Orders.IsEmpty())
                {
                    pNew  = pUnit->DefOrders.GetData();
                    while (pNew)
                    {
                        pNew = NewOrder.GetToken(pNew, '\n', TRIM_ALL);
                        if ( (!NewOrder.IsEmpty()) && (';'!=NewOrder.GetData()[0]) )
                        {
                            if (!pUnit->Orders.IsEmpty())
                                pUnit->Orders << EOL_SCR;
                            pUnit->Orders << NewOrder;
                            Changed = TRUE;
                        }
                    }
                }
            }
            else
            {
                pNew  = pUnit->DefOrders.GetData();
                while (pNew)
                {
                    pNew = NewOrder.GetToken(pNew, '\n', TRIM_ALL);
                    if ( (!NewOrder.IsEmpty()) && (';'!=NewOrder.GetData()[0]) )
                    {
                        Exists = FALSE;
                        pOld   = pUnit->Orders.GetData();
                        while (pOld)
                        {
                            pOld = OldOrder.GetToken(pOld, '\n', TRIM_ALL);
                            if (0==stricmp(OldOrder.GetData(), NewOrder.GetData()))
                            {
                                Exists = TRUE;
                                break;
                            }
                        }
                        if (!Exists)
                        {
                            if (!pUnit->Orders.IsEmpty())
                                pUnit->Orders << EOL_SCR;
                            pUnit->Orders << NewOrder;
                            Changed = TRUE;

                        }

                    }
                }
            }
        }
    }
    if (Changed)
        RunOrders(NULL);

    return Changed;
}

//-------------------------------------------------------------

int  CAtlaParser::SetUnitProperty(CUnit * pUnit, const char * name, EValueType type, const void * value, EPropertyType proptype)
{
    int       i;
    CStrInt * pSI;

    if (!m_UnitPropertyNames.Search((void*)name, i))
    {
        m_UnitPropertyNames.Insert(strdup(name));
        pSI = new CStrInt(name, (int)type);
        if (!m_UnitPropertyTypes.Insert(pSI))
            delete pSI;
    }
    return pUnit->SetProperty(name, type, value, proptype);
}

//-------------------------------------------------------------

void WriteOneMageSkill(CStr & Line, const char * skill, CUnit * pUnit, const char * separator)
{
    CStr                S;
    EValueType          type;
    const void        * value;
    long                nlvl, ndays;
    int                 n;

    Line << separator;
    
    S << skill << PRP_SKILLS_POSTFIX;
    if (pUnit->GetProperty(S.GetData(), type, value, eNormal) && (eLong==type) )
    {

        nlvl = (long)value;
        ndays= 0;
        Line << "_" << nlvl;
        S = skill;
        S << PRP_SKILL_DAYS_POSTFIX;
        if (pUnit->GetProperty(S.GetData(), type, value, eNormal) && (eLong==type) )
        {
            ndays = long(value);
            for (n=1; n<=nlvl; n++)
                ndays -= n*30;
            if (ndays<0)
                ndays=0;
        }
        n = ndays/30;

        while (n-- > 0)

            Line << "+";

    }
    else
        Line << "_";

}

void CAtlaParser::WriteMagesCSV(const char * FName, BOOL vertical, const char * separator)
{
    CBaseCollById       Mages;
    CUnit             * pUnit;
    int                 idx;
    const char        * Foundations[3] = {"FORC_", "PATT_", "SPIR_"};
    EValueType          type;
    const void        * value;
    CStringSortColl     Skills;
    const char        * propname;
    int                 i, n, postlen;
    CStr                S, Line(64);
    CFileWriter         Dest;


    postlen = strlen(PRP_SKILLS_POSTFIX);
    for (idx=0; idx<m_Units.Count(); idx++)
    {
        pUnit = (CUnit*)m_Units.At(idx);

        if ( (pUnit->GetProperty(Foundations[0], type, value, eNormal) && (eLong==type)) ||
             (pUnit->GetProperty(Foundations[1], type, value, eNormal) && (eLong==type)) ||
             (pUnit->GetProperty(Foundations[2], type, value, eNormal) && (eLong==type)) )
        {
            // that is a mage, put him in a list
            Mages.Insert(pUnit);


            i        = 0;
            propname = pUnit->GetPropertyName(i);
            while (propname)
            {
                if (pUnit->GetProperty(propname, type, value, eNormal) && (eLong==type) )
                {
                    S = propname;
                    if (S.FindSubStrR(PRP_SKILLS_POSTFIX) == S.GetLength()-postlen)
                    {
                        S.DelSubStr(S.GetLength()-postlen, postlen);
                        if (!Skills.Search((void*)S.GetData(), n))
                            Skills.Insert(strdup(S.GetData()));
                    }
                }

                propname = pUnit->GetPropertyName(++i);
            }
        }
    }


    if (Dest.Open(FName))
    {
        Line.Empty();
        if (vertical)
        {
            Line << "Skill";
            for (i=0; i<Mages.Count(); i++)
            {
                pUnit = (CUnit*)Mages.At(i);
                Line << separator << pUnit->Id << " " << pUnit->Name;
            }
            Line << EOL_FILE;
            Dest.WriteBuf(Line.GetData(), Line.GetLength());

            for (i=0; i<Skills.Count(); i++)
            {
                Line.Empty();
                Line << (const char *)Skills.At(i);
                for (idx=0; idx<Mages.Count(); idx++)
                {
                    pUnit = (CUnit*)Mages.At(idx);
                    WriteOneMageSkill(Line, (const char *)Skills.At(i), pUnit, separator);
                }
                Line << EOL_FILE;
                Dest.WriteBuf(Line.GetData(), Line.GetLength());
            }
        }
        else
        {
            Line << "Id" << separator << "Name";
            for (i=0; i<Skills.Count(); i++)
                Line << separator << (const char *)Skills.At(i);
            Line << EOL_FILE;
            Dest.WriteBuf(Line.GetData(), Line.GetLength());


            for (idx=0; idx<Mages.Count(); idx++)
            {
                pUnit = (CUnit*)Mages.At(idx);
                Line.Empty();
                Line << pUnit->Id << separator << pUnit->Name;
                for (i=0; i<Skills.Count(); i++)
                    WriteOneMageSkill(Line, (const char *)Skills.At(i), pUnit, separator);

                Line << EOL_FILE;
                Dest.WriteBuf(Line.GetData(), Line.GetLength());
            }
        }
        Dest.Close();
    }




    Mages.DeleteAll();
    Skills.FreeAll();
}

//-------------------------------------------------------------



