/*
 * This source file is part of the Atlantis Little Helper program.
 * Copyright (C) 2001 Maxim Shariy.
 *
 * Atlantis Little Helper is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Atlantis Little Helper is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Atlantis Little Helper; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __AH_REP_PARSER_H__
#define __AH_REP_PARSER_H__

#include "data.h"

extern const char * Monthes[];

extern const char * EOL_MS   ;
extern const char * EOL_UNIX ;
extern const char * EOL_SCR  ;
extern const char * EOL_FILE ;

#define DEFAULT_PLANE "Overworld"

typedef void         ( * REPORT_ORDER_ERR_FUNC  )  (const char * msg, int msglen);
typedef long         ( * GET_STUDY_COST_FUNC    )  (const char * skill);
typedef long         ( * GET_STRUCTURE_ATTR_FUNC)  (const char * kind);
typedef const char * ( * GET_CONF_STRING_FUNC   )  (const char * section, const char * param);
typedef BOOL         ( * GET_ORDER_ID_FUNC      )  (const char * order, long & id);
typedef BOOL         ( * IS_TRADE_ITEM_FUNC     )  (const char * item);


enum         eDirection     { North=0, Northeast,   Southeast,   South,   Southwest,   Northwest };
extern int   ExitFlags [];
extern int   EntryFlags[];

enum {
    O_ADDRESS,
    O_ADVANCE,
    O_ARMOR,
    O_ASSASSINATE,
    O_ATTACK,
    O_AUTOTAX,
    O_AVOID,
    O_BEHIND,
    O_BUILD,
    O_BUY,
    O_CAST,
    O_CLAIM,
    O_COMBAT,
    O_CONSUME,
    O_DECLARE,
    O_DESCRIBE,
    O_DESTROY,
    O_ENDFORM,
    O_ENDTURN,
    O_ENTER,
    O_ENTERTAIN,
    O_EVICT,
    O_EXCHANGE,
    O_FACTION,
    O_FIND,
    O_FORGET,
    O_FORM,
    O_GIVE,
    O_GUARD,
    O_HOLD,
    O_LEAVE,
    O_MOVE,
    O_NAME,
    O_NOAID,
    O_NOCROSS,
    O_NOSPOILS,
    O_OPTION,
    O_PASSWORD,
    O_PILLAGE,
    O_PREPARE,
    O_PRODUCE,
    O_PROMOTE,
    O_QUIT,
    O_RESTART,
    O_REVEAL,
    O_SAIL,
    O_SELL,
    O_SHOW,
    O_SPOILS,
    O_STEAL,
    O_STUDY,
    O_TAX,
    O_TEACH,
    O_TURN,
    O_WEAPON,
    O_WITHDRAW,
    O_WORK,
    NORDERS
};


typedef struct SAVE_HEX_OPTIONS_STRUCT
{
    BOOL   SaveStructs;
    BOOL   AlwaysSaveImmobStructs;
    BOOL   SaveUnits;
    BOOL   SaveResources;
    long   WriteTurnNo; // Add turn number atlaclient style
} SAVE_HEX_OPTIONS;



//======================================================================

class CAtlaParser
{
public:
    CAtlaParser();
    CAtlaParser(REPORT_ORDER_ERR_FUNC     OrderErrFunc, 
                GET_STUDY_COST_FUNC       StudyCostFunc,
                GET_STRUCTURE_ATTR_FUNC   GetStructAttrFunc,
                RESOLVE_ALIAS_FUNC        ResolveAliasFunc,
                GET_ITEM_WEIGHTS          GetItemWeightsFunc,
                GET_CONF_STRING_FUNC      GetConfStringFunc,
                GET_ORDER_ID_FUNC         GetOrderIdFunc,
                IS_TRADE_ITEM_FUNC        IsTradeItemFunc
                );
    ~CAtlaParser();
    void       Clear();
    int        ParseRep(const char * FNameIn, BOOL Join, BOOL IsHistory);     // History is a rep!
    int        SaveOrders  (const char * FNameOut, const char * password, BOOL decorate, int factid);
    int        LoadOrders  (const char * FNameIn, int & FactionId);  // return an id of the order's faction
    void       RunOrders(CLand * pLand, const char * sCheckTeach = NULL);
    BOOL       ShareSilver(CUnit * pMainUnit);
    BOOL       GenOrdersTeach(CUnit * pMainUnit);
    BOOL       GenGiveEverything(CUnit * pFrom, long nTo);
    BOOL       DiscardJunkItems(CUnit * pUnit, const char * junk);
    BOOL       DetectSpies(CUnit * pUnit, long lonum, long hinum, long amount);
    BOOL       ApplyDefaultOrders(BOOL EmptyOnly);
    int        ParseCBDataFile(const char * FNameIn);
    void       WriteMagesCSV(const char * FName, BOOL vertical, const char * separator);
             
    CLand    * GetLand(int x, int y, int nPlane, BOOL AdjustForEdge=FALSE);
    CLand    * GetLand(long LandId);
    CLand    * GetLand(const char * landcoords); //  "48,52[,somewhere]"
    void       GetUnitList(CCollection * pResultColl, int x, int y, int z);
    void       CountMenForTheFaction(int FactionId);
    void       ComposeProductsLine(CLand * pLand, const char * eol, CStr & S);
    BOOL       LandStrCoordToId(const char * landcoords, long & id);
    void       ComposeLandStrCoord(CLand * pLand, CStr & LandStr);
    CFaction * GetFaction(int id);
    BOOL       SaveOneHex(CFileWriter & Dest, CLand * pLand, CPlane * pPlane, SAVE_HEX_OPTIONS * pOptions);
            
            
    int               m_CrntFactionId;
    CStr              m_CrntFactionPwd;
    CLongColl         m_OurFactions;
    CBaseObject       m_Events;
    CBaseObject       m_Errors;
            
    CBaseCollById     m_Factions;
    CBaseCollById     m_Units;
    CBaseColl         m_Planes;
    long              m_YearMon;    // Current year/month accumulated for all loaded files
    CStringSortColl   m_UnitPropertyNames;
    CStrIntColl       m_UnitPropertyTypes;
    CBaseColl         m_Skills;
    CBaseColl         m_Items;
    CBaseColl         m_Objects;
    CBaseCollByName   m_Battles;
    CBaseCollById     m_Gates;
            
    long              m_nCurLine; 
    long              m_GatesCount;
    int               m_ParseErr;
    BOOL              m_OrdersLoaded;
    CStr              m_FactionInfo;

protected:
    int          ParseFactionInfo(BOOL GetNo);
    int          ParseEvents(BOOL IsEvents=TRUE);
    int          ParseUnclSilver(CStr & Line);
    int          ParseAttitudes(CStr & Line);
    int          ParseTerrain (CLand * pMotherLand, int ExitDir, CStr & FirstLine, BOOL FullMode, CLand ** ppParsedLand);
    int          AnalyzeTerrain(CLand * pLand);
    void         CheckExit(CPlane * pPlane, int Direction, CLand * pLandSrc, CLand * pLandExit);
    int          ParseUnit(CStr & FirstLine);
    int          ParseStructure (CStr & FirstLine);
    int          ParseErrors();
    int          ParseLines();
    int          ParseOneEvent(CStr & EventLine, BOOL IsEvent);
    int          ApplyLandFlags();
    int          SetLandFlag(const char * p, long flag);
    int          SetLandFlag(long LandId, long flag);
    int          ParseBattles();
    int          ParseSkills();
    int          ParseItems();
    int          ParseObjects();
    void         SetExitFlags();
    int          SetUnitProperty(CUnit * pUnit, const char * name, EValueType type, const void * value, EPropertyType proptype);
    void         LoadOrders  (CFileReader & F, int FactionId, BOOL GetComments);
    const char * ReadPropertyName(const char * src, CStr & Name);
    void         StoreBattle(CStr & s);
    void         SetShaftLinks();

    CUnit      * MakeUnit(long Id);
//    void         MakeUnitAndGetOld(long Id, CUnit *& pUnit, CUnit *& pUnitOld, int & idxold);
    CPlane     * MakePlane(const char * planename);

    BOOL         ReadNextLine(CStr & s);
    void         PutLineBack (CStr & s);
                
    void         OrderErr(int Severity, int UnitId, const char * Msg);
    void         OrderErrFinalize();
    void         RunLandOrders(CLand * pLand, const char * sCheckTeach = NULL);
    void         OrderProcess_Teach(CUnit * pUnit);
                
    int          ParseCBHex   (const char * FirstLine);
    int          ParseCBStruct(const char * FirstLine);

    CFileReader    * m_pSource;

    CStringSortColl  m_TaxLandStrs;
    CStringSortColl  m_TradeLandStrs;
    CStringSortColl  m_BattleLandStrs;
    CLongSortColl    m_TradeUnitIds;
    CBaseCollByName  m_PlanesNamed;
    CBaseColl        m_LandsToBeLinked;

    CLand          * m_pCurLand   ;
    CStruct        * m_pCurStruct ;
    int              m_NextStructId;
    CStr             m_sOrderErrors;
    long             m_MaxSkillDays;
    BOOL             m_JoiningRep;  // joining an allies' report
    BOOL             m_IsHistory;   // parsing history file
    long             m_CurYearMon; // Year/month for the file being loaded


};



#endif

