/*
 * This source file is part of the Atlantis Little Helper program.
 * Copyright (C) 2001 Maxim Shariy.
 *
 * Atlantis Little Helper is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Atlantis Little Helper is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Atlantis Little Helper; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>
#include "consts_ah.h"
#include "data.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


DefaultConfigRec DefaultConfig[] =
{
#if defined(_WIN32)
    {SZ_SECT_COMMON            ,     SZ_KEY_EOL            ,     SZ_EOL_MS           },
#else
    {SZ_SECT_COMMON            ,     SZ_KEY_EOL            ,     SZ_EOL_UNIX         },
#endif
    {SZ_SECT_COMMON            ,     SZ_KEY_HATCH_UNVISITED,     "1"                 },
    {SZ_SECT_COMMON            ,     SZ_KEY_DEFAULT_EMPTY_ONLY,  "1"                 },
    {SZ_SECT_COMMON            ,     SZ_KEY_REP_CACHE_COUNT,     "8"                 },
    {SZ_SECT_COMMON            ,     SZ_KEY_LOAD_ORDER     ,     "1"                 },
    {SZ_SECT_COMMON            ,     SZ_KEY_LOAD_REP       ,     "1"                 },
    {SZ_SECT_COMMON            ,     SZ_KEY_RCLICK_CENTERS ,     "1"                 },
    {SZ_SECT_COMMON            ,     SZ_KEY_HEX_SIZE_LIST  ,     "4,8,18,24,32,48"   },
    {SZ_SECT_COMMON            ,     SZ_KEY_MIN_SEL_MEN    ,     "5"                 },
    {SZ_SECT_COMMON            ,     SZ_KEY_LAYOUT         ,     "1"                 },
    {SZ_SECT_COMMON            ,     SZ_KEY_SPY_DETECT_LO  ,     "100"               },
    {SZ_SECT_COMMON            ,     SZ_KEY_SPY_DETECT_HI  ,     "5000"              },
    {SZ_SECT_COMMON            ,     SZ_KEY_SPY_DETECT_AMT ,     "1"                 },
    {SZ_SECT_COMMON            ,     SZ_KEY_BRIGHT_DELTA   ,     "8"                 },
    {SZ_SECT_COMMON            ,     SZ_KEY_DECORATE_ORDERS,     "1"                 },
    {SZ_SECT_COMMON            ,     SZ_KEY_MOVEMENTS      ,     "None,Walk,Ride,Fly"},
    {SZ_SECT_COMMON            ,     SZ_KEY_ORD_MONTH_LONG ,     "BUILD,ENTERTAIN,PRODUCE,STUDY,TEACH,WORK,MOVE,ADVANCE,SAIL"},
    {SZ_SECT_COMMON            ,     SZ_KEY_ORD_DUPLICATABLE,    "TEACH,MOVE,ADVANCE,SAIL"},
    {SZ_SECT_COMMON            ,     SZ_KEY_CHECK_TEACH_LVL,     "1"                 },
    {SZ_SECT_COMMON            ,     SZ_KEY_MAX_SKILL_DAYS ,     "450"               },
    {SZ_SECT_COMMON            ,     SZ_KEY_CHECK_OUTPUT_LIST,   "1"                 },
    {SZ_SECT_COMMON            ,     SZ_KEY_PWD_READ       ,     "1"                 },
    {SZ_SECT_COMMON            ,     SZ_KEY_VALID_ORDERS   ,     "address,advance,armor,assassinate,attack,autotax,avoid,behind,build,buy,cast,claim,combat,consume,declare,describe,destroy,end,endturn,enter,entertain,evict,exchange,faction,find,forget,form,give,guard,hold,leave,move,name,noaid,nocross,nospoils,option,password,pillage,prepare,produce,promote,quit,restart,reveal,sail,sell,show,spoils,steal,study,tax,teach,turn,weapon,withdraw,work"     },
    

    {SZ_SECT_COLORS            ,     SZ_KEY_COLOR_RED      ,     "255,   0,   0"     },
    {SZ_SECT_COLORS            ,     SZ_KEY_COLOR_BLACK    ,     "  0,   0,   0"     },
    {SZ_SECT_COLORS            ,     SZ_KEY_MAP_UNKNOWN    ,     "192, 192, 192"     },
    {SZ_SECT_COLORS            ,     SZ_KEY_MAP_GRID       ,     "128, 128, 128"     },
    {SZ_SECT_COLORS            ,     SZ_KEY_MAP_GRID_SEL   ,     "255,   0,   0"     },
    {SZ_SECT_COLORS            ,     SZ_KEY_MAP_ROAD       ,     "112, 128, 144"     },
    {SZ_SECT_COLORS            ,     SZ_KEY_MAP_ROAD_BAD   ,     "159, 182, 205"     },
    {SZ_SECT_COLORS            ,     SZ_KEY_MAP_DESERT     ,     "224, 164,  56"     },
    {SZ_SECT_COLORS            ,     SZ_KEY_MAP_FOREST     ,     " 72, 200,  72"     },
    {SZ_SECT_COLORS            ,     SZ_KEY_MAP_JUNGLE     ,     "  0, 128,   0"     },
    {SZ_SECT_COLORS            ,     SZ_KEY_MAP_MOUNTAIN   ,     "188,  96,   0"     },
    {SZ_SECT_COLORS            ,     SZ_KEY_MAP_OCEAN      ,     "  0,   0, 255"     },
    {SZ_SECT_COLORS            ,     SZ_KEY_MAP_PLAINS     ,     "255, 232, 168"     },
    {SZ_SECT_COLORS            ,     SZ_KEY_MAP_SWAMP      ,     "168, 168,  84"     },
    {SZ_SECT_COLORS            ,     SZ_KEY_MAP_TUNDRA     ,     "184, 200, 224"     },
    {SZ_SECT_COLORS            ,     SZ_KEY_MAP_NEXUS      ,     "  0, 147, 217"     },
    {SZ_SECT_COLORS            ,     SZ_KEY_MAP_CAVERN     ,     "128, 128, 192"     },
    {SZ_SECT_COLORS            ,     SZ_KEY_MAP_TUNNELS    ,     " 65, 146, 137"     },
    {SZ_SECT_COLORS            ,     SZ_KEY_MAP_UNDERFOREST,     "116, 158,  44"     },
    
    {SZ_SECT_COLORS            ,     SZ_KEY_MAP_FLAG_1     ,     "255,   0,   0"     },
    {SZ_SECT_COLORS            ,     SZ_KEY_MAP_FLAG_2     ,     "128, 255,   0"     },
    {SZ_SECT_COLORS            ,     SZ_KEY_MAP_FLAG_3     ,     "0,     0, 128"     },


#if defined(_WIN32)
    {SZ_SECT_MAP_PANE          ,     SZ_KEY_DASH_BAD_ROADS ,     "0"                 },
#else
    {SZ_SECT_MAP_PANE          ,     SZ_KEY_DASH_BAD_ROADS ,     "1"                 },
#endif
    {SZ_SECT_MAP_PANE          ,     SZ_KEY_STATE          ,     "0"                 },
    {SZ_SECT_MAP_PANE          ,     SZ_KEY_WALL_WIDTH     ,     "3"                 },
    {SZ_SECT_MAP_PANE          ,     SZ_KEY_FLAG_WIDTH     ,     "1"                 },
    {SZ_SECT_MAP_PANE          ,     SZ_KEY_ROAD_WIDTH     ,     "4"                 },
    
    {SZ_SECT_WND_MAP_2_WIN     ,     SZ_KEY_X1             ,     "2"                 },
    {SZ_SECT_WND_MAP_2_WIN     ,     SZ_KEY_Y1             ,     "2"                 },
    {SZ_SECT_WND_MAP_2_WIN     ,     SZ_KEY_X2             ,     "400"               },
    {SZ_SECT_WND_MAP_2_WIN     ,     SZ_KEY_Y2             ,     "600"               },
    {SZ_SECT_WND_MAP_2_WIN     ,     SZ_KEY_HEIGHT_0       ,     "300"               },
    {SZ_SECT_WND_MAP_2_WIN     ,     SZ_KEY_HEIGHT_1       ,     "200"               },
    
    {SZ_SECT_WND_UNITS_2_WIN   ,     SZ_KEY_X1             ,     "402"               },
    {SZ_SECT_WND_UNITS_2_WIN   ,     SZ_KEY_Y1             ,     "2"                 },
    {SZ_SECT_WND_UNITS_2_WIN   ,     SZ_KEY_X2             ,     "800"               },
    {SZ_SECT_WND_UNITS_2_WIN   ,     SZ_KEY_Y2             ,     "300"               },
    {SZ_SECT_WND_UNITS_2_WIN   ,     SZ_KEY_HEIGHT_0       ,     "100"               },
    {SZ_SECT_WND_UNITS_2_WIN   ,     SZ_KEY_HEIGHT_1       ,     "100"               },
    {SZ_SECT_WND_UNITS_2_WIN   ,     SZ_KEY_HEIGHT_2       ,     "100"               },
    {SZ_SECT_WND_UNITS_2_WIN   ,     SZ_KEY_WIDTH_0        ,     "200"               },
	{SZ_SECT_WND_UNITS_2_WIN   ,     SZ_KEY_OPEN           ,     "1"                 },
    {SZ_SECT_WND_UNITS_2_WIN   ,     SZ_KEY_SORT1          ,     "faction"           },
    {SZ_SECT_WND_UNITS_2_WIN   ,     SZ_KEY_SORT2          ,     "name"              },
                      

    {SZ_SECT_WND_MAP_3_WIN     ,     SZ_KEY_X1             ,     "2"                 },
    {SZ_SECT_WND_MAP_3_WIN     ,     SZ_KEY_Y1             ,     "2"                 },
    {SZ_SECT_WND_MAP_3_WIN     ,     SZ_KEY_X2             ,     "400"               },
    {SZ_SECT_WND_MAP_3_WIN     ,     SZ_KEY_Y2             ,     "400"               },
    
    {SZ_SECT_WND_EDITS_3_WIN   ,     SZ_KEY_X1             ,     "402"               },
    {SZ_SECT_WND_EDITS_3_WIN   ,     SZ_KEY_Y1             ,     "2"                 },
    {SZ_SECT_WND_EDITS_3_WIN   ,     SZ_KEY_X2             ,     "800"               },
    {SZ_SECT_WND_EDITS_3_WIN   ,     SZ_KEY_Y2             ,     "400"               },
    {SZ_SECT_WND_EDITS_3_WIN   ,     SZ_KEY_HEIGHT_0       ,     "100"               },
    {SZ_SECT_WND_EDITS_3_WIN   ,     SZ_KEY_HEIGHT_1       ,     "100"               },
    {SZ_SECT_WND_EDITS_3_WIN   ,     SZ_KEY_HEIGHT_2       ,     "100"               },
    {SZ_SECT_WND_EDITS_3_WIN   ,     SZ_KEY_WIDTH_0        ,     "200"               },
	{SZ_SECT_WND_EDITS_3_WIN   ,     SZ_KEY_OPEN           ,     "1"                 },
                      
    {SZ_SECT_WND_UNITS_3_WIN   ,     SZ_KEY_X1             ,     "2"                 },
    {SZ_SECT_WND_UNITS_3_WIN   ,     SZ_KEY_Y1             ,     "402"               },
    {SZ_SECT_WND_UNITS_3_WIN   ,     SZ_KEY_X2             ,     "800"               },
    {SZ_SECT_WND_UNITS_3_WIN   ,     SZ_KEY_Y2             ,     "600"               },
	{SZ_SECT_WND_UNITS_3_WIN   ,     SZ_KEY_OPEN           ,     "1"                 },
    {SZ_SECT_WND_UNITS_3_WIN   ,     SZ_KEY_SORT1          ,     "faction"           },
    {SZ_SECT_WND_UNITS_3_WIN   ,     SZ_KEY_SORT2          ,     "name"              },

    {SZ_SECT_WND_UNITS_FLTR    ,     SZ_KEY_X1             ,     "200"               },
    {SZ_SECT_WND_UNITS_FLTR    ,     SZ_KEY_Y1             ,     "300"               },
    {SZ_SECT_WND_UNITS_FLTR    ,     SZ_KEY_X2             ,     "500"               },
    {SZ_SECT_WND_UNITS_FLTR    ,     SZ_KEY_Y2             ,     "500"               },
	{SZ_SECT_WND_UNITS_FLTR    ,     SZ_KEY_OPEN           ,     "0"                 },
    
    {SZ_SECT_WND_DESCR_LIST    ,     SZ_KEY_X1             ,     "80"                },
    {SZ_SECT_WND_DESCR_LIST    ,     SZ_KEY_Y1             ,     "150"               },
    {SZ_SECT_WND_DESCR_LIST    ,     SZ_KEY_X2             ,     "400"               },
    {SZ_SECT_WND_DESCR_LIST    ,     SZ_KEY_Y2             ,     "380"               },

    {SZ_SECT_WND_DESCR_ONE     ,     SZ_KEY_X1             ,     "80"                 },
    {SZ_SECT_WND_DESCR_ONE     ,     SZ_KEY_Y1             ,     "200"               },
    {SZ_SECT_WND_DESCR_ONE     ,     SZ_KEY_X2             ,     "450"               },
    {SZ_SECT_WND_DESCR_ONE     ,     SZ_KEY_Y2             ,     "380"               },


    {SZ_SECT_WND_MSG           ,     SZ_KEY_X1             ,     "200"               },
    {SZ_SECT_WND_MSG           ,     SZ_KEY_Y1             ,     "100"               },
    {SZ_SECT_WND_MSG           ,     SZ_KEY_X2             ,     "500"               },
    {SZ_SECT_WND_MSG           ,     SZ_KEY_Y2             ,     "500"               },

    {SZ_SECT_WND_LST_COLEDIT_DLG,     SZ_KEY_X1             ,     "250"               },
    {SZ_SECT_WND_LST_COLEDIT_DLG,     SZ_KEY_Y1             ,     "150"               },
    {SZ_SECT_WND_LST_COLEDIT_DLG,     SZ_KEY_X2             ,     "750"               },
    {SZ_SECT_WND_LST_COLEDIT_DLG,     SZ_KEY_Y2             ,     "600"               },
    
    
    {SZ_SECT_FONTS_2           ,     "0"                   ,     SZ_DEFAULT_FONT_PROP   },
    {SZ_SECT_FONTS_2           ,     "1"                   ,     SZ_DEFAULT_FONT_MONO   },
    {SZ_SECT_FONTS_2           ,     "2"                   ,     SZ_DEFAULT_FONT_PROP   },
    {SZ_SECT_FONTS_2           ,     "3"                   ,     SZ_DEFAULT_FONT_PROP   },
    {SZ_SECT_FONTS_2           ,     "4"                   ,     SZ_DEFAULT_FONT_PROP   },
    {SZ_SECT_FONTS_2           ,     "5"                   ,     SZ_DEFAULT_FONT_PROP   },
    {SZ_SECT_FONTS_2           ,     "6"                   ,     SZ_DEFAULT_FONT_PROP   },
    
    
    {SZ_SECT_LIST_COL_CURRENT  , SZ_KEY_LIS_COL_UNITS_HEX  ,     SZ_SECT_LIST_COL_UNIT_DEF                },
    {SZ_SECT_LIST_COL_CURRENT  , SZ_KEY_LIS_COL_UNITS_FILTER,    SZ_SECT_LIST_COL_UNIT_FLTR_DEF           },

    {SZ_SECT_LIST_COL_UNIT_DEF ,     "000"                 ,     "30, 1, " PRP_FACTION_ID  ", Id F"       },    
    {SZ_SECT_LIST_COL_UNIT_DEF ,     "001"                 ,     "80, 0, " PRP_FACTION     ", Fact"       },    
    {SZ_SECT_LIST_COL_UNIT_DEF ,     "002"                 ,     "36, 1, " PRP_ID          ", Id"         },    
    {SZ_SECT_LIST_COL_UNIT_DEF ,     "003"                 ,     "90, 0, " PRP_NAME        ", Unit"       },    
    {SZ_SECT_LIST_COL_UNIT_DEF ,     "004"                 ,     "32, 1, " PRP_MEN         ", Men"        },    
    {SZ_SECT_LIST_COL_UNIT_DEF ,     "005"                 ,     "38, 1, " PRP_SILVER      ", Silv"       },    
    {SZ_SECT_LIST_COL_UNIT_DEF ,     "006"                 ,     "31, 1, " PRP_SKILLS      ", Skill"      },    
    {SZ_SECT_LIST_COL_UNIT_DEF ,     "007"                 ,     "37, 1, " PRP_STUFF       ", Items"      },    
    {SZ_SECT_LIST_COL_UNIT_DEF ,     "008"                 ,     "35, 1, " PRP_HORS        ", Hors"       },    
    {SZ_SECT_LIST_COL_UNIT_DEF ,     "009"                 ,     "38, 1, " PRP_ARMS        ", Arms"       },    
    {SZ_SECT_LIST_COL_UNIT_DEF ,     "010"                 ,     "65, 0, " PRP_STRUCT_NAME ", Struct"     },    
    {SZ_SECT_LIST_COL_UNIT_DEF ,     "011"                 ,     "37, 0, " PRP_STRUCT_OWNER", Own"        },    
    {SZ_SECT_LIST_COL_UNIT_DEF ,     "012"                 ,     "42, 0, " PRP_TEACHING    ", Teach"      },    
    {SZ_SECT_LIST_COL_UNIT_DEF ,     "013"                 ,     "77, 0, " PRP_ORDERS      ", Orders"     },    
    {SZ_SECT_LIST_COL_UNIT_DEF ,     "014"                 ,     "99, 0, " PRP_COMMENTS    ", Comments"   },
    
    {SZ_SECT_LIST_COL_UNIT_FLTR_DEF  ,     "000"           ,     "30, 1, " PRP_FACTION_ID  ", Id F"       },    
    {SZ_SECT_LIST_COL_UNIT_FLTR_DEF  ,     "001"           ,     "80, 0, " PRP_FACTION     ", Fact"       },    
    {SZ_SECT_LIST_COL_UNIT_FLTR_DEF  ,     "002"           ,     "36, 1, " PRP_ID          ", Id"         },    
    {SZ_SECT_LIST_COL_UNIT_FLTR_DEF  ,     "003"           ,     "90, 0, " PRP_NAME        ", Unit"       },    
    {SZ_SECT_LIST_COL_UNIT_FLTR_DEF  ,     "004"           ,     "31, 1, " PRP_SKILLS      ", Skill"      },    


    {SZ_SECT_UNITPROP_GROUPS   ,     PRP_MEN               ,     GRP_MEN                                  },
    {SZ_SECT_UNITPROP_GROUPS   ,     PRP_SKILLS            ,     GRP_SKILLS                               },
    {SZ_SECT_UNITPROP_GROUPS   ,     PRP_MAG_SKILLS        ,     GRP_MAG_SKILLS                           },
    {SZ_SECT_UNITPROP_GROUPS   ,     PRP_STUFF             ,     GRP_STUFF                                },
    {SZ_SECT_UNITPROP_GROUPS   ,     PRP_HORS              ,     GRP_HORS                                 },
    {SZ_SECT_UNITPROP_GROUPS   ,     PRP_ARMS              ,     GRP_ARMS                                 },
    {SZ_SECT_UNITPROP_GROUPS   ,     PRP_MAG_ITEMS         ,     GRP_MAG_ITEMS                            },
    {SZ_SECT_UNITPROP_GROUPS   ,     PRP_JUNK_ITEMS        ,     GRP_JUNK_ITEMS                           },
    {SZ_SECT_UNITPROP_GROUPS   ,     PRP_TRADE_ITEMS       ,     GRP_TRADE_ITEMS                          },

    

    {SZ_SECT_STRUCTS           ,     STRUCT_GATE           ,     SZ_ATTR_STRUCT_HIDDEN "," SZ_ATTR_STRUCT_GATE },
    {SZ_SECT_STRUCTS           ,     "Longboat"            ,     SZ_ATTR_STRUCT_MOBILE                    },
    {SZ_SECT_STRUCTS           ,     "Clipper"             ,     SZ_ATTR_STRUCT_MOBILE                    },
    {SZ_SECT_STRUCTS           ,     "Galleon"             ,     SZ_ATTR_STRUCT_MOBILE                    },
    {SZ_SECT_STRUCTS           ,     "Balloon"             ,     SZ_ATTR_STRUCT_MOBILE                    },
    {SZ_SECT_STRUCTS           ,     "Armored Galleon"     ,     SZ_ATTR_STRUCT_MOBILE                    },

    {SZ_SECT_STRUCTS           ,     "Shaft"               ,     SZ_ATTR_STRUCT_SHAFT                     },
    {SZ_SECT_STRUCTS           ,     "Road N"              ,     SZ_ATTR_STRUCT_ROAD_N                    },
    {SZ_SECT_STRUCTS           ,     "Road NE"             ,     SZ_ATTR_STRUCT_ROAD_NE                   },
    {SZ_SECT_STRUCTS           ,     "Road SE"             ,     SZ_ATTR_STRUCT_ROAD_SE                   },
    {SZ_SECT_STRUCTS           ,     "Road S"              ,     SZ_ATTR_STRUCT_ROAD_S                    },
    {SZ_SECT_STRUCTS           ,     "Road SW"             ,     SZ_ATTR_STRUCT_ROAD_SW                   },
    {SZ_SECT_STRUCTS           ,     "Road NW"             ,     SZ_ATTR_STRUCT_ROAD_NW                   },
                                                                 
    
    
    {SZ_SECT_STUDY_COST        ,     "MINI"                ,     "10 "                                    },
    {SZ_SECT_STUDY_COST        ,     "LUMB"                ,     "10 "                                    },
    {SZ_SECT_STUDY_COST        ,     "QUAR"                ,     "10 "                                    },
    {SZ_SECT_STUDY_COST        ,     "HUNT"                ,     "10 "                                    },
    {SZ_SECT_STUDY_COST        ,     "FISH"                ,     "10 "                                    },
    {SZ_SECT_STUDY_COST        ,     "HERB"                ,     "10 "                                    },
    {SZ_SECT_STUDY_COST        ,     "HORS"                ,     "10 "                                    },
    {SZ_SECT_STUDY_COST        ,     "WEAP"                ,     "10 "                                    },
    {SZ_SECT_STUDY_COST        ,     "ARMO"                ,     "10 "                                    },
    {SZ_SECT_STUDY_COST        ,     "CARP"                ,     "10 "                                    },
    {SZ_SECT_STUDY_COST        ,     "BUIL"                ,     "10 "                                    },
    {SZ_SECT_STUDY_COST        ,     "SHIP"                ,     "10 "                                    },
    {SZ_SECT_STUDY_COST        ,     "ENTE"                ,     "10 "                                    },
    {SZ_SECT_STUDY_COST        ,     "TACT"                ,     "200"                                    },
    {SZ_SECT_STUDY_COST        ,     "COMB"                ,     "10 "                                    },
    {SZ_SECT_STUDY_COST        ,     "RIDI"                ,     "10 "                                    },
    {SZ_SECT_STUDY_COST        ,     "XBOW"                ,     "10 "                                    },
    {SZ_SECT_STUDY_COST        ,     "LBOW"                ,     "10 "                                    },
    {SZ_SECT_STUDY_COST        ,     "STEA"                ,     "50 "                                    },
    {SZ_SECT_STUDY_COST        ,     "OBSE"                ,     "50 "                                    },
    {SZ_SECT_STUDY_COST        ,     "HEAL"                ,     "10 "                                    },
    {SZ_SECT_STUDY_COST        ,     "SAIL"                ,     "10 "                                    },
    {SZ_SECT_STUDY_COST        ,     "FARM"                ,     "10 "                                    },
    {SZ_SECT_STUDY_COST        ,     "RANC"                ,     "10 "                                    },
    {SZ_SECT_STUDY_COST        ,     "FORC"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "PATT"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "SPIR"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "FIRE"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "EQUA"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "FSHI"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "ESHI"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "SSHI"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "MHEA"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "GATE"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "FARS"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "TELE"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "PORT"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "MIND"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "WEAT"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "SWIN"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "SSTO"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "STOR"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "CALL"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "CLEA"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "EART"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "WOLF"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "BIRD"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "DRAG"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "NECR"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "SUSK"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "RAIS"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "SULI"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "FEAR"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "SBLA"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "BUND"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "DEMO"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "SUIM"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "SUDE"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "SUBA"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "BDEM"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "ILLU"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "PHEN"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "PHBE"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "PHUN"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "PHDE"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "INVI"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "TRUE"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "DISP"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "ARTI"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "CRRI"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "CRCL"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "CRSF"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "CRSL"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "CRTA"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "CRPA"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "CRRU"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "CRSH"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "CRMA"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "ENGR"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "CGAT"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "ESWO"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "EARM"                ,     "100"                                    },
    {SZ_SECT_STUDY_COST        ,     "CPOR"                ,     "100"                                    },


    {SZ_SECT_ALIAS             , "leader"                         , "LEAD"                                },
    {SZ_SECT_ALIAS             , "viking"                         , "VIKI"                                },
    {SZ_SECT_ALIAS             , "barbarian"                      , "BARB"                                },
    {SZ_SECT_ALIAS             , "plainsman"                      , "PLAI"                                },
    {SZ_SECT_ALIAS             , "eskimo"                         , "ESKI"                                },
    {SZ_SECT_ALIAS             , "nomad"                          , "NOMA"                                },
    {SZ_SECT_ALIAS             , "tribesman"                      , "TMAN"                                },
    {SZ_SECT_ALIAS             , "darkman"                        , "DMAN"                                },
    {SZ_SECT_ALIAS             , "wood_elf"                       , "WELF"                                },
    {SZ_SECT_ALIAS             , "sea_elf"                        , "SELF"                                },
    {SZ_SECT_ALIAS             , "high_elf"                       , "HELF"                                },
    {SZ_SECT_ALIAS             , "tribal_elf"                     , "TELF"                                },
    {SZ_SECT_ALIAS             , "ice_dwarf"                      , "IDWA"                                },
    {SZ_SECT_ALIAS             , "hill_dwarf"                     , "HDWA"                                },
    {SZ_SECT_ALIAS             , "under_dwarf"                    , "UDWA"                                },
    {SZ_SECT_ALIAS             , "desert_dwarf"                   , "DDWA"                                },
    {SZ_SECT_ALIAS             , "horse"                          , "HORS"                                },
    {SZ_SECT_ALIAS             , "sword"                          , "SWOR"                                },
    {SZ_SECT_ALIAS             , "crossbow"                       , "XBOW"                                },
    {SZ_SECT_ALIAS             , "longbow"                        , "LBOW"                                },
    {SZ_SECT_ALIAS             , "wagon"                          , "WAGO"                                },
    {SZ_SECT_ALIAS             , "ironwood"                       , "IRWD"                                },
    {SZ_SECT_ALIAS             , "winged_horse"                   , "WING"                                },
    {SZ_SECT_ALIAS             , "floater_hide"                   , "FLOA"                                },
    {SZ_SECT_ALIAS             , "mithril_sword"                  , "MSWO"                                },
    {SZ_SECT_ALIAS             , "double_bow"                     , "DBOW"                                },
    {SZ_SECT_ALIAS             , "ring_of_invisibility"           , "RING"                                },
    {SZ_SECT_ALIAS             , "cloak_of_invulnerability"       , "CLOA"                                },
    {SZ_SECT_ALIAS             , "staff_of_fire"                  , "STAF"                                },
    {SZ_SECT_ALIAS             , "staff_of_lightning"             , "STAL"                                },
    {SZ_SECT_ALIAS             , "amulet_of_true_seeing"          , "AMTS"                                },
    {SZ_SECT_ALIAS             , "amulet_of_protection"           , "AMPR"                                },
    {SZ_SECT_ALIAS             , "runesword"                      , "RUNE"                                },
    {SZ_SECT_ALIAS             , "shieldstone"                    , "SHST"                                },
    {SZ_SECT_ALIAS             , "magic_carpet"                   , "CARP"                                },
    {SZ_SECT_ALIAS             , "portal"                         , "PORT"                                },
    {SZ_SECT_ALIAS             , "peasant"                        , "PEAS"                                },
    {SZ_SECT_ALIAS             , "leaders"                        , "LEAD"                                },
    {SZ_SECT_ALIAS             , "vikings"                        , "VIKI"                                },
    {SZ_SECT_ALIAS             , "barbarians"                     , "BARB"                                },
    {SZ_SECT_ALIAS             , "plainsmen"                      , "PLAI"                                },
    {SZ_SECT_ALIAS             , "eskimos"                        , "ESKI"                                },
    {SZ_SECT_ALIAS             , "nomads"                         , "NOMA"                                },
    {SZ_SECT_ALIAS             , "tribesmen"                      , "TMAN"                                },
    {SZ_SECT_ALIAS             , "darkmen"                        , "DMAN"                                },
    {SZ_SECT_ALIAS             , "wood_elves"                     , "WELF"                                },
    {SZ_SECT_ALIAS             , "sea_elves"                      , "SELF"                                },
    {SZ_SECT_ALIAS             , "high_elves"                     , "HELF"                                },
    {SZ_SECT_ALIAS             , "tribal_elves"                   , "TELF"                                },
    {SZ_SECT_ALIAS             , "ice_dwarves"                    , "IDWA"                                },
    {SZ_SECT_ALIAS             , "hill_dwarves"                   , "HDWA"                                },
    {SZ_SECT_ALIAS             , "under_dwarves"                  , "UDWA"                                },
    {SZ_SECT_ALIAS             , "desert_dwarves"                 , "DDWA"                                },
    {SZ_SECT_ALIAS             , "orcs"                           , "ORC"                                 },
    {SZ_SECT_ALIAS             , "silver"                         , "SILV"                                },
    {SZ_SECT_ALIAS             , "grain"                          , "GRAI"                                },
    {SZ_SECT_ALIAS             , "livestock"                      , "LIVE"                                },
    {SZ_SECT_ALIAS             , "stone"                          , "STON"                                },
    {SZ_SECT_ALIAS             , "furs"                           , "FUR"                                 },
    {SZ_SECT_ALIAS             , "herbs"                          , "HERB"                                },
    {SZ_SECT_ALIAS             , "horses"                         , "HORS"                                },
    {SZ_SECT_ALIAS             , "swords"                         , "SWOR"                                },
    {SZ_SECT_ALIAS             , "crossbows"                      , "XBOW"                                },
    {SZ_SECT_ALIAS             , "longbows"                       , "LBOW"                                },
    {SZ_SECT_ALIAS             , "chain_armor"                    , "CARM"                                },
    {SZ_SECT_ALIAS             , "plate_armor"                    , "PARM"                                },
    {SZ_SECT_ALIAS             , "wagons"                         , "WAGO"                                },
    {SZ_SECT_ALIAS             , "mithril"                        , "MITH"                                },
    {SZ_SECT_ALIAS             , "ironwood"                       , "IRWD"                                },
    {SZ_SECT_ALIAS             , "winged_horses"                  , "WING"                                },
    {SZ_SECT_ALIAS             , "floater_hides"                  , "FLOA"                                },
    {SZ_SECT_ALIAS             , "rootstone"                      , "ROOT"                                },
    {SZ_SECT_ALIAS             , "mithril_swords"                 , "MSWO"                                },
    {SZ_SECT_ALIAS             , "mithril_armor"                  , "MARM"                                },
    {SZ_SECT_ALIAS             , "double_bows"                    , "DBOW"                                },
    {SZ_SECT_ALIAS             , "ivory"                          , "IVOR"                                },
    {SZ_SECT_ALIAS             , "pearls"                         , "PEAR"                                },
    {SZ_SECT_ALIAS             , "jewelry"                        , "JEWE"                                },
    {SZ_SECT_ALIAS             , "figurines"                      , "FIGU"                                },
    {SZ_SECT_ALIAS             , "tarot_cards"                    , "TARO"                                },
    {SZ_SECT_ALIAS             , "caviar"                         , "CAVI"                                },
    {SZ_SECT_ALIAS             , "spices"                         , "SPIC"                                },
    {SZ_SECT_ALIAS             , "chocolate"                      , "CHOC"                                },
    {SZ_SECT_ALIAS             , "truffles"                       , "TRUF"                                },
    {SZ_SECT_ALIAS             , "vodka"                          , "VODK"                                },
    {SZ_SECT_ALIAS             , "roses"                          , "ROSE"                                },
    {SZ_SECT_ALIAS             , "perfume"                        , "PERF"                                },
    {SZ_SECT_ALIAS             , "velvet"                         , "VELV"                                },
    {SZ_SECT_ALIAS             , "cashmere"                       , "CASH"                                },
    {SZ_SECT_ALIAS             , "cotton"                         , "COTT"                                },
    {SZ_SECT_ALIAS             , "rings_of_invisibility"          , "RING"                                },
    {SZ_SECT_ALIAS             , "cloaks_of_invulnerability"      , "CLOA"                                },
    {SZ_SECT_ALIAS             , "staves_of_fire"                 , "STAF"                                },
    {SZ_SECT_ALIAS             , "staves_of_lightning"            , "STAL"                                },
    {SZ_SECT_ALIAS             , "amulets_of_true_seeing"         , "AMTS"                                },
    {SZ_SECT_ALIAS             , "amulets_of_protection"          , "AMPR"                                },
    {SZ_SECT_ALIAS             , "runeswords"                     , "RUNE"                                },
    {SZ_SECT_ALIAS             , "shieldstones"                   , "SHST"                                },
    {SZ_SECT_ALIAS             , "magic_carpets"                  , "CARP"                                },
    {SZ_SECT_ALIAS             , "portals"                        , "PORT"                                },
    {SZ_SECT_ALIAS             , "peasants"                       , "PEAS"                                },


    {SZ_SECT_ALIAS             , "mining"                         , "MINI"                                },
    {SZ_SECT_ALIAS             , "lumberjack"                     , "LUMB"                                },
    {SZ_SECT_ALIAS             , "quarrying"                      , "QUAR"                                },
    {SZ_SECT_ALIAS             , "hunting"                        , "HUNT"                                },
    {SZ_SECT_ALIAS             , "fishing"                        , "FISH"                                },
    {SZ_SECT_ALIAS             , "herb_lore"                      , "HERB"                                },
    {SZ_SECT_ALIAS             , "horse_training"                 , "HORS"                                },
    {SZ_SECT_ALIAS             , "weaponsmith"                    , "WEAP"                                },
    {SZ_SECT_ALIAS             , "armorer"                        , "ARMO"                                },
    {SZ_SECT_ALIAS             , "carpenter"                      , "CARP"                                },
    {SZ_SECT_ALIAS             , "building"                       , "BUIL"                                },
    {SZ_SECT_ALIAS             , "shipbuilding"                   , "SHIP"                                },
    {SZ_SECT_ALIAS             , "entertainment"                  , "ENTE"                                },
    {SZ_SECT_ALIAS             , "tactics"                        , "TACT"                                },
    {SZ_SECT_ALIAS             , "combat"                         , "COMB"                                },
    {SZ_SECT_ALIAS             , "riding"                         , "RIDI"                                },
    {SZ_SECT_ALIAS             , "crossbow"                       , "XBOW"                                },
    {SZ_SECT_ALIAS             , "longbow"                        , "LBOW"                                },
    {SZ_SECT_ALIAS             , "stealth"                        , "STEA"                                },
    {SZ_SECT_ALIAS             , "observation"                    , "OBSE"                                },
    {SZ_SECT_ALIAS             , "healing"                        , "HEAL"                                },
    {SZ_SECT_ALIAS             , "sailing"                        , "SAIL"                                },
    {SZ_SECT_ALIAS             , "farming"                        , "FARM"                                },
    {SZ_SECT_ALIAS             , "ranching"                       , "RANC"                                },
    {SZ_SECT_ALIAS             , "force"                          , "FORC"                                },
    {SZ_SECT_ALIAS             , "pattern"                        , "PATT"                                },
    {SZ_SECT_ALIAS             , "spirit"                         , "SPIR"                                },
    {SZ_SECT_ALIAS             , "earthquake"                     , "EQUA"                                },
    {SZ_SECT_ALIAS             , "force_shield"                   , "FSHI"                                },
    {SZ_SECT_ALIAS             , "energy_shield"                  , "ESHI"                                },
    {SZ_SECT_ALIAS             , "spirit_shield"                  , "SSHI"                                },
    {SZ_SECT_ALIAS             , "magical_healing"                , "MHEA"                                },
    {SZ_SECT_ALIAS             , "gate_lore"                      , "GATE"                                },
    {SZ_SECT_ALIAS             , "farsight"                       , "FARS"                                },
    {SZ_SECT_ALIAS             , "teleportation"                  , "TELE"                                },
    {SZ_SECT_ALIAS             , "portal_lore"                    , "PORT"                                },
    {SZ_SECT_ALIAS             , "mind_reading"                   , "MIND"                                },
    {SZ_SECT_ALIAS             , "weather_lore"                   , "WEAT"                                },
    {SZ_SECT_ALIAS             , "summon_wind"                    , "SWIN"                                },
    {SZ_SECT_ALIAS             , "summon_storm"                   , "SSTO"                                },
    {SZ_SECT_ALIAS             , "summon_tornado"                 , "STOR"                                },
    {SZ_SECT_ALIAS             , "call_lightning"                 , "CALL"                                },
    {SZ_SECT_ALIAS             , "clear_skies"                    , "CLEA"                                },
    {SZ_SECT_ALIAS             , "earth_lore"                     , "EART"                                },
    {SZ_SECT_ALIAS             , "wolf_lore"                      , "WOLF"                                },
    {SZ_SECT_ALIAS             , "bird_lore"                      , "BIRD"                                },
    {SZ_SECT_ALIAS             , "dragon_lore"                    , "DRAG"                                },
    {SZ_SECT_ALIAS             , "necromancy"                     , "NECR"                                },
    {SZ_SECT_ALIAS             , "summon_skeletons"               , "SUSK"                                },
    {SZ_SECT_ALIAS             , "raise_undead"                   , "RAIS"                                },
    {SZ_SECT_ALIAS             , "summon_lich"                    , "SULI"                                },
    {SZ_SECT_ALIAS             , "create_aura_of_fear"            , "FEAR"                                },
    {SZ_SECT_ALIAS             , "summon_black_wind"              , "SBLA"                                },
    {SZ_SECT_ALIAS             , "banish_undead"                  , "BUND"                                },
    {SZ_SECT_ALIAS             , "demon_lore"                     , "DEMO"                                },
    {SZ_SECT_ALIAS             , "summon_imps"                    , "SUIM"                                },
    {SZ_SECT_ALIAS             , "summon_demon"                   , "SUDE"                                },
    {SZ_SECT_ALIAS             , "summon_balrog"                  , "SUBA"                                },
    {SZ_SECT_ALIAS             , "banish_demons"                  , "BDEM"                                },
    {SZ_SECT_ALIAS             , "illusion"                       , "ILLU"                                },
    {SZ_SECT_ALIAS             , "phantasmal_entertainment"       , "PHEN"                                },
    {SZ_SECT_ALIAS             , "create_phantasmal_beasts"       , "PHBE"                                },
    {SZ_SECT_ALIAS             , "create_phantasmal_undead"       , "PHUN"                                },
    {SZ_SECT_ALIAS             , "create_phantasmal_demons"       , "PHDE"                                },
    {SZ_SECT_ALIAS             , "invisibility"                   , "INVI"                                },
    {SZ_SECT_ALIAS             , "true_seeing"                    , "TRUE"                                },
    {SZ_SECT_ALIAS             , "dispel_illusions"               , "DISP"                                },
    {SZ_SECT_ALIAS             , "artifact_lore"                  , "ARTI"                                },
    {SZ_SECT_ALIAS             , "create_ring_of_invisibility"    , "CRRI"                                },
    {SZ_SECT_ALIAS             , "create_cloak_of_invulnerability", "CRCL"                                },
    {SZ_SECT_ALIAS             , "create_staff_of_fire"           , "CRSF"                                },
    {SZ_SECT_ALIAS             , "create_staff_of_lightning"      , "CRSL"                                },
    {SZ_SECT_ALIAS             , "create_amulet_of_true_seeing"   , "CRTA"                                },
    {SZ_SECT_ALIAS             , "create_amulet_of_protection"    , "CRPA"                                },
    {SZ_SECT_ALIAS             , "create_runesword"               , "CRRU"                                },
    {SZ_SECT_ALIAS             , "create_shieldstone"             , "CRSH"                                },
    {SZ_SECT_ALIAS             , "create_magic_carpet"            , "CRMA"                                },
    {SZ_SECT_ALIAS             , "engrave_runes_of_warding"       , "ENGR"                                },
    {SZ_SECT_ALIAS             , "construct_gate"                 , "CGAT"                                },
    {SZ_SECT_ALIAS             , "enchant_swords"                 , "ESWO"                                },
    {SZ_SECT_ALIAS             , "enchant_armor"                  , "EARM"                                },
    {SZ_SECT_ALIAS             , "construct_portal"               , "CPOR"                                },


    {SZ_SECT_WEIGHT_MOVE       , "LEAD"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "VIKI"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "BARB"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "PLAI"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "ESKI"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "NOMA"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "TMAN"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "DMAN"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "WELF"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "SELF"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "HELF"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "TELF"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "IDWA"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "HDWA"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "UDWA"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "DDWA"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "ORC"                            , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "SILV"                           , "0,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "GRAI"                           , "5,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "LIVE"                           , "50,50,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "IRON"                           , "5,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "WOOD"                           , "5,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "STON"                           , "50,0,0,0"         },
    {SZ_SECT_WEIGHT_MOVE       , "FUR"                            , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "FISH"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "HERB"                           , "0,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "HORS"                           , "50,70,70,0"       },
    {SZ_SECT_WEIGHT_MOVE       , "SWOR"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "XBOW"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "LBOW"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "CARM"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "PARM"                           , "3,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "WAGO"                           , "50,0,0,0"         },
    {SZ_SECT_WEIGHT_MOVE       , "MITH"                           , "10,0,0,0"         },
    {SZ_SECT_WEIGHT_MOVE       , "IRWD"                           , "10,0,0,0"         },
    {SZ_SECT_WEIGHT_MOVE       , "WING"                           , "50,70,70,70"      },
    {SZ_SECT_WEIGHT_MOVE       , "FLOA"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "ROOT"                           , "50,0,0,0"         },
    {SZ_SECT_WEIGHT_MOVE       , "YEW"                            , "5,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "MSWO"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "MARM"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "DBOW"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "IVOR"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "PEAR"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "JEWE"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "FIGU"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "TARO"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "CAVI"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "WINE"                           , "5,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "SPIC"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "CHOC"                           , "5,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "TRUF"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "VODK"                           , "5,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "ROSE"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "PERF"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "SILK"                           , "5,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "VELV"                           , "5,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "MINK"                           , "5,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "CASH"                           , "5,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "COTT"                           , "5,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "DYE"                            , "5,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "WOOL"                           , "5,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "LION"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "WOLF"                           , "10,15,15,0"       },
    {SZ_SECT_WEIGHT_MOVE       , "GRIZ"                           , "50,60,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "CROC"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "ANAC"                           , "50,60,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "SCOR"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "POLA"                           , "50,60,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "GRAT"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "GSPI"                           , "50,60,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "GLIZ"                           , "50,60,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "TREN"                           , "250,300,0,0"      },
    {SZ_SECT_WEIGHT_MOVE       , "ROC"                            , "250,300,300,300"  },
    {SZ_SECT_WEIGHT_MOVE       , "BOGT"                           , "50,60,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "KONG"                           , "250,300,0,0"      },
    {SZ_SECT_WEIGHT_MOVE       , "SPHI"                           , "250,300,300,0"    },
    {SZ_SECT_WEIGHT_MOVE       , "ICEW"                           , "250,300,0,0"      },
    {SZ_SECT_WEIGHT_MOVE       , "DRAG"                           , "250,300,300,300"  },
    {SZ_SECT_WEIGHT_MOVE       , "CENT"                           , "50,60,60,0"       },
    {SZ_SECT_WEIGHT_MOVE       , "KOBO"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "OGRE"                           , "50,60,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "LMAN"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "WMAN"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "SAND"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "YETI"                           , "50,60,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "GOBL"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "TROL"                           , "50,60,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "ETTI"                           , "50,60,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "SKEL"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "UNDE"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "LICH"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "IMP"                            , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "DEMO"                           , "50,60,60,0"       },
    {SZ_SECT_WEIGHT_MOVE       , "BALR"                           , "250,300,300,300"  },
    {SZ_SECT_WEIGHT_MOVE       , "EAGL"                           , "10,15,15,15"      },
    {SZ_SECT_WEIGHT_MOVE       , "XXXX"                           , "0,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "RING"                           , "0,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "CLOA"                           , "0,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "STAF"                           , "0,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "STAL"                           , "0,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "AMTS"                           , "0,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "AMPR"                           , "0,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "RUNE"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "SHST"                           , "0,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "CARP"                           , "0,15,15,15"       },
    {SZ_SECT_WEIGHT_MOVE       , "WOLF"                           , "1,1,1,1"          },
    {SZ_SECT_WEIGHT_MOVE       , "EAGL"                           , "1,1,1,1"          },
    {SZ_SECT_WEIGHT_MOVE       , "DRAG"                           , "1,1,1,1"          },
    {SZ_SECT_WEIGHT_MOVE       , "SKEL"                           , "1,1,1,1"          },
    {SZ_SECT_WEIGHT_MOVE       , "UNDE"                           , "1,1,1,1"          },
    {SZ_SECT_WEIGHT_MOVE       , "LICH"                           , "1,1,1,1"          },
    {SZ_SECT_WEIGHT_MOVE       , "IMP"                            , "1,1,1,1"          },
    {SZ_SECT_WEIGHT_MOVE       , "DEMO"                           , "1,1,1,1"          },
    {SZ_SECT_WEIGHT_MOVE       , "BALR"                           , "1,1,1,1"          },
    {SZ_SECT_WEIGHT_MOVE       , "PORT"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "PEAS"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "PICK"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "SPEA"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "AXE"                            , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "HAMM"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "MXBO"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "MWAG"                           , "50,250,250,0"     },
    {SZ_SECT_WEIGHT_MOVE       , "GLID"                           , "5,0,0,15"         },
    {SZ_SECT_WEIGHT_MOVE       , "NET"                            , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "LASS"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "BAG"                            , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "SPIN"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "LARM"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "CLAR"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "BOOT"                           , "0,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "PIRA"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "KRAK"                           , "10,300,0,0"       },
    {SZ_SECT_WEIGHT_MOVE       , "MERF"                           , "10,0,0,0"         },
    {SZ_SECT_WEIGHT_MOVE       , "ELEM"                           , "250,300,0,0"      },
    {SZ_SECT_WEIGHT_MOVE       , "MAN"                            , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "FAIR"                           , "3,13,0,13"        },
    {SZ_SECT_WEIGHT_MOVE       , "LIZA"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "URUK"                           , "11,21,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "GBLN"                           , "5,12,0,0"         },
    {SZ_SECT_WEIGHT_MOVE       , "HOBB"                           , "8,12,0,0"         },
    {SZ_SECT_WEIGHT_MOVE       , "GNOL"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "DRLF"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "MERC"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "TITA"                           , "20,60,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "AMAZ"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "OGER"                           , "15,30,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "GNOM"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "HILA"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "MINO"                           , "15,40,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "LANC"                           , "2,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "MUSH"                           , "0,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "RRAT"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "NOOG"                           , "250,300,0,0"      },
    {SZ_SECT_WEIGHT_MOVE       , "MUTA"                           , "15,20,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "BAXE"                           , "2,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "MBAX"                           , "2,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "ADMT"                           , "10,0,0,0"         },
    {SZ_SECT_WEIGHT_MOVE       , "ASWR"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "ABAX"                           , "2,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "IMTH"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "ARNG"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "AARM"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "SBOW"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "CAME"                           , "50,70,70,0"       },
    {SZ_SECT_WEIGHT_MOVE       , "GELF"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "DROW"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "HYDR"                           , "250,300,0,0"      },
    {SZ_SECT_WEIGHT_MOVE       , "STGI"                           , "200,250,0,0"      },
    {SZ_SECT_WEIGHT_MOVE       , "CLGI"                           , "200,250,0,0"      },
    {SZ_SECT_WEIGHT_MOVE       , "ILLY"                           , "50,75,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "SORC"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "MAGI"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "DMAG"                           , "10,15,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "WARR"                           , "10,40,0,0"        },
    {SZ_SECT_WEIGHT_MOVE       , "IDRA"                           , "250,300,300,0"    },
    {SZ_SECT_WEIGHT_MOVE       , "HPOT"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "RGEM"                           , "2,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "GEM"                            , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "JAVE"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "PIKE"                           , "2,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "MWOL"                           , "10,17,17,0"       },
    {SZ_SECT_WEIGHT_MOVE       , "MSPI"                           , "50,60,60,0"       },
    {SZ_SECT_WEIGHT_MOVE       , "MOLE"                           , "100,130,130,0"    },
    {SZ_SECT_WEIGHT_MOVE       , "BPLA"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "MCAR"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "FSWO"                           , "0,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "QUAR"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "SABR"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "MACE"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "MSTA"                           , "2,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "DAGG"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "PDAG"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "BHAM"                           , "2,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "BOW"                            , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "SBOW"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "HXBO"                           , "1,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , "HARP"                           , "1,0,0,0"          },

    {SZ_SECT_WEIGHT_MOVE       , PRP_STRUCT_ID                    , "0,0,0,0"          },
    {SZ_SECT_WEIGHT_MOVE       , PRP_SEQUENCE                     , "0,0,0,0"          }

};


int DefaultConfigSize = sizeof(DefaultConfig)/sizeof(DefaultConfigRec);

