/*
 * This source file is part of the Atlantis Little Helper program.
 * Copyright (C) 2001 Maxim Shariy.
 *
 * Atlantis Little Helper is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Atlantis Little Helper is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Atlantis Little Helper; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <math.h>
#include "collection.h"
#include "cfgfile.h"
#include "data.h"


RESOLVE_ALIAS_FUNC  ResolveAliasLoc  = NULL;
GET_ITEM_WEIGHTS    GetItemWeightsLoc= NULL;


const char * GRP_MEN        = "LEAD, VIKI, BARB, PLAI, ESKI, "
                              "NOMA, TMAN, DMAN, WELF, SELF, "
                              "HELF, TELF, IDWA, HDWA, UDWA, "
                              "DDWA, ORC" ;
                         
const char * GRP_SKILLS     = "MINI_, LUMB_, QUAR_, HUNT_, FISH_, "
                              "HERB_, HORS_, WEAP_, ARMO_, CARP_, "
                              "BUIL_, SHIP_, ENTE_, TACT_, COMB_, "
                              "RIDI_, XBOW_, LBOW_, STEA_, OBSE_, "
                              "HEAL_, SAIL_, FARM_, RANC_";
                         
const char * GRP_MAG_SKILLS = "FORC_, "
                              "PATT_, SPIR_, FIRE_, EQUA_, FSHI_, "
                              "ESHI_, SSHI_, MHEA_, GATE_, FARS_, "
                              "TELE_, PORT_, MIND_, WEAT_, SWIN_, "
                              "SSTO_, STOR_, CALL_, CLEA_, EART_, "
                              "WOLF_, BIRD_, DRAG_, NECR_, SUSK_, "
                              "RAIS_, SULI_, FEAR_, SBLA_, BUND_, "
                              "DEMO_, SUIM_, SUDE_, SUBA_, BDEM_, "
                              "ILLU_, PHEN_, PHBE_, PHUN_, PHDE_, "
                              "INVI_, TRUE_, DISP_, ARTI_, CRRI_, "
                              "CRCL_, CRSF_, CRSL_, CRTA_, CRPA_, "
                              "CRRU_, CRSH_, CRMA_, ENGR_, CGAT_, "
                              "ESWO_, EARM_, CPOR_";

const char * GRP_STUFF      = "GRAI, LIVE, IRON, WOOD, STON, "
                              "FUR , FISH, HERB, MITH, IRWD, "
                              "FLOA, ROOT, YEW";
                             
const char * GRP_HORS       = "HORS, WING";
                             
const char * GRP_ARMS       = "SWOR, XBOW, LBOW, CARM, PARM, "
                              "MSWO, MARM, DBOW";
                             
const char * GRP_MAG_ITEMS  = "RING, CLOA, STAF, STAL, AMTS, "
                              "AMPR, RUNE, SHST, CARP, PORT";
                             
const char * GRP_TRADE_ITEMS= "IVOR, PEAR, JEWE, FIGU, TARO, CAVI, "
                              "WINE, SPIC, CHOC, TRUF, VODK, ROSE, "
                              "PERF, SILK, VELV, MINK, CASH, COTT, " 
                              "DYE, WOOL, GEM";

const char * GRP_JUNK_ITEMS = GRP_TRADE_ITEMS;
                                                           
const char * STRUCT_GATE    = "GATE";

const char * STD_UNIT_PROPS[] = {PRP_FACTION_ID           ,
                                 PRP_FACTION              ,
                                 PRP_LAND_ID              ,
                                 PRP_STRUCT_ID            ,
                                 PRP_COMMENTS             ,
                                 PRP_ORDERS               ,
                                 PRP_STRUCT_OWNER         ,
                                 PRP_STRUCT_NAME          ,
                                 PRP_TEACHING             ,
                                 PRP_WEIGHT               ,
                                 PRP_MOVEMENT             ,
                                 
                                 PRP_MEN                  ,
                                 PRP_SKILLS               ,
                                 PRP_MAG_SKILLS           ,
                                 PRP_STUFF                ,
                                 PRP_HORS                 ,
                                 PRP_ARMS                 ,
                                 PRP_MAG_ITEMS            ,
                                 PRP_JUNK_ITEMS           ,
                                 PRP_SEL_FACT_MEN         ,
                                 PRP_SEQUENCE             };
const int STD_UNIT_PROPS_COUNT = sizeof(STD_UNIT_PROPS)/sizeof(*STD_UNIT_PROPS);

const char * STD_UNIT_POSTFIXES[] = {PRP_SKILLS_POSTFIX, PRP_SKILL_DAYS_POSTFIX};
const int STD_UNIT_POSTFIXES_COUNT = sizeof(STD_UNIT_POSTFIXES)/sizeof(*STD_UNIT_POSTFIXES);

//=============================================================

CBaseObject::CBaseObject() : Name(32), Description(128)
{
    Id = 0;
}

//-------------------------------------------------------------

void CBaseObject::Done()
{
}

//-------------------------------------------------------------

const char * CBaseObject::ResolveAlias(const char * alias)
{
    if (ResolveAliasLoc)
        return ResolveAliasLoc(alias);
    else
        return alias;
}

//-------------------------------------------------------------

BOOL CBaseObject::GetProperty(const char  *  name, 
                              EValueType   & type, 
                              const void  *& value,
                              EPropertyType  proptype
                              )
{
    BOOL          Ok = TRUE;

    name = ResolveAlias(name);

    if (!TPropertyHolder::GetProperty(name, type, value, proptype))
    {
        if      (0==stricmp(name, PRP_ID))
        {
            type  = eLong;
            value = (void*)Id;
        }
        else if (0==stricmp(name, PRP_NAME))
        {
            type  = eCharPtr;
            value = Name.GetData();
        }
        else if (0==stricmp(name, PRP_DESCR))
        {
            type  = eCharPtr;
            value = Description.GetData();
        }
        else
            Ok = FALSE;
    }
    return Ok;
}

//-------------------------------------------------------------

void CBaseObject::DebugPrint(CStr & sDest)
{
    sDest << "\n"
          << "Id          = " << Id << "\n"
          << "Name        = " << Name.GetData() << "\n"
          << "Description = " << Description.GetData() << "\n";
}

//=============================================================

void CFaction::DebugPrint(CStr & sDest)
{
    CBaseObject::DebugPrint(sDest);

    sDest << "UnclaimedSilver = " << UnclaimedSilver << "\n";
}

//=============================================================

CLand::~CLand()
{
    Structs.FreeAll();
    ResetUnits();
    Units.DeleteAll();
    UnitsSeq.DeleteAll();
    Products.FreeAll();
}

void CLand::DebugPrint(CStr & sDest)
{
    CBaseObject::DebugPrint(sDest);

    sDest << "Taxable   = " << Taxable << "\n";
}

//-------------------------------------------------------------

BOOL CLand::AddUnit(CUnit * pUnit)
{
    if (Units.Insert(pUnit))
    {
        pUnit->LandId = Id;
        Flags |= LAND_UNITS;
        UnitsSeq.Insert(pUnit);
        pUnit->SetProperty(PRP_SEQUENCE, eLong, (void*)UnitsSeq.Count(), eNormal);
        return TRUE;
    }
    else
        return FALSE;

}

//-------------------------------------------------------------

void CLand::RemoveUnit(CUnit * pUnit)
{
    int i;
    CUnit * p;
    
    if (Units.Search(pUnit, i))
    {
        Units.AtDelete(i);
        
        for (i=UnitsSeq.Count()-1; i>=0; i--)
        {
            p = (CUnit*)UnitsSeq.At(i);
            if (p->Id == pUnit->Id)
            {
                UnitsSeq.AtDelete(i);
                break;
            }
        }
    }
}

//-------------------------------------------------------------

void CLand::ResetUnits()
{
    int i;
    CUnit * pUnit;

    for (i=UnitsSeq.Count()-1; i>=0; i--)
    {
        pUnit = (CUnit*)UnitsSeq.At(i);
        if (IS_NEW_UNIT(pUnit))
            UnitsSeq.AtDelete(i);
        else
            break;
    }

    for (i=Units.Count()-1; i>=0; i--)
    {
        pUnit = (CUnit*)Units.At(i);
        if (IS_NEW_UNIT(pUnit))
            Units.AtFree(i);
        else
        {
            pUnit->ResetNormalProperties();
            if (pUnit->pMovement)
            {
                delete pUnit->pMovement;
                pUnit->pMovement = NULL;
            }
            if (pUnit->pStudents)
                pUnit->pStudents->DeleteAll(); // probably deleting it would not be very usefull
        }
    }
}

//-------------------------------------------------------------

CStruct * CLand::GetStructById(long id)
{
    CStruct     * pStruct = NULL;
    CBaseObject   Dummy;
    int       i;

    Dummy.Id = id;
    if (Structs.Search(&Dummy, i))
        pStruct = (CStruct*)Structs.At(i);

    return pStruct;
}

//-------------------------------------------------------------

CStruct * CLand::AddNewStruct(CStruct * pNewStruct)
{
    int       idx;
    CStruct * pStruct;

    if (Structs.Search(pNewStruct, idx))
    {
        pStruct = (CStruct*)Structs.At(idx);

//        if (0==stricmp(pStruct->Kind.GetData(), "Shaft") )
        if (pStruct->Attr & SA_SHAFT  ) 
        {
            // process links for shafts
            
            int  x1, x2, x3;
            BOOL Link;

            x1   = pNewStruct->Description.FindSubStr(";");
            x2   = pNewStruct->Description.FindSubStr("links");
            x3   = pNewStruct->Description.FindSubStr("to");
            Link = ( x1>=0 && x1<x2 && x2<x3 );
                       
            if (Link || pNewStruct->Description.GetLength() > pStruct->Description.GetLength())
                pStruct->Description= pNewStruct->Description;
        }
        else
            pStruct->Description= pNewStruct->Description;
        pStruct->Name           = pNewStruct->Name       ;
        pStruct->LandId         = pNewStruct->LandId     ;
        pStruct->OwnerUnitId    = pNewStruct->OwnerUnitId;
        pStruct->Load           = pNewStruct->Load       ;
        pStruct->Attr           = pNewStruct->Attr       ;
        pStruct->Kind           = pNewStruct->Kind       ;

        delete pNewStruct;
    }
    else
    {
        Structs.Insert(pNewStruct);
        if (0 == pNewStruct->Attr)                 Flags |= LAND_STR_GENERIC;
        else
        {
            if (pNewStruct->Attr & SA_HIDDEN )     Flags |= LAND_STR_HIDDEN ;
            if (pNewStruct->Attr & SA_MOBILE )     Flags |= LAND_STR_MOBILE ;
            if (pNewStruct->Attr & SA_SHAFT  )     Flags |= LAND_STR_SHAFT  ;
            if (pNewStruct->Attr & SA_GATE   )     Flags |= LAND_STR_GATE   ;
            if (pNewStruct->Attr & SA_ROAD_N )     Flags |= LAND_STR_ROAD_N ;
            if (pNewStruct->Attr & SA_ROAD_NE)     Flags |= LAND_STR_ROAD_NE;
            if (pNewStruct->Attr & SA_ROAD_SE)     Flags |= LAND_STR_ROAD_SE;
            if (pNewStruct->Attr & SA_ROAD_S )     Flags |= LAND_STR_ROAD_S ;
            if (pNewStruct->Attr & SA_ROAD_SW)     Flags |= LAND_STR_ROAD_SW;
            if (pNewStruct->Attr & SA_ROAD_NW)     Flags |= LAND_STR_ROAD_NW;
        }

            
        pStruct = pNewStruct;
    }

    return pStruct;
}

//-------------------------------------------------------------

void CLand::CalcStructsLoad()
{
    int                 i, k;
    CUnit             * pUnit;
    CBaseObject         Dummy;
    CStruct           * pStruct;
    EValueType          type;
    const void        * value;

    for (k=0; k<Structs.Count(); k++)
    {
        pStruct = (CStruct*)Structs.At(k);
        pStruct->Load = 0;
    }

    for (i=Units.Count()-1; i>=0; i--) 
    {
        pUnit = (CUnit*)UnitsSeq.At(i);
        if (pUnit->GetProperty(PRP_STRUCT_ID, type, value, eNormal) && (eLong==type) && ((long)value > 0))
        {
            Dummy.Id = (long)value;
            if (Structs.Search(&Dummy, k))
            {
                pStruct = (CStruct*)Structs.At(k);
                pStruct->Load += pUnit->Weight[0];
            }
        }
    }
}

//=============================================================

CStrStrColl * CUnit::m_PropertyGroupsColl = NULL;


CStrStrColl * CUnit::GetPropertyGroups()
{
    return m_PropertyGroupsColl;
}

//-------------------------------------------------------------

CUnit::CUnit() : CBaseObject(), Comments(16), DefOrders(32), Orders(32), Errors(32), Events(32) 
{
    IsOurs      = false;
    FactionId   = 0; 
    pFaction    = NULL;
    LandId      = 0; 
    Teaching    = 0.0;
    pMovement   = NULL; 
    pStudents   = NULL;
    SilvRcvd    = 0;
    Flags       = 0;
    memset(Weight, 0, sizeof(Weight));
};

//-------------------------------------------------------------

CUnit::~CUnit()
{
    if (pMovement)
        delete pMovement;
    if (pStudents)
    {
        pStudents->DeleteAll();
        delete pStudents;
    }
}

//-------------------------------------------------------------

void CUnit::ExtractCommentsFromDefOrders()
{
    const char * p;
    CStr         S;

    Comments.Empty();
    p = DefOrders.GetData();
    while (p)
    {
        p = S.GetToken(p, '\n', TRIM_ALL);
        if ( (S.GetLength() > 1) && (';' == S.GetData()[0]) )
        {
            Comments.SetStr(&S.GetData()[1], S.GetLength()-1);
            Comments.TrimLeft(TRIM_ALL);
            break;
        }
    }

}

//-------------------------------------------------------------

void CUnit::ResetNormalProperties()
{
    int           idx;
    const char  * propname;
    int         * weights;
    const char ** movenames;
    int           movecount;
    EValueType    type;
    const void  * n;


    TPropertyHolder::ResetNormalProperties();
    Teaching = 0;
    StudyingSkill.Empty();
    SetProperty(PRP_MOVEMENT, eCharPtr, "", eNormal);
    SilvRcvd = 0;

    // taxes and production
    if (Flags & UNIT_AUTOTAXING)
        Flags |= UNIT_TAXING;
    else
        Flags &= ~UNIT_TAXING;
    Flags &= ~(UNIT_PRODUCING | UNIT_GIVEN);    
    

    // calc weight;
    memset(Weight, 0, sizeof(Weight));
    idx        = 0;
    propname = GetPropertyName(idx);
    while (propname)
    {
        if (GetProperty(propname, type, n, eNormal) && 
            (eLong==type) &&
            GetItemWeightsLoc(propname, weights, movenames, movecount))
            AddWeight((long)n, weights, movenames, movecount);

        propname = GetPropertyName(++idx);
    }
}

//-------------------------------------------------------------

void CUnit::AddWeight(int nitems, int * weights, const char ** movenames, int nweights)
{
    int i;
    int NW = min(nweights, MOVE_MODE_MAX);

    for (i=0; i<NW; i++)
        Weight[i] += nitems*weights[i];

    if (Weight[0])
    {
        i = NW-1;
        while ((i>0) && (Weight[i] < Weight[0]))
            i--;
    }
    else
        i = 0;

    SetProperty(PRP_MOVEMENT, eCharPtr, movenames[i], eNormal);
}

//-------------------------------------------------------------

BOOL CUnit::GetProperty(const char  *  name, 
                        EValueType   & type, 
                        const void  *& value,
                        EPropertyType  proptype
                        )
{
    BOOL         Ok = TRUE;

    name = ResolveAlias(name);

    if (0==stricmp(name, PRP_ID))
    {
        type  = eLong;
        value = (void*)Id;
       // if ((long)value < 0)
       //     value = (void*)0L;
        return TRUE;
    }

    if (!CBaseObject::GetProperty(name, type, value, proptype))
    {
        if (0==stricmp(name, PRP_COMMENTS  ))
        {
            type  = eCharPtr;
            value = Comments.GetData();
        }
        else if (0==stricmp(name, PRP_ORDERS    ))
        {
            type  = eCharPtr;
            value = Orders.GetData();
        }
        else if (0==stricmp(name, PRP_FACTION_ID))
        {
            type  = eLong;
            value = (void*)FactionId;
        }
        else if (0==stricmp(name, PRP_FACTION))
        {
            type  = eCharPtr;
            if (pFaction)
                value = pFaction->Name.GetData();
            else
                value = "";
        }
        else if (0==stricmp(name, PRP_LAND_ID   ))
        {
            type  = eLong;
            value = (void*)LandId;
        }
        else if (0==stricmp(name, PRP_WEIGHT ))
        {
            type  = eLong;
            value = (void*)Weight[0];
        }
        else if (0==stricmp(name, PRP_TEACHING ))
        {
            type  = eLong;
            if (StudyingSkill.IsEmpty())
                value = (void*)(long)ceil(Teaching);
            else
                value = (void*)(long)floor(Teaching);

            if (Teaching <= 0)
                Ok = FALSE;
        }
        else
            Ok = FALSE;
    }

    return Ok;
}

//-------------------------------------------------------------

void CUnit::DebugPrint(CStr & sDest)
{
    CBaseObject::DebugPrint(sDest);

    sDest << "FactionId = " << FactionId << "\n"
          << "LandId    = " << LandId << "\n"
        ;
}


//=============================================================

CBaseColl::CBaseColl() : CCollection()
{
}

CBaseColl::CBaseColl(int nDelta) : CCollection(nDelta)
{
}

void CBaseColl::FreeItem(void * pItem)
{
    if (pItem)
    {
        CBaseObject * pBase = (CBaseObject*)pItem;

        pBase->Done();
        delete pBase;
    }
}

//-------------------------------------------------------------

CBaseCollById::CBaseCollById() : CSortedCollection()
{
}

CBaseCollById::CBaseCollById(int nDelta) : CSortedCollection(nDelta)
{
}

void CBaseCollById::FreeItem(void * pItem)
{
    if (pItem)
    {
        CBaseObject * pBase = (CBaseObject*)pItem;

        pBase->Done();
        delete pBase;
    }
}

//-------------------------------------------------------------

int  CBaseCollById::Compare(void * pItem1, void * pItem2) 
{
    CBaseObject * pBase1 = (CBaseObject*)pItem1;
    CBaseObject * pBase2 = (CBaseObject*)pItem2;

    if (pBase1->Id > pBase2->Id)
        return 1;
    else
        if (pBase1->Id < pBase2->Id)
            return -1;
        else
            return 0;
}


//-------------------------------------------------------------

CPlane::~CPlane()
{
    Lands.FreeAll();
}

//-------------------------------------------------------------

int  CBaseCollByName::Compare(void * pItem1, void * pItem2) 
{
    CBaseObject * pBase1 = (CBaseObject*)pItem1;
    CBaseObject * pBase2 = (CBaseObject*)pItem2;

    return stricmp(pBase1->Name.GetData(), pBase2->Name.GetData());
}


//-------------------------------------------------------------

/* creates problems with NEW_UNIT_ID!!!
   but that macro was changed to be land id independent */

#define XY_DELTA   0x00000800
#define X_MASK     0x00000FFF
#define Y_MASK     0x00FFF000
#define Z_MASK     0xFF000000
#define Y_SHIFT    12
#define Z_SHIFT    24


long LandCoordToId(int x, int y, int z)
{
    return ( (z           << Z_SHIFT) & Z_MASK) | 
           (((y+XY_DELTA) << Y_SHIFT) & Y_MASK) | 
           ( (x+XY_DELTA)             & X_MASK);
}

//-------------------------------------------------------------

void LandIdToCoord(long id, int & x, int & y, int & z)
{
    x = ( id & X_MASK)             - XY_DELTA;
    y = ((id & Y_MASK) >> Y_SHIFT) - XY_DELTA;
    z = ( id & Z_MASK) >> Z_SHIFT;
}
/**/



//-------------------------------------------------------------

/* slow

long LandCoordToId(int x, int y, int z)
{
    return (z*1000 + (y+500))*1000 + (x+500);  // range is -499 - +499
}

//-------------------------------------------------------------

void LandIdToCoord(long id, int & x, int & y, int & z)
{
    x  = id % 1000;
    x -= 500;

    id = id/1000;
    y  = id % 1000;
    y -=500;
    z  = id/1000;
}
*/

//-------------------------------------------------------------

void TestLandId()
{
    int  x0, y0, z0;
    int  x1, y1, z1;
    int  x = 0;
    long id;

#define _x_min_  -48
#define _x_max_  500

#define _y_min_  -48
#define _y_max_  500

#define _z_min_  0
#define _z_max_  255

    for (z0=_z_min_; z0<=_z_max_; z0++)
        for (y0=_y_min_; y0<=_y_max_; y0++)
            for (x0=_x_min_; x0<=_x_max_; x0++)
            {
                id = LandCoordToId(x0, y0, z0);
                LandIdToCoord(id, x1, y1, z1);
                if ( (x0!=x1) || (y0!=y1) || (z0!=z1) )
                {
                    x = 10/x;
                }
            }
    x = 0;
    x = 1;
};

//-------------------------------------------------------------








