/*
 * This source file is part of the Atlantis Little Helper program.
 * Copyright (C) 2001 Maxim Shariy.
 *
 * Atlantis Little Helper is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Atlantis Little Helper is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Atlantis Little Helper; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __DATA_H_INCL__
#define __DATA_H_INCL__


#include "cstr.h"
#include "collection.h"
#include "objs.h"
#include <string.h>
#include "compat.h"

// common properties
#define PRP_ID                          "id"
#define PRP_NAME                        "name"
#define PRP_DESCR                       "description"
                                       
// unit properties                     
#define PRP_FACTION_ID                  "factionid" 
#define PRP_FACTION                     "faction" 
#define PRP_LAND_ID                     "landid"
#define PRP_STRUCT_ID                   "structid"
#define PRP_COMMENTS                    "comments"
#define PRP_ORDERS                      "orders"
#define PRP_STRUCT_OWNER                "structowner" 
#define PRP_STRUCT_NAME                 "structname" 
#define PRP_TEACHING                    "teaching"
#define PRP_WEIGHT                      "weight"
#define PRP_MOVEMENT                    "movement"
                                       
#define PRP_SILVER                      "silv"
                                       
#define PRP_SKILLS_POSTFIX              "_"
#define PRP_SKILL_DAYS_POSTFIX          "_d"
#define PRP_GIVE_ALL_POSTFIX            "_ga"

#define PRP_MEN                         "men"
#define PRP_SKILLS                      "skills" 
#define PRP_MAG_SKILLS                  "mag_skills" 
#define PRP_STUFF                       "stuff"
#define PRP_HORS                        "mounts" 
#define PRP_ARMS                        "armament"
#define PRP_MAG_ITEMS                   "mag_items"
#define PRP_JUNK_ITEMS                  "junk_items"
#define PRP_TRADE_ITEMS                 "trade_items"
#define PRP_SEL_FACT_MEN                "sel_fact_men"
#define PRP_SEQUENCE                    "sequence"

// land properties
#define PRP_LAND_LINK                   "*landlink"
#define PRP_SALE_AMOUNT_POSTFIX         "_s"
#define PRP_SALE_PRICE_POSTFIX          "_s$"
#define PRP_WANTED_AMOUNT_POSTFIX       "_w"
#define PRP_WANTED_PRICE_POSTFIX        "_w$"
                                       
#define MOVE_MODE_MAX                   8

#define LAND_FLAG_COUNT  3

// land flags
#define LAND_UNITS          0x00000001
#define LAND_VISITED        0x00000002
#define LAND_TAX            0x00000004
#define LAND_TRADE          0x00000008    
#define LAND_BATTLE         0x00000010
#define LAND_SET_EXITS      0x00000020
#define LAND_HAS_FLAGS      0x00000040
#define LAND_IS_CURRENT     0x00000080

#define LAND_STR_HIDDEN     0x00000100
#define LAND_STR_MOBILE     0x00000200
#define LAND_STR_SHAFT      0x00000400
#define LAND_STR_GATE       0x00000800
#define LAND_STR_ROAD_N     0x00001000
#define LAND_STR_ROAD_NE    0x00002000
#define LAND_STR_ROAD_SE    0x00004000
#define LAND_STR_ROAD_S     0x00008000
#define LAND_STR_ROAD_SW    0x00010000
#define LAND_STR_ROAD_NW    0x00020000
#define LAND_STR_GENERIC    0x00040000
#define LAND_STR_SPECIAL   (LAND_STR_HIDDEN  | LAND_STR_MOBILE  | LAND_STR_SHAFT   | LAND_STR_GATE   | \
                            LAND_STR_ROAD_N  | LAND_STR_ROAD_NE | LAND_STR_ROAD_SE | LAND_STR_ROAD_S | \
                            LAND_STR_ROAD_SW | LAND_STR_ROAD_NW)
                            
#define LAND_TAX_NEXT       0x00080000                      
#define LAND_TRADE_NEXT     0x00100000                        

                                                         
// structure attributes      
#define SA_HIDDEN   0x0001    
#define SA_MOBILE   0x0002    
#define SA_SHAFT    0x0004    
#define SA_GATE     0x0008    
#define SA_ROAD_N   0x0010    
#define SA_ROAD_NE  0x0020    
#define SA_ROAD_SE  0x0040    
#define SA_ROAD_S   0x0080
#define SA_ROAD_SW  0x0100
#define SA_ROAD_NW  0x0200
#define SA_ROAD_BAD 0x0400

// unit flags
#define UNIT_AUTOTAXING     0x0001
#define UNIT_TAXING         0x0002
#define UNIT_PRODUCING      0x0004
#define UNIT_GIVEN          0x0008


extern const char * STD_UNIT_PROPS[];
extern const int    STD_UNIT_PROPS_COUNT;

extern const char * STD_UNIT_POSTFIXES[];
extern const int    STD_UNIT_POSTFIXES_COUNT;


extern const char * GRP_MEN;
extern const char * GRP_SKILLS;
extern const char * GRP_MAG_SKILLS;
extern const char * GRP_STUFF;
extern const char * GRP_HORS;
extern const char * GRP_ARMS;
extern const char * GRP_MAG_ITEMS;
extern const char * GRP_JUNK_ITEMS;
extern const char * GRP_TRADE_ITEMS;
extern const char * STRUCT_GATE;

/* it looks like there is no need for the new unit id to be related to land id
#define NEW_UNIT_ID(pLand,n) (-pLand->Id*100 - n)
*/
#define NEW_UNIT_ID(pLand,n) (-n)
#define IS_NEW_UNIT_ID(Id)   (Id<0)
#define IS_NEW_UNIT(pUnit)   IS_NEW_UNIT_ID(pUnit->Id)


typedef const char * (* RESOLVE_ALIAS_FUNC)  (const char * alias);
extern RESOLVE_ALIAS_FUNC  ResolveAliasLoc;

typedef BOOL (* GET_ITEM_WEIGHTS)  (const char * item, int *& weights, const char **& movenames, int & movecount );
extern GET_ITEM_WEIGHTS  GetItemWeightsLoc;


class CPlane;

//-----------------------------------------------------------------

class CBaseObject : public TPropertyHolder
{
public:
    CBaseObject();
    virtual ~CBaseObject(){};
    virtual void Done();


    virtual const char  * ResolveAlias(const char * alias);
    virtual BOOL GetProperty(const char  *  name,
                             EValueType   & type,
                             const void  *& value, // returns pointer to inner location
                             EPropertyType  proptype = eNormal
                            );
    long Id;
    CStr Name;
    CStr Description;

    virtual void DebugPrint(CStr & sDest);
    virtual void Clear() {Id=0; Name.Empty(); Description.Empty();};

};

//-----------------------------------------------------------------

class CBaseColl : public CCollection
{
public:
    CBaseColl();
    CBaseColl(int nDelta);
protected:
    virtual void FreeItem(void * pItem);
};

//-----------------------------------------------------------------

class CBaseCollById : public CSortedCollection
{
public:
    CBaseCollById();
    CBaseCollById(int nDelta);
protected:
    virtual void FreeItem(void * pItem);
    virtual int Compare(void * pItem1, void * pItem2) ;
};

//-----------------------------------------------------------------

class CProduct
{
public:
    long  Amount;
    CStr  ShortName;
    CStr  LongName;
};

//-----------------------------------------------------------------

class CProductColl : public CSortedCollection
{
public:
    CProductColl()            : CSortedCollection()      {};
    CProductColl(int nDelta)  : CSortedCollection(nDelta){};
protected:
    virtual void FreeItem(void * pItem) {delete (CProduct*)pItem;};
    virtual int Compare(void * pItem1, void * pItem2) 
    {return(SafeCmp( ((CProduct*)pItem1)->ShortName.GetData(), 
                     ((CProduct*)pItem2)->ShortName.GetData() ));};
};

//-----------------------------------------------------------------

class CFaction : public CBaseObject
{
public:
    CFaction() : CBaseObject() {UnclaimedSilver=0;};
    long UnclaimedSilver;

    virtual void DebugPrint(CStr & sDest);
};

//-----------------------------------------------------------------

class CUnit : public CBaseObject
{
public:
    CUnit();
    virtual ~CUnit();
    virtual BOOL GetProperty(const char  *  name,
                             EValueType   & type,
                             const void  *& value, // returns pointer to inner location
                             EPropertyType  proptype = eNormal
                            );
    virtual CStrStrColl * GetPropertyGroups();
    void    ExtractCommentsFromDefOrders();
    virtual void ResetNormalProperties();
    void    AddWeight(int nitems, int * weights, const char ** movenames, int nweights);

    bool            IsOurs;
    long            FactionId;
    CFaction      * pFaction;
    long            LandId;
    long            Weight[MOVE_MODE_MAX];
    long            SilvRcvd;
    double          Teaching; // number of students per teacher / number of days per student
    CStr            Comments;
    CStr            DefOrders;
    CStr            Orders;
    CStr            Errors;
    CStr            Events;
    CStr            StudyingSkill;
    CLongColl     * pMovement; // Collection of ids of hexes to move through
    CBaseCollById * pStudents;
    long            Flags;

    static CStrStrColl * m_PropertyGroupsColl;

    virtual void DebugPrint(CStr & sDest);
};

//-----------------------------------------------------------------

class CStruct : public CBaseObject
{
public:
    CStruct() : CBaseObject() {LandId=0; OwnerUnitId=0; /*No=0;*/ Load=0; Attr=0;};
    long LandId;
    long OwnerUnitId;
    //long No;
    long Load;
    long Attr;
    CStr Kind;
};

//-----------------------------------------------------------------

class CLand : public CBaseObject
{
public:
    CLand() : CBaseObject(), Units(32), UnitsSeq(32) 
    {Flags=0; guiUnit=0; pPlane=NULL; ExitBits=0; AtlaclientsLastTurnNo=0; guiColor=-1;};
    ~CLand();

    BOOL      AddUnit(CUnit * pUnit);
    void      RemoveUnit(CUnit * pUnit);
    void      ResetUnits();
    CStruct * GetStructById(long id);
    void      CalcStructsLoad();
    CStruct * AddNewStruct(CStruct * pNewStruct);

    virtual void DebugPrint(CStr & sDest);

    long          Taxable;
    long          Peasants;
    CStr          TerrainType;
    CStr          CityName;
    CStr          CityType;
    CStr          FlagText[LAND_FLAG_COUNT];
    CBaseCollById Structs;
    CBaseCollById Units;
    CBaseColl     UnitsSeq; // this will keep units in the sequence they were met in the report
    CProductColl  Products;
    long          Flags;
    int           ExitBits;
    CPlane      * pPlane;
    int           AtlaclientsLastTurnNo;

    long          guiUnit;  // will be used by GUI only
    int           guiColor; // will be used by GUI only
};

//-----------------------------------------------------------------

class CPlane : public CBaseObject
{
public:
    CPlane() : Lands(32) {EastEdge=0;  WestEdge=0;   Width=0;
                          EdgeSrcId=0; EdgeExitId=0; EdgeDir=0; ExitsCount=0;
                         };
    virtual ~CPlane();
    
    CBaseCollById Lands;
    int           EastEdge;
    int           WestEdge;
    int           Width;

    long          EdgeSrcId ;
    long          EdgeExitId;
    int           EdgeDir   ;
    long          ExitsCount; // is used to determine 1.0.0 history file

};

//-----------------------------------------------------------------

class CShortNamedObj : public CBaseObject
{
public:
    CShortNamedObj() : CBaseObject() {Level = 0;};
    CStr ShortName;
    long Level; 

};

class CBattle : public CBaseObject
{
public:
    CStr LandStrId;
};

//-----------------------------------------------------------------

class CBaseCollByName : public CBaseCollById
{
public:
    CBaseCollByName()           : CBaseCollById()       {};
    CBaseCollByName(int nDelta) : CBaseCollById(nDelta) {};
protected:
    virtual int Compare(void * pItem1, void * pItem2) ;
};

//-----------------------------------------------------------------

class CUnitsByHex : public CSortedCollection
{
public:
    CUnitsByHex() : CSortedCollection() {};
    CUnitsByHex(int nDelta) : CSortedCollection(nDelta) {};
protected:
    virtual void FreeItem(void * pItem) {};
    virtual int Compare(void * pItem1, void * pItem2) 
    {
        CUnit * p1 = (CUnit*)pItem1;
        CUnit * p2 = (CUnit*)pItem2;

        if (p1->LandId > p2->LandId)
            return 1;
        else
            if (p1->LandId < p2->LandId)
                return -1;
            else
                if (p1->Id > p2->Id)
                    return 1;
                else
                    if (p1->Id < p2->Id)
                        return -1;
                    else
                        return 0;
    };
};

//-----------------------------------------------------------------

long LandCoordToId(int x, int y, int z);
void LandIdToCoord(long id, int & x, int & y, int & z);
void TestLandId();

#endif
