/*
 * This source file is part of the Atlantis Little Helper program.
 * Copyright (C) 2001 Maxim Shariy.
 *
 * Atlantis Little Helper is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Atlantis Little Helper is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Atlantis Little Helper; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "stdhdr.h"
#include "wx/splitter.h"
#include "wx/listctrl.h"

#include "cstr.h"
#include "collection.h"
#include "cfgfile.h"
#include "files.h"
#include "atlaparser.h"
#include "consts.h"
#include "consts_ah.h"
#include "hash.h"

#include "ahapp.h"
#include "ahframe.h"
#include "mapframe.h"
#include "unitframe.h"
#include "mappane.h"
#include "editpane.h"
#include "listpane.h"
#include "unitpane.h"
#include "utildlgs.h"
#include "unitpanefltr.h"

//#include "unitframefltr.h"


// define this to use XPMs everywhere (by default, BMPs are used under Win)
#ifdef __WXMSW__
    #define USE_XPM_BITMAPS 0
#else
    #define USE_XPM_BITMAPS 1
#endif


#if USE_XPM_BITMAPS
    #include "bitmaps/zoomin.xpm"   
    #include "bitmaps/zoomout.xpm"  
    #include "bitmaps/centerout.xpm"
    #include "bitmaps/prevzoom.xpm" 
    #include "bitmaps/showcoord.xpm"
    #include "bitmaps/planeup.xpm"  
    #include "bitmaps/planedwn.xpm" 
    #include "bitmaps/nextturn.xpm" 
    #include "bitmaps/prevturn.xpm" 
    #include "bitmaps/lastturn.xpm" 
    #include "bitmaps/lastvisitturn.xpm" 
#endif 


BEGIN_EVENT_TABLE(CMapFrame, CAhFrame)
    
    EVT_MENU     (menu_LoadReport         , CMapFrame::OnLoadReport            )
    EVT_MENU     (menu_JoinReport         , CMapFrame::OnJoinReport            )
    EVT_MENU     (menu_LoadOrders         , CMapFrame::OnLoadOrders            )
    EVT_MENU     (menu_SaveOrders         , CAhFrame ::OnSaveOrders            )
    EVT_MENU     (menu_SaveOrdersAs       , CMapFrame::OnSaveOrdersAs          )
    EVT_MENU     (menu_Options            , CMapFrame::OnOptions               )
    EVT_MENU     (menu_ViewSkillsAll      , CMapFrame::OnViewSkillsAll         )
    EVT_MENU     (menu_ViewSkillsNew      , CMapFrame::OnViewSkillsNew         )
    EVT_MENU     (menu_ViewItemsAll       , CMapFrame::OnViewItemsAll          )
    EVT_MENU     (menu_ViewItemsNew       , CMapFrame::OnViewItemsNew          )
    EVT_MENU     (menu_ViewObjectsAll     , CMapFrame::OnViewObjectsAll        )
    EVT_MENU     (menu_ViewObjectsNew     , CMapFrame::OnViewObjectsNew        )
    EVT_MENU     (menu_ViewBattlesAll     , CMapFrame::OnViewBattlesAll        )
    EVT_MENU     (menu_ViewEvents         , CMapFrame::OnViewEvents            )
    EVT_MENU     (menu_ViewErrors         , CMapFrame::OnViewErrors            )
    EVT_MENU     (menu_ViewGates          , CMapFrame::OnViewGates             )
    EVT_MENU     (menu_ViewFactionInfo    , CMapFrame::OnViewFactionInfo       )
    EVT_MENU     (menu_ViewFactionOverview, CMapFrame::OnViewFactionOverview)

    EVT_MENU     (menu_WindowUnits        , CMapFrame::OnWindowUnits           )
    EVT_MENU     (menu_WindowMessages     , CMapFrame::OnWindowMessages        )
    EVT_MENU     (menu_WindowEditors      , CMapFrame::OnWindowEditors         )
    EVT_MENU     (menu_WindowUnitsFltr    , CMapFrame::OnWindowUnitsFltr       )
    
    EVT_MENU     (menu_ListColUnits       , CMapFrame::OnListCol               )
    EVT_MENU     (menu_ListColUnitsFltr   , CMapFrame::OnListCol               )
                                                         
    EVT_MENU     (menu_ApplyDefaultOrders , CMapFrame::OnApplyDefaultOrders    )
    EVT_MENU     (menu_WriteMagesCSV      , CMapFrame::OnWriteMagesCSV         )
    EVT_MENU     (menu_ExportHexes        , CMapFrame::OnExportHexes           )
    EVT_MENU     (menu_CheckMonthLongOrd  , CMapFrame::OnCheckMonthLongOrd     )
    EVT_MENU     (menu_CheckTaxTrade      , CMapFrame::OnCheckTaxTrade         )
    
    
    EVT_MENU     (menu_Quit               , CMapFrame::OnQuit                  )
    EVT_MENU     (menu_QuitNoSave         , CMapFrame::OnQuitNoSave            )
    EVT_MENU     (menu_About              , CMapFrame::OnAbout                 )

    EVT_UPDATE_UI(menu_ViewBattlesAll     , CMapFrame::OnViewBattlesAllUpdate  )
    EVT_UPDATE_UI(menu_ViewSkillsAll      , CMapFrame::OnViewSkillsAllUpdate   )
    EVT_UPDATE_UI(menu_ViewSkillsNew      , CMapFrame::OnViewSkillsNewUpdate   )
    EVT_UPDATE_UI(menu_ViewItemsAll       , CMapFrame::OnViewItemsAllUpdate    )
    EVT_UPDATE_UI(menu_ViewItemsNew       , CMapFrame::OnViewItemsNewUpdate    )
    EVT_UPDATE_UI(menu_ViewObjectsAll     , CMapFrame::OnViewObjectsAllUpdate  )
    EVT_UPDATE_UI(menu_ViewObjectsNew     , CMapFrame::OnViewObjectsNewUpdate  )
    EVT_UPDATE_UI(menu_ViewEvents         , CMapFrame::OnViewEventsUpdate      )
    EVT_UPDATE_UI(menu_ViewErrors         , CMapFrame::OnViewErrorsUpdate      )

    EVT_MENU_RANGE      (tool_zoomin, tool_lastturn, CMapFrame::OnToolbarCmd   )
    EVT_UPDATE_UI_RANGE (tool_zoomin, tool_lastturn, CMapFrame::OnToolbarUpdate)

    EVT_CLOSE    (CMapFrame::OnCloseWindow)

END_EVENT_TABLE()




//--------------------------------------------------------------------

CMapFrame::CMapFrame(wxWindow * parent, int layout)
          :CAhFrame (parent, "", wxDEFAULT_FRAME_STYLE | wxCLIP_CHILDREN ) 
{
#ifdef __WXMAC__
    // we need this in order to allow the about menu relocation, since ABOUT is
    // not the default id of the about menu
    wxApp::s_macAboutMenuItemId = menu_About;
#endif

    MakeMenu(layout);
    MakeToolBar();
}


//--------------------------------------------------------------------

void CMapFrame::MakeMenu(int layout)
{
    wxMenuBar * menuBar;
    wxMenu    * menuItem;
//    wxMenu    * menuSubItem;

    menuBar = new wxMenuBar();

    menuItem = new wxMenu("", wxMENU_TEAROFF);
    menuItem->Append(menu_LoadReport        , "Load Report"                 , "" );
    menuItem->Append(menu_JoinReport        , "Join Report"                 , "" );
    menuItem->Append(menu_LoadOrders        , "Load Orders"                 , "" );
    menuItem->AppendSeparator();                                            
    menuItem->Append(menu_SaveOrders        , "Save Orders\tCtrl-S"         , "" );
    menuItem->Append(menu_SaveOrdersAs      , "Save Orders As"              , "" );
    menuItem->AppendSeparator();                                            
    menuItem->Append(menu_Quit              , "Exit\tAlt-X"                 , "" );
    menuItem->Append(menu_QuitNoSave        , "Quit and Discard Changes"    , "" );
    menuBar->Append(menuItem, "&File");                                     
                                                                            
    menuItem = new wxMenu;                                                  
    menuItem->Append(menu_ViewFactionInfo   , "Faction Info"                , "" );
    menuItem->Append(menu_ViewFactionOverview, "Faction Overview"           , "" );
    menuItem->AppendSeparator();                                            
    menuItem->Append(menu_ViewSkillsAll     , "All Skills"                  , "" );
    menuItem->Append(menu_ViewSkillsNew     , "New Skills"                  , "" );
    menuItem->Append(menu_ViewItemsAll      , "All Items"                   , "" );
    menuItem->Append(menu_ViewItemsNew      , "New Items"                   , "" );
    menuItem->Append(menu_ViewObjectsAll    , "All Objects"                 , "" );
    menuItem->Append(menu_ViewObjectsNew    , "New Objects"                 , "" );
    menuItem->Append(menu_ViewBattlesAll    , "All Battles"                 , "" );
    menuItem->AppendSeparator();                                            
    menuItem->Append(menu_ViewGates         , "Gates"                       , "" );
    menuItem->Append(menu_ViewEvents        , "Non-unit Events"             , "" );
    menuItem->Append(menu_ViewErrors        , "All Errors"                  , "" );
    menuItem->AppendSeparator();                                            
    menuItem->Append(menu_Options           , "Options"                     , "" );
    menuBar->Append(menuItem, "&View");                                     


    menuItem = new wxMenu;                                                  
    menuItem->Append(menu_ApplyDefaultOrders, "Apply default orders"        , "");
    menuItem->Append(menu_CheckMonthLongOrd , "Check month long orders"     , "");
    menuItem->Append(menu_CheckTaxTrade     , "Check tax, trade for next turn", "");
    menuItem->Append(menu_WriteMagesCSV     , "Export mages data to CSV file", "");
    menuItem->Append(menu_ExportHexes       , "Export Hexes"                , "");
    menuBar->Append(menuItem, "&Actions");

    

    menuItem = new wxMenu;
    menuItem->Append(menu_WindowUnits       , "Units (Hex)"                 , "");
    menuItem->Append(menu_WindowUnitsFltr   , "Unit Locator"                , "");
    menuItem->Append(menu_WindowMessages    , "Messages"                    , "");
    if (AH_LAYOUT_3_WIN==layout)                                            
        menuItem->Append(menu_WindowEditors , "Editors"                     , "");
        
//    menuSubItem = new wxMenu;
//    menuSubItem->Append(menu_ListColUnits    , "Units (Hex)"                 , "");
//    menuSubItem->Append(menu_ListColUnitsFltr, "Unit Locator"                , "");
//    menuItem->   Append(menu_ListColumns     , "List columns", menuSubItem   , "");
    menuItem->AppendSeparator();                                            
    menuItem->Append(menu_ListColUnits      , "Columns for Units (Hex)"     , "");
    menuItem->Append(menu_ListColUnitsFltr  , "Columns for Unit Locator"    , "");
        
    menuBar->Append(menuItem, "&Windows");                                  
            
    
    menuItem = new wxMenu;                                                  
    menuItem->Append(menu_About             , "About"                       , "");
    menuBar->Append(menuItem, "&Help");

    SetMenuBar(menuBar);
}

//--------------------------------------------------------------------

void CMapFrame::MakeToolBar()
{
#define TOOLCOUNT  11

    wxToolBar  * toolBar;
    wxBitmap     toolBarBitmap[TOOLCOUNT];
    const char * toolTip      [TOOLCOUNT];
    int          i;


    // if changing order, change tool_xxx constants in consts_ah.h accordingly
    toolBarBitmap[ 0] = wxBITMAP(zoomin         );     toolTip[ 0] = "Zoom In"            ;
    toolBarBitmap[ 1] = wxBITMAP(zoomout        );     toolTip[ 1] = "Zoom Out"           ;
    toolBarBitmap[ 2] = wxBITMAP(centerout      );     toolTip[ 2] = "Zoom Out and Center";
    toolBarBitmap[ 3] = wxBITMAP(prevzoom       );     toolTip[ 3] = "Prev Zoom Level"    ;
    toolBarBitmap[ 4] = wxBITMAP(showcoord      );     toolTip[ 4] = "Coordinates"        ;
    toolBarBitmap[ 5] = wxBITMAP(planeup        );     toolTip[ 5] = "Plane Up"           ;
    toolBarBitmap[ 6] = wxBITMAP(planedwn       );     toolTip[ 6] = "Plane Down"         ;
    toolBarBitmap[ 7] = wxBITMAP(lastvisitturn  );     toolTip[ 7] = "Last Turn Visited"  ;
    toolBarBitmap[ 8] = wxBITMAP(prevturn       );     toolTip[ 8] = "Prev Turn"          ;
    toolBarBitmap[ 9] = wxBITMAP(nextturn       );     toolTip[ 9] = "Next Turn"          ;
    toolBarBitmap[10] = wxBITMAP(lastturn       );     toolTip[10] = "Last Turn"          ;

    toolBar = CreateToolBar(wxNO_BORDER | wxTB_FLAT | wxTB_VERTICAL);

    for (i=0; i<TOOLCOUNT; i++)
        toolBar->AddTool(tool_zoomin+i, toolBarBitmap[i], toolTip[i]);

    toolBar->Realize();
    toolBar->SetRows(TOOLCOUNT+1);
}


//--------------------------------------------------------------------

const char * CMapFrame::GetConfigSection(int layout)
{
    switch (layout)
    {
    case AH_LAYOUT_2_WIN:        return SZ_SECT_WND_MAP_2_WIN;
    case AH_LAYOUT_3_WIN:        return SZ_SECT_WND_MAP_3_WIN;
    default:                     return "";
    }
}


//--------------------------------------------------------------------

void CMapFrame::Init(int layout, const char * szConfigSection)
{
    CMapPane          * p1;
    CEditPane         * p2;
    long                y0;


    szConfigSection = GetConfigSection(layout);

    switch (layout)
    {
    case AH_LAYOUT_2_WIN:
        CAhFrame::Init(layout, szConfigSection);

        y0  = atol(gpApp->GetConfig(szConfigSection, SZ_KEY_HEIGHT_0));
        m_Splitter = new wxSplitterWindow(this, -1, wxDefaultPosition, wxDefaultSize, 
                                          wxSP_3D | wxCLIP_CHILDREN);

        p1 = new CMapPane (m_Splitter, -1, layout  );
        p2 = new CEditPane(m_Splitter, NULL, FALSE, FALSE);

        SetPane(AH_PANE_MAP       , p1);
        SetPane(AH_PANE_MAP_DESCR , p2);

        p1->Init(this);
        p2->Init();

        m_Splitter->SetMinimumPaneSize(2);
        m_Splitter->SplitHorizontally(p1, p2, y0);
        break;

    case AH_LAYOUT_3_WIN:
        CAhFrame::Init(layout, szConfigSection);

        p1 = new CMapPane (this, -1, layout  );
        SetPane(AH_PANE_MAP, p1);
        p1->Init(this);
        break;
    }
    
}


//--------------------------------------------------------------------

void CMapFrame::Done(BOOL SetClosedFlag)
{
    CMapPane    * pMapPane;

    switch (m_Layout)
    {
    case AH_LAYOUT_2_WIN:
        gpApp->SetConfig(m_sConfigSection.GetData(), SZ_KEY_HEIGHT_0, m_Splitter->GetSashPosition());
        break;

    case AH_LAYOUT_3_WIN:
        break;
    }

    pMapPane  = (CMapPane* )gpApp->m_Panes[AH_PANE_MAP];
    if (pMapPane)
        pMapPane->Done();

    CAhFrame::Done(SetClosedFlag);
}

//--------------------------------------------------------------------

void CMapFrame::OnQuit(wxCommandEvent& WXUNUSED(event))
{
    // TRUE is to force the frame to close
    Close(TRUE);
}


//--------------------------------------------------------------------

void CMapFrame::OnQuitNoSave(wxCommandEvent& WXUNUSED(event))
{
    gpApp->m_DiscardChanges = TRUE;

    // TRUE is to force the frame to close
    Close(TRUE);
}

//--------------------------------------------------------------------

void CMapFrame::OnAbout(wxCommandEvent& WXUNUSED(event))
{
    wxString msg;
//    msg.Printf("Atlantis Little Helpern\nVersion %s.\n\nPlease read doc/readme.html for more information.", AH_VERSION_2);
    msg.Printf("Atlantis Little Helper version %s.\n\nBuilt using %s.\n\n"
               "Please read doc/readme.html for more information.", 
               AH_VERSION_2,
               wxVERSION_STRING );

    wxMessageBox(msg, "About", wxOK | wxICON_INFORMATION, this);
}

//--------------------------------------------------------------------

void CMapFrame::OnToolbarCmd(wxCommandEvent& event)
{
    if (gpApp->m_Panes[AH_PANE_MAP])
        ((CMapPane*)gpApp->m_Panes[AH_PANE_MAP])->OnToolbarCmd(event);
}

//--------------------------------------------------------------------

void CMapFrame::OnToolbarUpdate(wxUpdateUIEvent& event)
{
    BOOL Ok = false;
    if (gpApp->m_Panes[AH_PANE_MAP])
        Ok = ((CMapPane*)gpApp->m_Panes[AH_PANE_MAP])->IsToolActive(event);

    event.Enable(FALSE!=Ok); // otherwise that bitch VC++, well, bitches
}

//--------------------------------------------------------------------

void CMapFrame::OnViewBattlesAllUpdate(wxUpdateUIEvent& event)
{
    event.Enable(gpApp->m_pAtlantis->m_Battles.Count()>0);
}


//--------------------------------------------------------------------

void CMapFrame::OnViewSkillsAllUpdate(wxUpdateUIEvent& event)
{
    const char  * szName;
    const char  * szValue;
    int           sectidx;

    sectidx = gpApp->GetSectionFirst(SZ_SECT_SKILLS, szName, szValue);
    event.Enable(sectidx>=0);
}

//--------------------------------------------------------------------

void CMapFrame::OnViewSkillsNewUpdate(wxUpdateUIEvent& event)
{
    event.Enable(gpApp->m_pAtlantis->m_Skills.Count()>0);
}


//--------------------------------------------------------------------

void CMapFrame::OnViewItemsAllUpdate(wxUpdateUIEvent& event)
{
    const char  * szName;
    const char  * szValue;
    int           sectidx;

    sectidx = gpApp->GetSectionFirst(SZ_SECT_ITEMS, szName, szValue);
    event.Enable(sectidx>=0);
}

//--------------------------------------------------------------------

void CMapFrame::OnViewItemsNewUpdate(wxUpdateUIEvent& event)
{
    event.Enable(gpApp->m_pAtlantis->m_Items.Count()>0);
}

//--------------------------------------------------------------------

void CMapFrame::OnViewObjectsAllUpdate(wxUpdateUIEvent& event)
{
    const char  * szName;
    const char  * szValue;
    int           sectidx;

    sectidx = gpApp->GetSectionFirst(SZ_SECT_OBJECTS, szName, szValue);
    event.Enable(sectidx>=0);
}

//--------------------------------------------------------------------

void CMapFrame::OnViewObjectsNewUpdate(wxUpdateUIEvent& event)
{
    event.Enable(gpApp->m_pAtlantis->m_Objects.Count()>0);
}

//--------------------------------------------------------------------

void CMapFrame::OnViewEventsUpdate(wxUpdateUIEvent& event)
{
    event.Enable(!gpApp->m_pAtlantis->m_Events.Description.IsEmpty());
}

//--------------------------------------------------------------------

void CMapFrame::OnViewErrorsUpdate(wxUpdateUIEvent& event)
{
    event.Enable(!gpApp->m_pAtlantis->m_Errors.Description.IsEmpty());
}

//--------------------------------------------------------------------

void CMapFrame::OnCloseWindow(wxCloseEvent& event)
{
    // This is the cental shutdown point!
    if (gpApp->CanCloseApp())
    {
        gpApp->FrameClosing(this);
        Destroy();
    }
    else
        wxMessageBox("Can not save orders or save cancelled.");
}

//--------------------------------------------------------------------

void CMapFrame::OnLoadReport(wxCommandEvent& WXUNUSED(event))
{
    gpApp->LoadReport(FALSE);
}

//--------------------------------------------------------------------

void CMapFrame::OnJoinReport(wxCommandEvent& WXUNUSED(event))
{
    gpApp->LoadReport(TRUE);
}

//--------------------------------------------------------------------

void CMapFrame::OnLoadOrders(wxCommandEvent& WXUNUSED(event))
{
    gpApp->LoadOrders();
}

//--------------------------------------------------------------------

void CMapFrame::OnSaveOrdersAs(wxCommandEvent& WXUNUSED(event))
{
    gpApp->SaveOrders(FALSE);
}

//--------------------------------------------------------------------

void CMapFrame::OnOptions(wxCommandEvent& event)
{
    gpApp->OpenOptionsDlg();
}

//--------------------------------------------------------------------

void CMapFrame::OnViewSkillsAll(wxCommandEvent& event)
{
    gpApp->ViewShortNamedObjects(TRUE, SZ_SECT_SKILLS, "Skills", gpApp->m_pAtlantis->m_Skills);
}

//--------------------------------------------------------------------

void CMapFrame::OnViewSkillsNew(wxCommandEvent& event)
{
    gpApp->ViewShortNamedObjects(FALSE, SZ_SECT_SKILLS, "Skills", gpApp->m_pAtlantis->m_Skills);
}

//--------------------------------------------------------------------

void CMapFrame::OnViewItemsAll(wxCommandEvent& event)
{
    gpApp->ViewShortNamedObjects(TRUE, SZ_SECT_ITEMS, "Items", gpApp->m_pAtlantis->m_Items);
}

//--------------------------------------------------------------------

void CMapFrame::OnViewItemsNew(wxCommandEvent& event)
{
    gpApp->ViewShortNamedObjects(FALSE, SZ_SECT_ITEMS, "Items", gpApp->m_pAtlantis->m_Items);
}

//--------------------------------------------------------------------

void CMapFrame::OnViewObjectsAll(wxCommandEvent& event)
{
    gpApp->ViewShortNamedObjects(TRUE, SZ_SECT_OBJECTS, "Objects", gpApp->m_pAtlantis->m_Objects);
}

//--------------------------------------------------------------------

void CMapFrame::OnViewObjectsNew(wxCommandEvent& event)
{
    gpApp->ViewShortNamedObjects(FALSE, SZ_SECT_OBJECTS, "Objects", gpApp->m_pAtlantis->m_Objects);
}

//--------------------------------------------------------------------

void CMapFrame::OnViewBattlesAll(wxCommandEvent& event)
{
    gpApp->ViewBattlesAll();
}

//--------------------------------------------------------------------

void CMapFrame::OnViewEvents(wxCommandEvent& event)
{
    gpApp->ViewEvents(TRUE);
}

//--------------------------------------------------------------------

void CMapFrame::OnViewErrors(wxCommandEvent& event)
{
    gpApp->ViewEvents(FALSE);
}

//--------------------------------------------------------------------

void CMapFrame::OnViewGates(wxCommandEvent& event)
{
    gpApp->ViewGates();
}

//--------------------------------------------------------------------

void CMapFrame::OnViewFactionInfo(wxCommandEvent& event)
{
    gpApp->ViewFactionInfo();
}

//--------------------------------------------------------------------

void CMapFrame::OnViewFactionOverview(wxCommandEvent& event)
{
    gpApp->ViewFactionOverview();
}

//--------------------------------------------------------------------

void CMapFrame::OnWindowUnits(wxCommandEvent& event)
{
    gpApp->OpenUnitFrame();
}

//--------------------------------------------------------------------

void CMapFrame::OnWindowUnitsFltr(wxCommandEvent& event)
{
    gpApp->OpenUnitFrameFltr(TRUE);
}

//--------------------------------------------------------------------

void CMapFrame::OnWindowMessages(wxCommandEvent& event)
{
    gpApp->OpenMsgFrame();
}

//--------------------------------------------------------------------

void CMapFrame::OnWindowEditors(wxCommandEvent& event)
{
    gpApp->OpenEditsFrame();
}

//--------------------------------------------------------------------

void CMapFrame::OnApplyDefaultOrders(wxCommandEvent& event)
{
    gpApp->SetOrdersChanged(gpApp->m_pAtlantis->ApplyDefaultOrders(TRUE) //(BOOL)atol(gpApp->GetConfig(SZ_SECT_COMMON, SZ_KEY_DEFAULT_EMPTY_ONLY)))
                            || gpApp->GetOrdersChanged());
    
    CUnitPane * p = (CUnitPane*)gpApp->m_Panes[AH_PANE_UNITS_HEX];
    if (p)
        p->Update(p->m_pCurLand);

}

//--------------------------------------------------------------------

void CMapFrame::OnWriteMagesCSV(wxCommandEvent& event)
{
    gpApp->WriteMagesCSV();
}

//--------------------------------------------------------------------

void CMapFrame::OnCheckMonthLongOrd(wxCommandEvent& event)
{
    gpApp->CheckMonthLongOrders();
}

//--------------------------------------------------------------------

void CMapFrame::OnCheckTaxTrade(wxCommandEvent& event)
{
    gpApp->CheckTaxTrade();
}

//--------------------------------------------------------------------

void CMapFrame::OnExportHexes(wxCommandEvent& event)
{
    gpApp->ExportHexes();
}

//--------------------------------------------------------------------

void CMapFrame::OnListCol(wxCommandEvent& event)
{
    gpApp->EditListColumns(event.GetId());
}

//--------------------------------------------------------------------


