/*
 * This source file is part of the Atlantis Little Helper program.
 * Copyright (C) 2001 Maxim Shariy.
 *
 * Atlantis Little Helper is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Atlantis Little Helper is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Atlantis Little Helper; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "stdhdr.h"

#include "wx/listctrl.h"
#include "wx/dialog.h"


#include "cstr.h"
#include "collection.h"
#include "cfgfile.h"
#include "files.h"
#include "atlaparser.h"
#include "consts.h"
#include "consts_ah.h"
#include "hash.h"

#include "ahapp.h"
#include "ahframe.h"
#include "data.h"
#include "mappane.h"
#include "editpane.h"
#include "listpane.h"
#include "unitpane.h"
#include "atlaparser.h"

#include <math.h>



const double     cos30               = sqrt(3)/2;
const double     tan30               = 1/sqrt(3);
int              SmallHexesCount     = 2; 
int              VerySmallHexesCount = 1; 


static wxPoint   glb_PopupPoint;
CLand          * glb_pPopupLand = NULL;

#define          MAX_DP_BORDER      1

#define          COORD_PANE_WIDTH  24
#define          COORD_PANE_HEIGHT 24


BEGIN_EVENT_TABLE(CMapPane, wxWindow)
    EVT_PAINT            (                      CMapPane::OnPaint            )
    EVT_ERASE_BACKGROUND (                      CMapPane::OnEraseBackground  )
    EVT_LEFT_DOWN        (                      CMapPane::OnMouseEvent       )
    EVT_LEFT_UP          (                      CMapPane::OnMouseEvent       )
    EVT_MOTION           (                      CMapPane::OnMouseEvent       )
    EVT_LEAVE_WINDOW     (                      CMapPane::OnMouseEvent       )
    EVT_ENTER_WINDOW     (                      CMapPane::OnMouseEvent       )
    EVT_RIGHT_DOWN       (                      CMapPane::OnMouseEvent       )
    EVT_LEFT_DCLICK      (                      CMapPane::OnMouseEvent       )

    EVT_MENU             (menu_Popup_Flag     , CMapPane::OnPopupMenuFlag    )
    EVT_MENU             (menu_Popup_Center   , CMapPane::OnPopupMenuCenter  )
    EVT_MENU             (menu_Popup_Battles  , CMapPane::OnPopupMenuBattles )

    
    
END_EVENT_TABLE()



//--------------------------------------------------------------------------

CMapPane::CMapPane(wxWindow * parent, wxWindowID id, int layout)
         :wxWindow(parent, id, wxDefaultPosition, wxDefaultSize, 
                   (AH_LAYOUT_3_WIN==layout)?wxSUNKEN_BORDER:0 )
{
    int i;

    m_pPen             = new wxPen;
    m_pPenWall         = new wxPen;
    m_pPenSel          = new wxPen;
    m_pPenRoad         = new wxPen;
    m_pPenRoadBad      = new wxPen;
    m_pPenBlack        = new wxPen(*wxBLACK, 1, wxSOLID);
    m_pPenWhite        = new wxPen(*wxWHITE, 1, wxSOLID);
    m_pPenRed          = new wxPen(*wxRED  , 1, wxSOLID);
    
    for (i=0; i<LAND_FLAG_COUNT; i++)
        m_pPenFlag[i]  = new wxPen;

    m_pBrushBlack      = new wxBrush(*wxBLACK, wxSOLID);
    m_pBrushWhite      = new wxBrush(*wxWHITE, wxSOLID);
    m_pBrushRed        = new wxBrush(*wxRED  , wxSOLID);
    
    m_HexSizeIdx       = -1;
    m_HexSizeIdxOld    = -1;
    m_ShowState        = 0;
    m_MinSelMen        = 0;
    m_pCities          = new CBaseCollById(32);
    m_pTrackHexes      = new CLongColl(8);
    m_pPopupLand       = NULL;
    m_UnknownColorIdx  = -1;
    m_Hatch            = FALSE;

    m_Rect_IsPaused    = false;
    m_Rect_IsDragging  = false;
    m_Rect_x1          = 0;
    m_Rect_y1          = 0;
    m_Rect_x2          = 0;
    m_Rect_y2          = 0;
}

//--------------------------------------------------------------------------

CMapPane::~CMapPane()
{
    int i;

    delete m_pPen;
    delete m_pPenWall;
    delete m_pPenSel;
    delete m_pPenRoad;
    delete m_pPenRoadBad;
    delete m_pPenBlack;   
    delete m_pPenWhite;   
    delete m_pPenRed;   

    for (i=0; i<LAND_FLAG_COUNT; i++)
        delete m_pPenFlag[i];

    delete m_pBrushBlack;
    delete m_pBrushWhite;
    delete m_pBrushRed  ;

    m_pCities->DeleteAll();
    delete m_pCities;
    delete m_pTrackHexes;

    m_TerrainBrushes.FreeAll();
    m_TerrainNames  .FreeAll();
}

//--------------------------------------------------------------------------


void CMapPane::Init(CAhFrame * pParentFrame)
{
    const char * p;
    long         x;
    CStr         S;

    m_pFrame        = pParentFrame;

    m_ShowState = atol(gpApp->GetConfig(SZ_SECT_MAP_PANE, SZ_KEY_STATE));
    m_MinSelMen = atol(gpApp->GetConfig(SZ_SECT_COMMON  , SZ_KEY_MIN_SEL_MEN));

    p = gpApp->GetConfig(SZ_SECT_COMMON, SZ_KEY_HEX_SIZE_LIST);
    while (p && *p)
    {
        p = SkipSpaces(S.GetToken(p, ','));
        x = atol(S.GetData());
        x = (x/2)*2;  // make it even!
        if (x>0)
            m_HexSizes.Insert((void*)x);
    }
    if (0==m_HexSizes.Count())
        m_HexSizes.Insert((void*)24);

    m_SelPlane             = atol(gpApp->GetConfig(SZ_SECT_MAP_PANE  , SZ_KEY_PLANE_SEL       ));

    LoadPlaneConfig();
    ApplyColors();
    //UpdateEditPane();
    gpApp->OnMapSelectionChange();
}

//--------------------------------------------------------------------------

void CMapPane::Done()
{
    gpApp->SetConfig(SZ_SECT_MAP_PANE ,   SZ_KEY_STATE          , m_ShowState);   
    gpApp->SetConfig(SZ_SECT_MAP_PANE ,   SZ_KEY_PLANE_SEL      , m_SelPlane );   

    SavePlaneConfig();
}

//--------------------------------------------------------------------------

void CMapPane::SavePlaneConfig()
{
    CStr   sSection;

    sSection << "PLANE_" << m_SelPlane;

    gpApp->SetConfig(sSection.GetData(), SZ_KEY_HEX_SIZE       , m_HexSizeIdx               );
    gpApp->SetConfig(sSection.GetData(), SZ_KEY_HEX_SIZE_OLD   , m_HexSizeIdxOld            );
    gpApp->SetConfig(sSection.GetData(), SZ_KEY_ATLA_X0        , m_AtlaX0                   );
    gpApp->SetConfig(sSection.GetData(), SZ_KEY_ATLA_Y0        , m_AtlaY0                   );
    gpApp->SetConfig(sSection.GetData(), SZ_KEY_HEX_SEL_X      , m_SelHexX                  );   
    gpApp->SetConfig(sSection.GetData(), SZ_KEY_HEX_SEL_Y      , m_SelHexY                  );   

}   
    
//-------------------------------------------------------------------------

void CMapPane::LoadPlaneConfig()
{
    CStr   sSection;
    long   x;

    sSection << "PLANE_" << m_SelPlane;

    x                      = atol(gpApp->GetConfig(sSection.GetData(), SZ_KEY_HEX_SIZE        ));
    SetHexSize(x);                     
    m_HexSizeIdxOld        = atol(gpApp->GetConfig(sSection.GetData(), SZ_KEY_HEX_SIZE_OLD    ));
    m_AtlaX0               = atol(gpApp->GetConfig(sSection.GetData(), SZ_KEY_ATLA_X0         ));
    m_AtlaY0               = atol(gpApp->GetConfig(sSection.GetData(), SZ_KEY_ATLA_Y0         ));
    m_SelHexX              = atol(gpApp->GetConfig(sSection.GetData(), SZ_KEY_HEX_SEL_X       ));   
    m_SelHexY              = atol(gpApp->GetConfig(sSection.GetData(), SZ_KEY_HEX_SEL_Y       ));   
}


//-------------------------------------------------------------------------

void CMapPane::ApplyFonts()
{
    Refresh(FALSE);
}

//-------------------------------------------------------------------------

void CMapPane::ApplyOneColor(wxColour & cr, const char * name)
{
    wxBrush    * pBrush; 

    m_TerrainNames.Insert(strdup(name));

    if (0==stricmp(SZ_KEY_MAP_UNKNOWN, name))
        m_UnknownColorIdx = m_TerrainBrushes.Count();

    pBrush = new wxBrush(cr, wxSOLID);
    m_TerrainBrushes.Insert(pBrush);

    cr.Set(cr.Red() >> 1, cr.Green()  >> 1, cr.Blue() >> 1);
    pBrush = new wxBrush(cr, wxCROSSDIAG_HATCH);
    m_TerrainBrushes.Insert(pBrush);
}

//-------------------------------------------------------------------------

void CMapPane::ApplyColors()
{
    static const char * FlagColorName[LAND_FLAG_COUNT] = {SZ_KEY_MAP_FLAG_1, SZ_KEY_MAP_FLAG_2, SZ_KEY_MAP_FLAG_3};
    static wxDash dash[2];
    
    wxColour     cr;
    long         x;
    int          idx;
    const char * name;
    const char * value;
    int          i,n;
    int          flagwidth;
    CLand      * pLand;
    CPlane     * pPlane;


    delete m_pPen;  
    StrToColor(&cr,   gpApp->GetConfig(SZ_SECT_COLORS, SZ_KEY_MAP_GRID)    );
    m_pPen = new wxPen(cr, 1, wxSOLID);
    
    x = atol(gpApp->GetConfig(SZ_SECT_MAP_PANE, SZ_KEY_WALL_WIDTH));
    if (x<1) x=1;
    if (x>5) x=5;
    delete m_pPenWall;  
    m_pPenWall = new wxPen(*wxBLACK, x, wxSOLID);

    delete m_pPenSel;   
    StrToColor(&cr,   gpApp->GetConfig(SZ_SECT_COLORS, SZ_KEY_MAP_GRID_SEL)    );
    m_pPenSel = new wxPen(cr, 2, wxSOLID);

    x = atol(gpApp->GetConfig(SZ_SECT_MAP_PANE, SZ_KEY_ROAD_WIDTH));
    if (x<1) x=1;
    if (x>6) x=6;
    delete m_pPenRoad;
    StrToColor(&cr,   gpApp->GetConfig(SZ_SECT_COLORS, SZ_KEY_MAP_ROAD)    );
    m_pPenRoad = new wxPen(cr, x, wxSOLID);

    delete m_pPenRoadBad;
    i = atol(gpApp->GetConfig(SZ_SECT_MAP_PANE, SZ_KEY_DASH_BAD_ROADS));
    if (i)
    {
        m_pPenRoadBad = new wxPen(cr, x, wxUSER_DASH); //wxSOLID);
        dash[0] = 1; dash[1] = 2;
        m_pPenRoadBad->SetDashes(2, dash);
    }
    else
    {
        StrToColor(&cr,   gpApp->GetConfig(SZ_SECT_COLORS, SZ_KEY_MAP_ROAD_BAD)    );
        m_pPenRoadBad = new wxPen(cr, x, wxSOLID);
    }
    
    flagwidth = atoi(gpApp->GetConfig(SZ_SECT_MAP_PANE, SZ_KEY_FLAG_WIDTH));
    for (i=0; i<LAND_FLAG_COUNT; i++)
    {
        delete m_pPenFlag[i];
        StrToColor(&cr,   gpApp->GetConfig(SZ_SECT_COLORS, FlagColorName[i])    );
        m_pPenFlag[i] = new wxPen(cr, flagwidth, wxSOLID);
    }


    m_TerrainBrushes.FreeAll();
    m_TerrainNames  .FreeAll();
    m_UnknownColorIdx  = -1;

    for (n=0; n<gpApp->m_pAtlantis->m_Planes.Count(); n++)
    {
        pPlane = (CPlane*)gpApp->m_pAtlantis->m_Planes.At(n);
        for (i=0; i<pPlane->Lands.Count(); i++)
        {
            pLand = (CLand*)pPlane->Lands.At(i);
            pLand->guiColor = -1;
        }
    }
    
    m_Hatch = atol(gpApp->GetConfig(SZ_SECT_COMMON,  SZ_KEY_HATCH_UNVISITED));

    idx = gpApp->GetSectionFirst(SZ_SECT_COLORS, name, value);
    while (idx>=0)
    {
        StrToColor(&cr, value);
        ApplyOneColor(cr, name);

        idx = gpApp->GetSectionNext (idx, SZ_SECT_COLORS, name, value);
    }

    Refresh(FALSE);
}

//--------------------------------------------------------------------------

wxBrush * CMapPane::GetLandBrush(CLand * pLand, BOOL GetHatched)
{
    int          idx, i;
    const char * name;
    wxColour     cr;
    int          Hatch;

    idx = pLand?pLand->guiColor:m_UnknownColorIdx;
    if (idx<0 || idx>=m_TerrainBrushes.Count())
    {
        idx = -1;
        name = pLand?pLand->TerrainType.GetData():SZ_KEY_MAP_UNKNOWN;

        for (i=0; i<m_TerrainNames.Count(); i++)
            if (0==stricmp(name, (const char*)m_TerrainNames.At(i)))
            {
                idx = i*2;
                break;
            }

        if (idx<0)
        {
            // totally new color, but try to read it, so it gets into config file
            StrToColor(&cr,   gpApp->GetConfig(SZ_SECT_COLORS, name) );
            Hatch = atol(gpApp->GetConfig(SZ_SECT_COMMON,  SZ_KEY_HATCH_UNVISITED));
            idx   = m_TerrainBrushes.Count();
            ApplyOneColor(cr, name);
        }

        if (pLand)
            pLand->guiColor   = idx;
        else
            m_UnknownColorIdx = idx; // should be set by this moment, but just in case.

    }
    if (GetHatched)
        idx++;

    return (wxBrush*)m_TerrainBrushes.At(idx);
}

//--------------------------------------------------------------------------

inline BOOL CMapPane::ValidHexNo(int NoX, int NoY)
{
    // sum of coordinates must be even

    int x = abs(NoX) + abs(NoY);

    return x == ((x>>1)<<1);
}


//--------------------------------------------------------------------------

int  CMapPane::NormalizeHexX(int NoX, CPlane * pPlane)
{
    if (pPlane && pPlane->Width>0)
    {
        while (NoX < pPlane->WestEdge)
            NoX += pPlane->Width;
        while (NoX > pPlane->EastEdge)
            NoX -= pPlane->Width;
    }
    return NoX;
}

//--------------------------------------------------------------------------

void CMapPane::GetHexNo(int & NoX, int & NoY, int WinX, int WinY)
{
    // center of (0,0) hex has 0,0 Atla coordinates

    long AtlaX, AtlaY;
    int  ApprX, ApprY; //approximite hex no
    int  x, y;
    int  x0, y0, x1, y1, x2, y2;

    WinToAtla(WinX, WinY, AtlaX, AtlaY);

    ApprX = AtlaX / (m_HexSize * 3 / 2);       // m_HexSize must be even!
    ApprY = AtlaY / m_HexHalfHeight;

    for (x = ApprX-1; x <= ApprX+1; x++)
        for (y = ApprY-2; y <= ApprY+2; y++)
            if ( ValidHexNo(x,y) &&
                (AtlaY >= y*m_HexHalfHeight - m_HexHalfHeight) &&
                (AtlaY <= y*m_HexHalfHeight + m_HexHalfHeight) )
            {
                // hex center
                x0 = x * m_HexSize * 3 / 2;
                y0 = y * m_HexHalfHeight;
                
                x1 = x0 - m_HexSize;
                y1 = y0;
                x2 = x0 - m_HexHalfSize;
                y2 = y0 - m_HexHalfHeight;
                if (AtlaY < y1 + (double)(y2-y1)/(x2-x1)*(AtlaX-x1))
                    continue;
            
                x1 = x0 + m_HexSize;
                y1 = y0;
                x2 = x0 + m_HexHalfSize;
                y2 = y0 - m_HexHalfHeight;
                if (AtlaY < y1 + (double)(y2-y1)/(x2-x1)*(AtlaX-x1))
                    continue;
            
              
                x1 = x0 - m_HexSize;
                y1 = y0;
                x2 = x0 - m_HexHalfSize;
                y2 = y0 + m_HexHalfHeight;
                if (AtlaY > y1 + (double)(y2-y1)/(x2-x1)*(AtlaX-x1))
                    continue;
            
                x1 = x0 + m_HexSize;
                y1 = y0;
                x2 = x0 + m_HexHalfSize;
                y2 = y0 + m_HexHalfHeight;
                if (AtlaY > y1 + (double)(y2-y1)/(x2-x1)*(AtlaX-x1))
                    continue;
            
                NoX = x;
                NoY = y;
                return;
            }
}

//--------------------------------------------------------------------------

void CMapPane::GetHexCenter(int NoX, int NoY, int & WinX, int & WinY)
{
    // center of (0,0) hex has 0,0 Atla coordinates

    long AtlaX, AtlaY;

    AtlaX = NoX * m_HexSize * 3 / 2;
    AtlaY = NoY * m_HexHalfHeight;

    AtlaToWin(WinX, WinY, AtlaX, AtlaY);
}

//--------------------------------------------------------------------------

inline void CMapPane::WinToAtla(int   WinX, int   WinY, long & AtlaX, long & AtlaY)
{
    AtlaX = WinX + m_AtlaX0;
    AtlaY = WinY + m_AtlaY0;
}

//--------------------------------------------------------------------------

inline void CMapPane::AtlaToWin(int & WinX, int & WinY, long   AtlaX, long   AtlaY)
{
    WinX = AtlaX - m_AtlaX0;
    WinY = AtlaY - m_AtlaY0;
}

//--------------------------------------------------------------------------

BOOL CMapPane::SetHexSize(int HexSizeIdx)
{
  //  wxRect rect;
    long CX, CY;
    long CXOld, CYOld;
    BOOL Changed;
    int  width, height;
    int  OldIdx   = m_HexSizeIdx;
    int  OldSize       = m_HexSize; //(long)m_HexSizes.At(m_HexSizeIdx);
    int  OldHalfHeight = m_HexHalfHeight;

    m_HexSizeIdx    = HexSizeIdx;
    if (m_HexSizeIdx < 0) 
        m_HexSizeIdx = 0;
    if (m_HexSizeIdx >= m_HexSizes.Count()) 
        m_HexSizeIdx = m_HexSizes.Count()-1;

    Changed = (OldIdx != m_HexSizeIdx);
    if (Changed)
    {
        m_HexSizeIdxOld = OldIdx;
        if (m_HexSizeIdxOld < 0)
            m_HexSizeIdxOld = m_HexSizeIdx;

        m_HexSize       = (long)m_HexSizes.At(m_HexSizeIdx);
        m_HexHalfSize   = m_HexSize / 2;
        m_HexHalfHeight = (int)(m_HexSize * cos30 + 0.5);    
        m_HexHeight     = m_HexHalfHeight * 2;

        if (OldIdx>=0)
        {
            GetClientSize(&width, &height);

            WinToAtla(width/2, height/2, CXOld, CYOld);
            CX = (long)( (double)CXOld * (double)m_HexSize       / (double)OldSize       + 0.5);
            CY = (long)( (double)CYOld * (double)m_HexHalfHeight / (double)OldHalfHeight + 0.5);

            m_AtlaX0 = CX - width/2;
            m_AtlaY0 = CY - height/2;

            //m_AtlaX0 += CXOld*((double)m_HexSize/(double)OldSize - 1);
            //m_AtlaY0 += CYOld*((double)m_HexSize/(double)OldSize - 1);;
        }
    }

    return Changed;
}

//--------------------------------------------------------------------------

void CMapPane::DrawHexTest(int NoX, int NoY, wxDC * pDC, CLand * pLand) 
{
    /*
    if (!ValidHexNo(NoX, NoY))
        return;
    int           x0, y0;  // hex center

    GetHexCenter(NoX, NoY, x0, y0);
    pDC->Ellipse(x0-6, y0-6, x0+6, y0+6);
    */
}

//--------------------------------------------------------------------------

void CMapPane::DrawEdge(wxDC * pDC, CPlane * pPlane)
{

}


//--------------------------------------------------------------------------

#define ICON_SIZE       8
#define ICON_SIZE_SMALL 6

void CMapPane::GetNextIconPos(int x0, int y0, int & x, int & y)
{
    x += (ICON_SIZE+1);

    // assume we never get out of the lower part of a hex
        
    if (x + ICON_SIZE > x0 + m_HexHalfSize + (long)(tan30*(y0 + m_HexHalfHeight - y))) 
    {
        y -= (ICON_SIZE+1);
        x = x0 - m_HexHalfSize - (long)(tan30*(y0 + m_HexHalfHeight - y));
    }
}

//--------------------------------------------------------------------------

#define SET_ROAD_PEN(attr)                                              \
    BadRoad = FALSE;                                                    \
    for (i=0; i<pLand->Structs.Count(); i++)                            \
    {                                                                   \
        pStruct = (CStruct*)pLand->Structs.At(i);                       \
        if (pStruct->Attr & attr)                                       \
        {                                                               \
            BadRoad = ((pStruct->Attr & SA_ROAD_BAD) > 0);              \
            break;                                                      \
        }                                                               \
    }                                                                   \
    if (BadRoad)                                                        \
        pDC->SetPen(*m_pPenRoadBad);                                    \
    else                                                                \
        pDC->SetPen(*m_pPenRoad);                                       \


void CMapPane::DrawOneHex(int NoX, int NoY, int x0, int y0, wxDC * pDC, CLand * pLand, CPlane * pPlane)
{

    char          buf[128]  = {0};
    int           SymLeft   = x0 - m_HexHalfSize;
    int           SymBottom = y0 + m_HexHalfHeight - 2;
    int           i;
    long          x;
    EValueType    type;
    wxBrush     * pbrush;
    CStruct     * pStruct;
    BOOL          BadRoad;
    

    wxPoint       point[7]  =  
    { 
        wxPoint(x0 - m_HexSize    , y0                   ),   //0
        wxPoint(x0 - m_HexHalfSize, y0 - m_HexHalfHeight ),   //1
        wxPoint(x0 + m_HexHalfSize, y0 - m_HexHalfHeight ),   //2
        wxPoint(x0 + m_HexSize    , y0                   ),   //3
        wxPoint(x0 + m_HexHalfSize, y0 + m_HexHalfHeight ),   //4
        wxPoint(x0 - m_HexHalfSize, y0 + m_HexHalfHeight ),   //5
        wxPoint(x0 - m_HexSize    , y0                   )    //6
    };

    if (m_HexSizeIdx<SmallHexesCount)
        pDC->SetPen(*wxTRANSPARENT_PEN); 
    else
        pDC->SetPen(*m_pPen);

    // Hexes    
    pbrush = GetLandBrush(pLand, FALSE);
    pDC->SetBrush(*pbrush);
    pDC->DrawPolygon(7, point, 0, 0, wxWINDING_RULE);

    if (m_Hatch && pLand && (0==(pLand->Flags&LAND_VISITED)))
    {
        pbrush = GetLandBrush(pLand, TRUE);
        pDC->SetBrush(*pbrush);
        pDC->DrawPolygon(7, point, 0, 0, wxWINDING_RULE);
    }

    // Roads
    if (m_HexSizeIdx>=SmallHexesCount)
    {
        int x2, y2;
        if (pLand && (pLand->Flags&LAND_STR_ROAD_N))
        {
            SET_ROAD_PEN(SA_ROAD_N)
            pDC->DrawLine(x0, y0, x0, y0-m_HexHalfHeight);
        }
        if (pLand && (pLand->Flags&LAND_STR_ROAD_S))
        {
            SET_ROAD_PEN(SA_ROAD_S)
            pDC->DrawLine(x0, y0, x0, y0+m_HexHalfHeight);
        }
        if (pLand && (pLand->Flags&LAND_STR_ROAD_SE))
        {
            SET_ROAD_PEN(SA_ROAD_SE)
            x2 = x0+m_HexHalfSize*3/2; 
            y2 = y0+m_HexHalfHeight/2;
            pDC->DrawLine(x0, y0, x2, y2);
        }
        if (pLand && (pLand->Flags&LAND_STR_ROAD_SW))
        {
            SET_ROAD_PEN(SA_ROAD_SW)
            x2 = x0-m_HexHalfSize*3/2; 
            y2 = y0+m_HexHalfHeight/2;
            pDC->DrawLine(x0, y0, x2, y2);
        }
        if (pLand && (pLand->Flags&LAND_STR_ROAD_NE))
        {
            SET_ROAD_PEN(SA_ROAD_NE)
            x2 = x0+m_HexHalfSize*3/2; 
            y2 = y0-m_HexHalfHeight/2;
            pDC->DrawLine(x0, y0, x2, y2);
        }
        if (pLand && (pLand->Flags&LAND_STR_ROAD_NW))
        {
            SET_ROAD_PEN(SA_ROAD_NW)
            x2 = x0-m_HexHalfSize*3/2; 
            y2 = y0-m_HexHalfHeight/2;
            pDC->DrawLine(x0, y0, x2, y2);
        }
    }    
    
    // Tax and Production
    if (m_HexSizeIdx>=SmallHexesCount)
    {
      
        i = 0;
        if (pLand && (pLand->Flags&LAND_TAX))
        {
            buf[i]   = '$'; 
            buf[++i] = 0;
        }
        if (pLand && (pLand->Flags&LAND_TRADE))
        {
            buf[i]   = 'P'; 
            buf[++i] = 0;
        }

        if (*buf)
            pDC->DrawText(buf,  point[1].x, point[1].y+1 );
    }

    // Cities    
    if (pLand && m_HexSizeIdx>=SmallHexesCount && !pLand->CityName.IsEmpty())
    {
        pDC->SetPen  (*m_pPenBlack);
        pDC->SetBrush(*m_pBrushWhite);
        pDC->DrawArc(x0-4, y0, x0-4, y0, x0, y0);
        pDC->DrawArc(x0-3, y0, x0-3, y0, x0, y0);
    }
        
// Draw land icons ====================    
    
    // Units
    if (pLand && pLand->Flags&LAND_UNITS && m_HexSizeIdx>=VerySmallHexesCount)
    {

        if (!pLand->GetProperty(PRP_SEL_FACT_MEN, type, (const void *&)x, eNormal) || (eLong!=type))
            x=0;

        if (x>m_MinSelMen)
            x = ICON_SIZE;
        else
            x = ICON_SIZE_SMALL;


        pDC->SetPen  (*m_pPenBlack);
        pDC->SetBrush(*m_pBrushRed);
        pDC->DrawArc(SymLeft+x/2, SymBottom, 
                     SymLeft+x/2, SymBottom, 
                     SymLeft+x/2, SymBottom-x/2);

        GetNextIconPos(x0, y0, SymLeft, SymBottom);
    }
    
                
    if (m_HexSizeIdx>=SmallHexesCount)
    {
        // Gate
        if (pLand && (pLand->Flags&LAND_STR_GATE))
        {
            pDC->SetPen  (*m_pPenBlack);
            pDC->SetBrush(*m_pBrushBlack);
            pDC->DrawRectangle(SymLeft, SymBottom-ICON_SIZE, ICON_SIZE, ICON_SIZE);

            pDC->SetBrush(*m_pBrushWhite);
            pDC->DrawArc(SymLeft+ICON_SIZE/2, SymBottom, 
                         SymLeft+ICON_SIZE/2, SymBottom, 
                         SymLeft+ICON_SIZE/2, SymBottom-ICON_SIZE/2);

            GetNextIconPos(x0, y0, SymLeft, SymBottom);
        }

        // Shaft
        if (pLand && (pLand->Flags&LAND_STR_SHAFT))
        {
            pDC->SetPen  (*m_pPenBlack);
            pDC->DrawLine(SymLeft          , SymBottom-ICON_SIZE, SymLeft          , SymBottom);
            pDC->DrawLine(SymLeft+ICON_SIZE, SymBottom-ICON_SIZE, SymLeft+ICON_SIZE, SymBottom);

            pDC->DrawLine(SymLeft, SymBottom-3          , SymLeft+ICON_SIZE, SymBottom-3          );
            pDC->DrawLine(SymLeft, SymBottom-ICON_SIZE+2, SymLeft+ICON_SIZE, SymBottom-ICON_SIZE+2);
            
            GetNextIconPos(x0, y0, SymLeft, SymBottom);
        }

        // Ship
        if (pLand && pLand->Flags & LAND_STR_MOBILE)
        {
            wxPoint tri[3] = 
            {
                wxPoint(SymLeft,             SymBottom-ICON_SIZE),
                wxPoint(SymLeft+ICON_SIZE/2, SymBottom-1),
                wxPoint(SymLeft+ICON_SIZE,   SymBottom-ICON_SIZE)
            };

            pDC->SetPen  (*m_pPenBlack);
            pDC->SetBrush(*m_pBrushBlack);
            pDC->DrawPolygon(3, tri, 0, 0, wxWINDING_RULE);

            GetNextIconPos(x0, y0, SymLeft, SymBottom);
        }

        // Generic struct
        if (pLand && pLand->Flags & LAND_STR_GENERIC)
        {
            wxPoint tri[3] = 
            {
                wxPoint(SymLeft,             SymBottom-1),
                wxPoint(SymLeft+ICON_SIZE/2, SymBottom-ICON_SIZE),
                wxPoint(SymLeft+ICON_SIZE,   SymBottom-1)
            };

            pDC->SetPen  (*m_pPenBlack);
            pDC->SetBrush(*m_pBrushBlack);
            pDC->DrawPolygon(3, tri, 0, 0, wxWINDING_RULE);

            GetNextIconPos(x0, y0, SymLeft, SymBottom);
        }

        // Battle
        if (pLand && (pLand->Flags&LAND_BATTLE))
        {
            pDC->SetPen  (*m_pPenRed);
            pDC->DrawLine(x0-5, y0-5, x0+6, y0+6);
            pDC->DrawLine(x0+5, y0-5, x0-6, y0+6);
        }
    }

    // Flags
    if (m_HexSizeIdx>=VerySmallHexesCount)
    {
        static int dfx[LAND_FLAG_COUNT] = {-4,  0,  4};
        static int dfy[LAND_FLAG_COUNT] = { 4,  0,  4};

        for (i=0; i<LAND_FLAG_COUNT; i++)
            if (pLand && !pLand->FlagText[i].IsEmpty())
            {
                pDC->SetPen  (*m_pPenFlag[i]);
                pDC->DrawLine(x0+dfx[i],   y0+dfy[i]   , x0+dfx[i]  , y0+dfy[i]-10);
                pDC->DrawLine(x0+dfx[i],   y0+dfy[i]-10, x0+dfx[i]+6, y0+dfy[i]-7 );
                pDC->DrawLine(x0+dfx[i]+6, y0+dfy[i]-7 , x0+dfx[i],   y0+dfy[i]-4 );
            }
    }
}






//--------------------------------------------------------------------------

void CMapPane::DrawHex(int NoX, int NoY, wxDC * pDC, CLand * pLand, CPlane * pPlane, wxRect * prect) 
{
    if (!ValidHexNo(NoX, NoY))
        return;

    int           x0, y0;  // hex center
    int           dwx;

    //NormalizeHexX(NoX, pPlane);
    GetHexCenter(NoX, NoY, x0, y0);

    if (pPlane && (pPlane->Width>0) )
    {
        dwx = (m_HexSize * 3 / 2) * pPlane->Width;
        
        while (x0 > prect->x-m_HexSize)
            x0 -= dwx;
        x0 += dwx;

        while (x0<prect->x + prect->width + m_HexSize)
        {
            DrawOneHex(NoX, NoY, x0, y0, pDC, pLand, pPlane);
            x0 += dwx;
        }

    }
    else
        DrawOneHex(NoX, NoY, x0, y0, pDC, pLand, pPlane);
    
}
    
//--------------------------------------------------------------------------

void CMapPane::DrawOneHexBorder(int x0, int y0, wxDC * pDC, BOOL IsSelected, BOOL FullBorder, wxPen ** pPens, int ExitBits)
{
    int  i;
    int  IsExit;
    int  ExitPen = 1;

    wxPoint point[7] =  
    { 
        wxPoint(x0 - m_HexSize    , y0                   ),   //0
        wxPoint(x0 - m_HexHalfSize, y0 - m_HexHalfHeight ),   //1
        wxPoint(x0 + m_HexHalfSize, y0 - m_HexHalfHeight ),   //2
        wxPoint(x0 + m_HexSize    , y0                   ),   //3
        wxPoint(x0 + m_HexHalfSize, y0 + m_HexHalfHeight ),   //4
        wxPoint(x0 - m_HexHalfSize, y0 + m_HexHalfHeight ),   //5
        wxPoint(x0 - m_HexSize    , y0                   )    //6
    };
    int Dir[] = {Northwest, North, Northeast,   Southeast,   South,   Southwest};



    for (i=0; i<6; i++)
    {
        IsExit = ((ExitBits&ExitFlags[Dir[i]])>0)?1:0;
        if (IsSelected || !IsExit)
        {
            if ( ExitPen ^ IsExit )
            {
                pDC->SetPen(*pPens[ExitPen]);
                ExitPen ^= 1;
            }
            pDC->DrawLine(point[i].x, point[i].y, point[i+1].x, point[i+1].y);
        }
    }
    if (0==ExitPen)
        pDC->SetPen(*pPens[0]);
}

//--------------------------------------------------------------------------

void CMapPane::DrawHexBorder(int NoX, int NoY, wxDC * pDC, CLand * pLand, BOOL FullBorder, CPlane * pPlane, wxRect * prect, int DrawPhase, BOOL AutoEmpty)
{
    if (!ValidHexNo(NoX, NoY))
        return;

    // draw empty hexes first, then non-empty
    if (pLand)
    {
        if (0==DrawPhase)
            return;
    }
    else
        if (DrawPhase>0)
            return;


    BOOL          IsSelected = (NoX==m_SelHexX && NoY==m_SelHexY);
    wxPen        * pPen;
    wxPen        * Pens[2];
    int           x0, y0;  // hex center
    int           dwx;
    int           ExitBits;
    int           i;


    if ( (m_HexSizeIdx<SmallHexesCount && !IsSelected) || 
         (m_HexSizeIdx<VerySmallHexesCount)
       )
       return;
    



    //NormalizeHexX(NoX, pPlane);
    GetHexCenter(NoX, NoY, x0, y0);
    if (IsSelected)
        pPen = m_pPenSel;
    else
        pPen = m_pPen;

    Pens[0] = pPen;
    Pens[1] = m_pPenWall;


    if (pLand)
    {
        ExitBits = pLand->ExitBits;
        //if (ExitBits!=63)
        //{
        //    int x = 0;
        //}
    }
    else
        if (AutoEmpty)
        {
            int  x, y;

            ExitBits = 0;
            for (i=0; i<6; i++)
            {
                x = NoX;
                y = NoY;

                switch (i)
                {
                case North     : y -= 2;     break;
                case Northeast : y--; x++;   break;
                case Southeast : y++; x++;   break;
                case South     : y += 2;     break;
                case Southwest : y++; x--;   break;
                case Northwest : y--; x--;   break;
                }

                pLand = gpApp->m_pAtlantis->GetLand(x, y, m_SelPlane, TRUE);
                if ( pLand && (pLand->Flags&LAND_VISITED) )
                {
                    if (pLand->ExitBits&EntryFlags[i])
                        ExitBits |= ExitFlags[i];
                }
                else
                    ExitBits |= ExitFlags[i]; // assume there is an exit if nothing is known
            }
        }
        else
            ExitBits = 0xFF;

    pDC->SetPen(*pPen);

    if (pPlane && (pPlane->Width>0) )
    {
        dwx = (m_HexSize * 3 / 2) * pPlane->Width;
        
        while (x0>prect->x-m_HexSize)
            x0 -= dwx;
        x0 += dwx;

        while (x0 < prect->x + prect->width + m_HexSize)
        {
            DrawOneHexBorder(x0, y0, pDC, IsSelected, FullBorder, Pens, ExitBits);
            x0 += dwx;
        }

    }
    else
        DrawOneHexBorder(x0, y0, pDC, IsSelected, FullBorder, Pens, ExitBits);

}

//--------------------------------------------------------------------------

void CMapPane::DrawCities(wxDC * pDC, wxRect * pRect, CPlane * pPlane)
{
    int               i;
    int               wx, wy;
    int               nx, ny, nz;
    CLand           * pLand;
    int               dwx;
    wxCoord           w, h, descent, ext;

    pDC->SetPen (*m_pPen);
    pDC->SetFont(*gpApp->m_Fonts[FONT_MAP_TEXT]);
    pDC->SetBackgroundMode(wxTRANSPARENT);

    for (i=0; i<m_pCities->Count(); i++)
    {
        pLand = (CLand*)m_pCities->At(i);
        LandIdToCoord(pLand->Id, nx, ny, nz);
        GetHexCenter(nx, ny, wx, wy);
        pDC->GetTextExtent(pLand->CityName.GetData(), &w, &h, &descent, &ext);

        if (pPlane && (pPlane->Width>0) )
        {
            dwx = (m_HexSize * 3 / 2) * pPlane->Width;
        
            while (wx > pRect->x - m_HexSize)
                wx -= dwx;
            wx += dwx;
            while (wx < pRect->x + pRect->width + m_HexSize)
            {
                pDC->DrawText(pLand->CityName.GetData(), wx-w/2, wy-ext);
                wx += dwx;
            }
        }
        else
            pDC->DrawText(pLand->CityName.GetData(), wx-w/2, wy-ext);
    }
}

//--------------------------------------------------------------------------

void CMapPane::NormalizeCoordinates(CPlane * pPlane, wxRect * pRect)
{
    int               dnx;
    int               wx;
    int               nx, ny;

    if (pPlane && (pPlane->Width>0))
    {
        dnx   = 0;

        wx    = pRect->x + pRect->width/2;
        GetHexNo(nx, ny, wx, pRect->y);
        if (nx > pPlane->WestEdge)
            dnx = -((nx - pPlane->WestEdge)/pPlane->Width)*pPlane->Width;
        else
            if (nx < pPlane->EastEdge)
                dnx = ((pPlane->EastEdge - nx)/pPlane->Width)*pPlane->Width;

        m_AtlaX0 += dnx*(m_HexSize * 3 / 2);
    }

}

//--------------------------------------------------------------------------

void CMapPane::OnPaint(wxPaintEvent &WXUNUSED(event))
{
    int               width, height;

    GetClientSize(&width, &height);

    wxPaintDC         dc(this); // device context for painting
    wxRect            rect(0, 0, width, height);
    int               nx1, ny1;
    int               nx2, ny2;
    int               nx, ny, nz;
    int               i;
    int               wx, wy;
    CPlane          * pPlane;
    CLand           * pLand;
    CUnit           * pUnit;
    wxBrush         * pbrush;

    m_pCities->DeleteAll();
    dc.SetFont (*gpApp->m_Fonts[FONT_MAP_TEXT]);

    if (m_ShowState & SHOW_COORD)
    {
        width  -= COORD_PANE_WIDTH;
        height -= COORD_PANE_HEIGHT;
        dc.SetClippingRegion(0,0,width,height);
    }

    pPlane   = (CPlane*)gpApp->m_pAtlantis->m_Planes.At(m_SelPlane);

    if (m_HexSizeIdx >= SmallHexesCount) // big hexes
    {
        // get raw hex numbers
        GetHexNo(nx1, ny1, 0    , 0   );
        GetHexNo(nx2, ny2, width, height);

        if (pPlane && (pPlane->Width>0) && (nx2-nx1+1 > pPlane->Width) )
        {
            nx1 = pPlane->WestEdge;
            nx2 = pPlane->EastEdge;
        }
        else
        {
            nx1--;
            nx2++;
        }

        nx = nx1-1;
        while (++nx <= nx2)
            for (ny=ny1-1; ny<=ny2+1; ny++)
            {
                pLand = gpApp->m_pAtlantis->GetLand(nx, ny, m_SelPlane, TRUE);
                if (pLand && !pLand->CityName.IsEmpty())
                    m_pCities->Insert(pLand);
                DrawHex(nx, ny, &dc, pLand, pPlane, &rect);
            }

    /*
        for (dp=0; dp<=MAX_DP_BORDER; dp++)
        {
        */
            nx = nx1-1;
            while (++nx <= nx2)
                for (ny=ny1-1; ny<=ny2+1; ny++)
                {
                    pLand = gpApp->m_pAtlantis->GetLand(nx, ny, m_SelPlane, TRUE);
                    DrawHexBorder(nx, ny, &dc, pLand, FALSE, pPlane, &rect, MAX_DP_BORDER /*dp*/, FALSE);
                }
    /*
        }
        */
    }
    else // small hexes
    {
        pbrush = GetLandBrush(NULL, FALSE);
        dc.SetBrush(*pbrush);
        dc.SetPen  (*wxTRANSPARENT_PEN); 
        dc.DrawRectangle(0, 0, width, height);
        if (pPlane)
            for (i=0; i<pPlane->Lands.Count(); i++)
            {
                pLand = (CLand*)pPlane->Lands.At(i);
                LandIdToCoord(pLand->Id, nx, ny, nz);
                
                GetHexCenter(nx, ny, wx, wy);
                DrawHex(nx, ny, &dc, pLand, pPlane, &rect);

            }
    }

    pLand = gpApp->m_pAtlantis->GetLand(m_SelHexX, m_SelHexY, m_SelPlane, TRUE);
    DrawHexBorder(m_SelHexX, m_SelHexY, &dc, pLand, TRUE, pPlane, &rect, pLand?MAX_DP_BORDER:0, FALSE);
    DrawEdge(&dc, pPlane);

    pUnit = gpApp->GetSelectedUnit();
    RedrawTracksForUnit(pPlane, pUnit, &dc, FALSE);

    DrawCities(&dc, &rect, pPlane);
    PaintRectangle(&dc);

    // draw coord panes
    if (m_ShowState & SHOW_COORD)
    {
        dc.DestroyClippingRegion();
        DrawCoordPanes(&dc, width, height, pPlane);
    }

}

//--------------------------------------------------------------------------

void CMapPane::DrawCoordPanes(wxDC * pDC, int mapwidth, int mapheight, CPlane * pPlane)
{
    wxCoord   w, h, descent, ext;
    wxCoord   txtx, txty;
    int       x, y, nx, ny ;
    wxBrush * pBrush = GetLandBrush(NULL, FALSE);
    char      buf[32];


    int left  =  5;
    int right =  mapwidth-5;
    int top   =  5;
    int bottom=  mapheight-5;

    pDC->SetBrush(*pBrush);
    pDC->SetPen  (*wxTRANSPARENT_PEN); 
    pDC->SetFont (*gpApp->m_Fonts[FONT_MAP_COORD]);
    pDC->SetBackgroundMode(wxTRANSPARENT);

    pDC->DrawRectangle(0       , mapheight, mapwidth+COORD_PANE_WIDTH, COORD_PANE_HEIGHT);
    pDC->DrawRectangle(mapwidth, 0        ,          COORD_PANE_WIDTH, mapheight        );


    GetHexNo(nx, ny, left, top);

    // horizontal pane
    pDC->GetTextExtent("8", &w, &h, &descent, &ext);
    txty = mapheight + (COORD_PANE_HEIGHT-h)/2;
    nx--;
    while (TRUE)
    {
        nx++;
        GetHexCenter(nx, ny, x, y);
        sprintf(buf, "%d", NormalizeHexX(nx, pPlane));
        pDC->GetTextExtent(buf, &w, &h, &descent, &ext);
        txtx = x - w/2;
        if (txtx < left)
            continue;
        if (x + w/2 >= right)
            break;
        pDC->DrawText(buf, txtx, txty);
        left = x + w/2 + 5;
    }

    // vertical pane
    ny--;
    while (TRUE)
    {
        ny++;
        GetHexCenter(nx, ny, x, y);
        sprintf(buf, "%d", ny);
        pDC->GetTextExtent(buf, &w, &h, &descent, &ext);
        txtx = mapwidth + (COORD_PANE_WIDTH-w)/2;
        txty = y - h/2;
        if (txty < top)
            continue;
        if (y + h/2 >= bottom)
            break;
        pDC->DrawText(buf, txtx, txty);
        top = y + h/2 + 10;
    }

}

//--------------------------------------------------------------------------

/*
void CMapPane::SelectHex(int NoX, int NoY)
{
    int              wx, wy;
    int              width, height;

    if (NoX!=m_SelHexX || NoY!=m_SelHexY)
    {
        m_SelHexX = NoX;
        m_SelHexY = NoY;

        GetClientSize(&width, &height);
        GetHexCenter(NoX, NoY, wx, wy);

        if (wx-m_HexSize < 0) 
            m_AtlaX0 -= (0 - (wx-m_HexSize) + m_HexSize);
        else
            if ((wx+m_HexSize > width))
                m_AtlaX0 += (wx+m_HexSize - width + m_HexSize);

            
        if (wy-m_HexHalfHeight < 0)    
            m_AtlaY0 -= (0  - (wy-m_HexHalfHeight) + m_HexHalfHeight);
        else
            if ((wy+m_HexHalfHeight > height))
                m_AtlaY0 += (wy+m_HexHalfHeight - height + m_HexHalfHeight);

        //UpdateEditPane();
        gpApp->OnMapSelectionChange();
    }
}
*/


//--------------------------------------------------------------------------

void CMapPane::RedrawTracksForUnit(CPlane * pPlane, CUnit * pUnit, wxDC * pDC, BOOL DoDrawCities)
{
    int            i, X, Y, Z;
    int            wx0, wy0, dx;
    long           HexId; 
    BOOL           CloseDC = FALSE;
    CLand        * pLand;
    wxRect         rect;
    int            copyno=0;
    int            dp;

    if (!pPlane)
        return;

    if (NULL==pDC)
    {
        CloseDC = TRUE;
        pDC = new wxClientDC(this);
    }
    GetClientSize(&rect.width, &rect.height);
    if (CloseDC && (m_ShowState & SHOW_COORD))
    {
        rect.width  -= COORD_PANE_WIDTH;
        rect.height -= COORD_PANE_HEIGHT;
        pDC->SetClippingRegion(0,0,rect.width,rect.height);
    }

    
    NormalizeCoordinates(pPlane, &rect);
    pDC->SetFont (*gpApp->m_Fonts[FONT_MAP_TEXT]);



    // Erase old tracks
    for (i=0; i<m_pTrackHexes->Count(); i++)
    {
        HexId = (long)m_pTrackHexes->At(i);
        LandIdToCoord(HexId, X, Y, Z);
        pLand = gpApp->m_pAtlantis->GetLand(HexId);

        DrawHex(X, Y, pDC, pLand, pPlane, &rect);
    }
    for (dp=0; dp<=MAX_DP_BORDER; dp++)
        for (i=0; i<m_pTrackHexes->Count(); i++)
        {
            HexId = (long)m_pTrackHexes->At(i);
            LandIdToCoord(HexId, X, Y, Z);
            pLand = gpApp->m_pAtlantis->GetLand(HexId);

            DrawHexBorder(X, Y, pDC, pLand, TRUE, pPlane, &rect, dp, TRUE);
        }
    m_pTrackHexes->DeleteAll();

    pLand = gpApp->m_pAtlantis->GetLand(m_SelHexX, m_SelHexY, m_SelPlane, TRUE);
    DrawHexBorder(m_SelHexX, m_SelHexY, pDC, pLand, TRUE, pPlane, &rect, pLand?MAX_DP_BORDER:0, TRUE);


    pDC->SetPen(*m_pPenSel);

    // draw new tracks and remeber hexes
    if (pUnit && pUnit->pMovement)
    {
        m_pTrackHexes->Insert((void*)pUnit->LandId);
        LandIdToCoord(pUnit->LandId, X, Y, Z);
        GetHexCenter(X, Y, wx0, wy0);

        if (pPlane->Width>0)
        {
            dx = (m_HexSize * 3 / 2) * pPlane->Width;
            while (wx0-m_HexSize >= rect.x)
                wx0 -= dx;
            wx0+=dx;
            while (wx0+m_HexSize <= rect.x + rect.width)
            {
                DrawSingleTrack(X, Y, wx0, wy0, pDC, pUnit, pPlane, copyno++);
                wx0 += dx;
            }
        }
        else
            DrawSingleTrack(X, Y, wx0, wy0, pDC, pUnit, pPlane, copyno);
    }

    if (DoDrawCities)
        DrawCities(pDC, &rect, pPlane);

    if (CloseDC)
        delete pDC;
}

//--------------------------------------------------------------------------

void CMapPane::DrawSingleTrack(int X, int Y, int wx, int wy, wxDC * pDC, CUnit * pUnit, CPlane * pPlane,int copyno)
{
    int            i, X1, Y1, Z;
    long           HexId;
    int            wx0, wy0;

    for (i=0; i<pUnit->pMovement->Count(); i++)
    {
        wx0 = wx;
        wy0 = wy;
        
        HexId = (long)pUnit->pMovement->At(i);
        if (0==copyno)
            m_pTrackHexes->Insert((void*)HexId);

        LandIdToCoord(HexId, X1, Y1, Z);
    
        if (X1-X > 1)
            X1 = X-1;
        if (X-X1 > 1)
            X1 = X+1;

        wx += (m_HexSize * 3 / 2)*(X1-X);
        wy += m_HexHalfHeight*(Y1-Y);

        if (pPlane->Width>0)
        {
            if (X1>pPlane->EastEdge)
                X1 = pPlane->WestEdge;
            if (X1<pPlane->WestEdge)
                X1 = pPlane->EastEdge;
        }
        X = X1;
        Y = Y1;

        pDC->DrawLine(wx0, wy0, wx, wy);
    }
}

//--------------------------------------------------------------------------

void CMapPane::Center()
{
    CPlane  * pPlane;
    CLand   * pLand;
    long       sx = 0;
    long       sy = 0;
    int       i;
    int       x, y, z;
    long      ax, ay;
    int       width, height;

    pPlane = (CPlane*)gpApp->m_pAtlantis->m_Planes.At(m_SelPlane);
    if (pPlane)
    {
        for (i=0; i<pPlane->Lands.Count(); i++)
        {
            pLand = (CLand*)pPlane->Lands.At(i);
            LandIdToCoord(pLand->Id, x, y, z);
            sx += x;
            sy += y;
        }
        x = sx/pPlane->Lands.Count();
        y = sy/pPlane->Lands.Count();

        if (!ValidHexNo(x,y))
            y++;

        GetClientSize(&width, &height);

        ax = x * m_HexSize * 3 / 2;
        ay = y * m_HexHalfHeight;

        m_AtlaX0 = ax - width/2;
        m_AtlaY0 = ay - height/2;
    }
}

//--------------------------------------------------------------------------

BOOL CMapPane::EnsureLandVisible(int nx, int ny, int nz, BOOL Anyway)
{
    BOOL NeedRefresh     = FALSE;
    int  wx, wy;
    int  width, height;


    if (m_SelPlane!=nz)
    {
        SavePlaneConfig();
        m_SelPlane=nz;
        LoadPlaneConfig();
        NeedRefresh=TRUE;
    }

    if (NeedRefresh || (m_SelHexX!=nx) || (m_SelHexY!=ny) || Anyway)
    {
        GetClientSize(&width, &height);
        if (m_ShowState & SHOW_COORD)
        {
            width  -= COORD_PANE_WIDTH;
            height -= COORD_PANE_HEIGHT;
        }
        GetHexCenter(nx, ny, wx, wy);

        if (wx-m_HexSize < 0) 
        {
            m_AtlaX0 -= (0 - (wx-m_HexSize) + m_HexSize);
            NeedRefresh = TRUE;
        }
        else
            if ((wx+m_HexSize > width))
            {
                m_AtlaX0 += (wx+m_HexSize - width + m_HexSize);
                NeedRefresh = TRUE;
            }
            
        if (wy-m_HexHalfHeight < 0)    
        {
            m_AtlaY0 -= (0  - (wy-m_HexHalfHeight) + m_HexHalfHeight);
            NeedRefresh = TRUE;
        }
        else
            if ((wy+m_HexHalfHeight > height))
            {
                m_AtlaY0 += (wy+m_HexHalfHeight - height + m_HexHalfHeight);
                NeedRefresh = TRUE;
            }
    }
    return NeedRefresh;
}

//--------------------------------------------------------------------------

void CMapPane::SetSelection(int nx, int ny, CUnit * pUnit, CPlane * pPlane, BOOL Anyway)
{
    int           oldx, oldy;
    int           x, y;
    CLand       * pLand;
    wxClientDC    dc(this);

    wxRect        rect;
    ;
    int           dp = MAX_DP_BORDER;

    GetClientSize(&rect.width, &rect.height);
    if (m_ShowState & SHOW_COORD)
    {
        rect.width  -= COORD_PANE_WIDTH;
        rect.height -= COORD_PANE_HEIGHT;
        dc.SetClippingRegion(0,0,rect.width,rect.height);
    }
    NormalizeCoordinates(pPlane, &rect);

    if (nx!=m_SelHexX || ny!=m_SelHexY || Anyway)
    {
        oldx = m_SelHexX;
        oldy = m_SelHexY;
        m_SelHexX = nx;
        m_SelHexY = ny;
        dc.SetFont (*gpApp->m_Fonts[FONT_MAP_TEXT]);

        for (x=oldx-1; x<=oldx+1; x++)
            for (y=oldy-2; y<=oldy+2; y++)
            {
                pLand = gpApp->m_pAtlantis->GetLand(x, y, m_SelPlane, TRUE);
                DrawHex      (x, y, &dc, pLand, pPlane, &rect);
            }
        for (dp=0; dp<=MAX_DP_BORDER; dp++)
            for (x=oldx-1; x<=oldx+1; x++)
                for (y=oldy-2; y<=oldy+2; y++)
                {
                    pLand = gpApp->m_pAtlantis->GetLand(x, y, m_SelPlane, TRUE);
                    DrawHexBorder(x, y, &dc, pLand, TRUE,  pPlane, &rect, dp, TRUE);
                }

        pLand = gpApp->m_pAtlantis->GetLand(nx, ny, m_SelPlane, TRUE);
        DrawHex      (nx, ny, &dc, pLand, pPlane, &rect);
        DrawHexBorder(nx, ny, &dc, pLand, TRUE,  pPlane, &rect, pLand?MAX_DP_BORDER:0, TRUE);
        DrawEdge(&dc, pPlane);

        RedrawTracksForUnit(pPlane, pUnit, &dc, FALSE);
        DrawCities(&dc, &rect, pPlane);
        gpApp->OnMapSelectionChange();
    }

}

//--------------------------------------------------------------------------

void CMapPane::RemoveRectangle()
{
    m_Rect_IsPaused    = false;
    m_Rect_IsDragging  = false;
    m_Rect_x1          = 0;
    m_Rect_y1          = 0;
    m_Rect_x2          = 0;
    m_Rect_y2          = 0;
}

//--------------------------------------------------------------------------

void CMapPane::StartRectangle(int x, int y)
{
    wxClientDC    dc(this);

    PaintRectangle(&dc);
    
    m_Rect_IsPaused    = false;
    m_Rect_IsDragging  = true;
    m_Rect_x1          = x;
    m_Rect_y1          = y;
    m_Rect_x2          = x;
    m_Rect_y2          = y;
}

//--------------------------------------------------------------------------

void CMapPane::EndRectangle  (int x, int y)
{
    if (m_Rect_IsDragging)
    {
        m_Rect_IsDragging  = false;
        m_Rect_IsPaused    = false;
        m_Rect_x2          = x;
        m_Rect_y2          = y;
    }
}

//--------------------------------------------------------------------------

void CMapPane::MoveRectangle (int x, int y)
{
    if (m_Rect_IsDragging && !m_Rect_IsPaused )
    {
        wxClientDC    dc(this);
        
        PaintRectangle(&dc);
        m_Rect_x2          = x;
        m_Rect_y2          = y;
        PaintRectangle(&dc);
    }
}

//--------------------------------------------------------------------------

void CMapPane::PauseRectangle()
{
    if (m_Rect_IsDragging)
        m_Rect_IsPaused = true;
}

//--------------------------------------------------------------------------

void CMapPane::ResumeRectangle()
{
    if (m_Rect_IsDragging)
        m_Rect_IsPaused = false;
}

//--------------------------------------------------------------------------

bool CMapPane::ShouldDrawRectangle()
{
    // TBD: write some more decent code checking if there is a hex center inside the rect
//    return 
//        m_Rect_x1 - m_Rect_x2 > 10 && m_Rect_y1 - m_Rect_y2 > 10  ||
//        m_Rect_x1 - m_Rect_x2 > 10 && m_Rect_y1 - m_Rect_y2 < 10  ||
//        m_Rect_x1 - m_Rect_x2 < 10 && m_Rect_y1 - m_Rect_y2 > 10  ||
//        m_Rect_x1 - m_Rect_x2 < 10 && m_Rect_y1 - m_Rect_y2 < 10  ;

    int nx1, ny1, nx2, ny2;
            
    GetHexNo(nx1, ny1, m_Rect_x1, m_Rect_y1);
    GetHexNo(nx2, ny2, m_Rect_x2, m_Rect_y2);

    return (nx1!=nx2 || ny1!=ny2);
            
}

//--------------------------------------------------------------------------

void CMapPane::PaintRectangle(wxDC * pDC)
{
    if (ShouldDrawRectangle())
    {
        int     OldFunc = pDC->GetLogicalFunction();
        wxPen & OldPen  = pDC->GetPen();
        pDC->SetLogicalFunction(wxXOR);  
            
        pDC->SetPen  (*m_pPenSel);//m_pPenRed);
        pDC->DrawLine(m_Rect_x1, m_Rect_y1, m_Rect_x2, m_Rect_y1);
        pDC->DrawLine(m_Rect_x2, m_Rect_y1, m_Rect_x2, m_Rect_y2);
        pDC->DrawLine(m_Rect_x2, m_Rect_y2, m_Rect_x1, m_Rect_y2);
        pDC->DrawLine(m_Rect_x1, m_Rect_y2, m_Rect_x1, m_Rect_y1);

        pDC->SetPen(OldPen);  
        pDC->SetLogicalFunction(OldFunc);    
    }
}

//--------------------------------------------------------------------------

BOOL CMapPane::GetSelectedHexes(int & x1, int & y1, int & x2, int & y2 )
{
    int       xx;
    BOOL      ToTheRight;  // drawn from left to right
    BOOL      EdgeCrossed;
    CPlane  * pPlane = (CPlane*)gpApp->m_pAtlantis->m_Planes.At(m_SelPlane);
    
    if (m_Rect_x1==m_Rect_x2 || m_Rect_y1==m_Rect_y2)
        return FALSE;   // no selection
    
    GetHexNo(x1, y1, m_Rect_x1, m_Rect_y1);
    GetHexNo(x2, y2, m_Rect_x2, m_Rect_y2);

    x1 = NormalizeHexX(x1, pPlane);
    x2 = NormalizeHexX(x2, pPlane);
    
    ToTheRight = m_Rect_x2 > m_Rect_x1; 
    if (!ToTheRight)
    {
        xx = x1; x1 = x2; x2 = xx; // normalize the rect
    }
    
    EdgeCrossed = x1>x2;
    
    if (y1 > y2)
    {
        xx = y1; y1 = y2; y2 = xx;
    }

    return TRUE;    
}

//--------------------------------------------------------------------------

void CMapPane::OnMouseEvent(wxMouseEvent& event)
{
    wxCoord       xpos, ypos;
    int           nx, ny;

    event.GetPosition(&xpos, &ypos);

    if (event.LeftDown())
    {
        StartRectangle(xpos, ypos);
        GetHexNo(nx, ny, xpos, ypos);
        SetSelection(nx, ny, gpApp->GetSelectedUnit(), (CPlane*)gpApp->m_pAtlantis->m_Planes.At(m_SelPlane), FALSE);
    }

    else if (event.LeftUp())
        EndRectangle(xpos, ypos);
    
    else if (event.Dragging())
        MoveRectangle(xpos, ypos);
    
    else if (event.Leaving())
        PauseRectangle();
    
    else if (event.Entering())
        ResumeRectangle();

    else if ( (event.RightDown()  &&  atol(gpApp->GetConfig(SZ_SECT_COMMON, SZ_KEY_RCLICK_CENTERS))) ||
              (event.LeftDClick() && !atol(gpApp->GetConfig(SZ_SECT_COMMON, SZ_KEY_RCLICK_CENTERS))) )
    {
        // center
        wxPoint  point(xpos, ypos);

        CenterClick(point);
    }

    else if ( (event.RightDown()  && !atol(gpApp->GetConfig(SZ_SECT_COMMON, SZ_KEY_RCLICK_CENTERS))) ||
              (event.LeftDClick() &&  atol(gpApp->GetConfig(SZ_SECT_COMMON, SZ_KEY_RCLICK_CENTERS))) )
    {
        // popup menu

        wxMenu  menu;
        int     nx, ny;

        m_PopupPoint.x = xpos;
        m_PopupPoint.y = ypos;

        GetHexNo(nx, ny, xpos, ypos);
        m_pPopupLand   = gpApp->m_pAtlantis->GetLand(nx, ny, m_SelPlane, TRUE);
        
        if (m_pPopupLand)
        {
            menu.Append(menu_Popup_Flag, "Flags");
            if (m_pPopupLand->Flags&LAND_BATTLE)
                menu.Append(menu_Popup_Battles, "Battles");
        }

        menu.Append(menu_Popup_Center, "Center");

        PopupMenu( &menu, xpos, ypos);
    }
    else
    {
        //That must be the mouse move event
    }
}

//--------------------------------------------------------------------------

void CMapPane::CenterClick(wxPoint point)
{
    wxRect        rect;
    CPlane      * pPlane;

    GetClientSize(&rect.width, &rect.height);
    pPlane   = (CPlane*)gpApp->m_pAtlantis->m_Planes.At(m_SelPlane);

    m_AtlaX0 += (point.x - rect.width/2); 
    m_AtlaY0 += (point.y - rect.height/2);

    RemoveRectangle();
    NormalizeCoordinates(pPlane, &rect);
    Refresh(FALSE, NULL);
}

//--------------------------------------------------------------------------

void CMapPane::OnPopupMenuCenter(wxCommandEvent& WXUNUSED(event)) 
{
    CenterClick(m_PopupPoint);
}

//--------------------------------------------------------------------------

void CMapPane::OnPopupMenuFlag(wxCommandEvent& WXUNUSED(event)) 
{
    int      Cmd;
    int      i;

    if (m_pPopupLand)
    {
        wxPoint     point = ClientToScreen(m_PopupPoint);
        CMapFlagDlg Dlg(gpApp->m_Frames[AH_FRAME_MAP], m_pPopupLand, point);

        Cmd = Dlg.ShowModal();

        switch (Cmd)
        {
        case wxID_OK:
            for (i=0; i<LAND_FLAG_COUNT; i++)
                m_pPopupLand->FlagText[i] = Dlg.m_FlagText[i]->GetValue();
            break;
        case wxID_NO:
            for (i=0; i<LAND_FLAG_COUNT; i++)
                m_pPopupLand->FlagText[i].Empty();
            break;
        default:
            return;
        }

//        gpApp->m_LandFlagsChanged = TRUE;
        Refresh(FALSE);
        gpApp->OnMapSelectionChange();
    }
}

//--------------------------------------------------------------------------

void CMapPane::OnPopupMenuBattles(wxCommandEvent & event)
{
    int       i;
    long      id;
    CBattle * pBattle;
    CBaseColl Battles;

    for (i=0; i<gpApp->m_pAtlantis->m_Battles.Count(); i++)
    {
        pBattle = (CBattle * )gpApp->m_pAtlantis->m_Battles.At(i);

        if (gpApp->m_pAtlantis->LandStrCoordToId(pBattle->LandStrId.GetData(), id) && id==m_pPopupLand->Id)
            Battles.Insert(pBattle);
    }
    gpApp->ShowDescriptionList(Battles, "Battles");
    
    Battles.DeleteAll();
}

//--------------------------------------------------------------------------

void CMapPane::OnEraseBackground(wxEraseEvent& event)
{
}

//--------------------------------------------------------------------------

void CMapPane::OnToolbarCmd(wxCommandEvent& event)
{
//    int  i;


    switch (event.GetId())
    {
    case tool_zoomin    :
        if (SetHexSize(m_HexSizeIdx+1))
        {
            RemoveRectangle();
            Refresh(FALSE, NULL);
        }
        break;

    case tool_zoomout   :
        if (SetHexSize(m_HexSizeIdx-1))
        {
            RemoveRectangle();
            Refresh(FALSE, NULL);
        }
        break;

    case tool_centerout :
        SetHexSize(0);
        Center();
        RemoveRectangle();
        Refresh(FALSE, NULL);
        break;

    case tool_prevzoom  :
        if (SetHexSize(m_HexSizeIdxOld))
        {
            RemoveRectangle();
            Refresh(FALSE, NULL);
        }
        break;

    case tool_showcoord :
        m_ShowState ^= SHOW_COORD;
        Refresh(FALSE, NULL);
        break;

    case tool_planeup   :
        if (m_SelPlane > 0)
        {
            SavePlaneConfig();
            m_SelPlane--;
            LoadPlaneConfig();
            Refresh(FALSE, NULL);
            gpApp->OnMapSelectionChange();
        }
        break;

    case tool_planedwn  :
        if (m_SelPlane < gpApp->m_pAtlantis->m_Planes.Count()-1)
        {
            SavePlaneConfig();
            m_SelPlane++;
            LoadPlaneConfig();
            Refresh(FALSE, NULL);
            gpApp->OnMapSelectionChange();
        }
        break;

    case tool_nextturn  :
        gpApp->SwitchToRep(repNext);
        break;

    case tool_prevturn  :
        gpApp->SwitchToRep(repPrev);
        break;

    case tool_lastturn  :
        gpApp->SwitchToRep(repLast);
        break;

    case tool_lastvisitturn  :
        gpApp->SwitchToRep(repLastVisited);
        break;

    }
}

//--------------------------------------------------------------------------

BOOL CMapPane::IsToolActive(wxUpdateUIEvent& event)
{
    BOOL Ok = FALSE;
    int  i;

    switch (event.GetId())
    {
    case tool_zoomin    :
        Ok = (m_HexSizeIdx < m_HexSizes.Count()-1);
        break;

    case tool_zoomout   :
        Ok = (m_HexSizeIdx > 0);
        break;

    case tool_centerout :
        Ok = TRUE;
        break;

    case tool_prevzoom  :
        Ok = (m_HexSizeIdx != m_HexSizeIdxOld);
        break;

    case tool_showcoord :
        Ok = TRUE;
        if (m_ShowState & SHOW_COORD)
            event.Check(TRUE);
        else
            event.Check(FALSE);
        break;

    case tool_planeup   :
        Ok = (m_SelPlane > 0);
        break;

    case tool_planedwn  :
        Ok = (m_SelPlane < gpApp->m_pAtlantis->m_Planes.Count()-1);
        break;

    case tool_nextturn  :
        Ok = gpApp->CanSwitchToRep(repNext, i);
        break;

    case tool_prevturn  :
        Ok = gpApp->CanSwitchToRep(repPrev, i);
        break;

    case tool_lastturn  :
        Ok = gpApp->CanSwitchToRep(repLast, i);
        break;
    
    case tool_lastvisitturn  :
        Ok = gpApp->CanSwitchToRep(repLastVisited, i);
        break;
    
    }

    return Ok;
}

//==========================================================================

BEGIN_EVENT_TABLE(CMapFlagDlg, wxDialog)
    EVT_BUTTON(-1, CMapFlagDlg::OnButton)
END_EVENT_TABLE()



CMapFlagDlg::CMapFlagDlg(wxWindow *parent, CLand * pLand, wxPoint & position) 
            :wxDialog( parent, -1, "Hex flags", position, wxDefaultSize, wxDIALOG_MODAL | wxRESIZE_BORDER )
{  
    int i;

    m_pLand = pLand;

    wxBoxSizer *topsizer = new wxBoxSizer( wxVERTICAL );

    m_btnSet    = new wxButton  (this, wxID_OK, "Set" );
    m_btnRemove = new wxButton  (this, wxID_NO, "Remove" );
    for (i=0; i<LAND_FLAG_COUNT; i++)
    {
        m_FlagText[i]  = new wxTextCtrl(this, -1, pLand->FlagText[i].GetData(), wxDefaultPosition, wxDefaultSize); //wxSize(100,30));

        // create text ctrl with minimal size 100x60  
        topsizer->Add(  m_FlagText[i]  ,
                        0,            // make vertically NOT stretchable
                        wxEXPAND |    // make horizontally stretchable
                        wxALL,        //   and make border all around
                        10 );         // set border width to 10
    }

    wxBoxSizer *button_sizer = new wxBoxSizer( wxHORIZONTAL );  
    
    button_sizer->Add(  m_btnSet,
                        0,           // make horizontally unstretchable
                        wxALL,       // make border all around (implicit top alignment)
                        10 );        // set border width to 10  
    button_sizer->Add(  m_btnRemove,
                        0,           // make horizontally unstretchable
                        wxALL,       // make border all around (implicit top alignment)
                        10 );        // set border width to 10  
    
    topsizer->Add( button_sizer,
                   0,                // make vertically unstretchable
                   wxALIGN_CENTER ); // no border and centre horizontally

    SetAutoLayout( TRUE );     // tell dialog to use sizer
    SetSizer( topsizer );      // actually set the sizer
    topsizer->Fit( this );            // set size to minimum size as calculated by the sizer
    topsizer->SetSizeHints( this );   // set size hints to honour mininum size}

    m_FlagText[0]->SetFocus();
    m_btnSet  ->SetDefault();

    int x,y, w,h, w1,h1;
    BOOL change = FALSE;

    GetPosition(&x,&y);
    GetSize(&w,&h);
    wxDisplaySize(&w1,&h1);

    if (x<0)
    {
        x      = 0;
        change = TRUE;
    }
    if (y<0)
    {
        y      = 0;
        change = TRUE;
    }
    if (x+w > w1)
    {
        x = w1-w;
        change = TRUE;
    }
    if (y+h > h1)
    {
        y = h1-h;
        change = TRUE;
    }
    if (change)
        SetSize(x,y,w,h);

}

//--------------------------------------------------------------------------

void CMapFlagDlg::OnButton(wxCommandEvent& event)
{
    if ( event.GetEventObject() == m_btnRemove )
        EndModal(wxID_NO);
    else
        event.Skip();
}

//--------------------------------------------------------------------------

