/*
 * This source file is part of the Atlantis Little Helper program.
 * Copyright (C) 2001 Maxim Shariy.
 *
 * Atlantis Little Helper is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Atlantis Little Helper is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Atlantis Little Helper; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __AH_MAP_PANE_INCL__
#define __AH_MAP_PANE_INCL__


class CLand;
class CPlane;
class CCollection;

#define SHOW_COORD   0x0001


//---------------------------------------------------------------------

class CBrushColl : public CCollection
{
public:
    CBrushColl() : CCollection() {};
    CBrushColl(int nDelta) : CCollection(nDelta) {};
protected:
    virtual void FreeItem(void * pItem) { delete (wxBrush*)pItem; };
};

//---------------------------------------------------------------------

class CMapPane : public wxWindow
{
public:
    CMapPane(wxWindow * parent, wxWindowID id, int layout);
    virtual ~CMapPane();
    virtual void Init(CAhFrame * pParentFrame);
    virtual void Done();
    BOOL     SetHexSize(int HexSizeIdx);  // was size really changed?
    void     GetHexCenter(int NoX, int NoY, int & WinX, int & WinY);
    BOOL     ValidHexNo(int NoX, int NoY);
    //void     SelectHex(int NoX, int NoY);
    void     SetSelection(int nx, int ny, CUnit * pUnit, CPlane * pPlane, BOOL Anyway);
    BOOL     EnsureLandVisible(int nx, int ny, int nz, BOOL Anyway);
    void     Center();
    void     SavePlaneConfig();
    void     LoadPlaneConfig();
    void     ApplyFonts();
    void     ApplyColors();
    void     RedrawTracksForUnit(CPlane * pPlane, CUnit * pUnit, wxDC * pDC, BOOL DoDrawCities);
    void     OnToolbarCmd(wxCommandEvent& event);
    BOOL     IsToolActive(wxUpdateUIEvent& event);
//  void     CleanCities() {    m_pCities->DeleteAll(); };
    BOOL     GetSelectedHexes(int & x1, int & y1, int & x2, int & y2 );

    int         m_HexSizeIdx;
    int         m_HexSizeIdxOld;
    int         m_HexSize;
    int         m_HexHalfSize;
    int         m_HexHeight;
    int         m_HexHalfHeight;

    long        m_AtlaX0; // corresponds to WinX=0
    long        m_AtlaY0; // corresponds to WinY=0

    long        m_ShowState;
    long        m_MinSelMen;

    long        m_SelHexX;
    long        m_SelHexY;
    long        m_SelPlane;

    CLongColl   m_HexSizes;


protected:

    void      WinToAtla(int   WinX, int   WinY, long & AtlaX, long & AtlaY);
    void      AtlaToWin(int & WinX, int & WinY, long   AtlaX, long   AtlaY);
             
    void      GetHexNo(int & NoX, int & NoY, int WinX, int WinY);
    void      DrawHex         (int NoX, int NoY, wxDC * pDC, CLand * pLand, CPlane * pPlane, wxRect * prect);
    void      DrawOneHex      (int NoX, int NoY, int x0, int y0, wxDC * pDC, CLand * pLand, CPlane * pPlane);
    void      DrawHexBorder   (int NoX, int NoY, wxDC * pDC, CLand * pLand, BOOL FullBorder, CPlane * pPlane, wxRect * prect, int DrawPhase, BOOL AutoEmpty);
    void      DrawOneHexBorder(int x0, int y0,   wxDC * pDC, BOOL IsSelected, BOOL FullBorder, wxPen ** pPens, int ExitBits);
    void      DrawHexTest     (int NoX, int NoY, wxDC * pDC, CLand * pLand) ;
    void      DrawCities      (wxDC * pDC, wxRect * pRect, CPlane * pPlane);
    void      NormalizeCoordinates(CPlane * pPlane, wxRect * pRect);
    int       NormalizeHexX(int NoX, CPlane * pPlane);
    void      DrawEdge(wxDC * pDC, CPlane * pPlane);
    void      DrawSingleTrack(int X, int Y, int wx, int wy, wxDC * pDC, CUnit * pUnit, CPlane * pPlane, int copyno);
    void      CenterClick(wxPoint point);
    wxBrush * GetLandBrush(CLand * pLand, BOOL GetHatched);
    void      ApplyOneColor(wxColour & cr, const char * name);
    void      DrawCoordPanes(wxDC * pDC, int mapwidth, int mapheight, CPlane * pPlane);
    void      GetNextIconPos(int x0, int y0, int & x, int & y);
             
    void      OnPaint           (wxPaintEvent   & event);
    void      OnMouseEvent      (wxMouseEvent   & event);
    void      OnEraseBackground (wxEraseEvent   & event);
    void      OnPopupMenuFlag   (wxCommandEvent & event); 
    void      OnPopupMenuCenter (wxCommandEvent & event); 
    void      OnPopupMenuBattles(wxCommandEvent & event);

    void      StartRectangle(int x, int y);
    void      EndRectangle  (int x, int y);
    void      MoveRectangle (int x, int y);
    void      PauseRectangle();
    void      ResumeRectangle();
    bool      ShouldDrawRectangle();
    void      PaintRectangle(wxDC * pDC);
    void      RemoveRectangle();

    wxPen        * m_pPen;
    wxPen        * m_pPenWall;
    wxPen        * m_pPenSel;
    wxPen        * m_pPenRoad;
    wxPen        * m_pPenRoadBad;
    wxPen        * m_pPenBlack;
    wxPen        * m_pPenRed;
    wxPen        * m_pPenWhite;
    wxPen        * m_pPenFlag[LAND_FLAG_COUNT];
                   
    wxBrush      * m_pBrushBlack; 
    wxBrush      * m_pBrushWhite; 
    wxBrush      * m_pBrushRed  ; 
    BOOL           m_Hatch;

    CBrushColl     m_TerrainBrushes;
    CBufColl       m_TerrainNames;
    int            m_UnknownColorIdx;

    CCollection  * m_pCities;
    CCollection  * m_pTrackHexes;

    CAhFrame     * m_pFrame;

    CLand        * m_pPopupLand;
    wxPoint        m_PopupPoint;

    bool           m_Rect_IsPaused;
    bool           m_Rect_IsDragging;
    int            m_Rect_x1;
    int            m_Rect_y1;
    int            m_Rect_x2;
    int            m_Rect_y2;
    
    
    DECLARE_EVENT_TABLE()
};


//---------------------------------------------------------------------

class CMapFlagDlg : public wxDialog
{
public:
    CMapFlagDlg(wxWindow *parent, CLand * pLand, wxPoint & position);
    void OnButton(wxCommandEvent& event);

    wxTextCtrl * m_FlagText[LAND_FLAG_COUNT];

private:
    wxButton   * m_btnSet;
    wxButton   * m_btnRemove;
    CLand      * m_pLand;

    DECLARE_EVENT_TABLE()
};


//---------------------------------------------------------------------




#endif
