/*
 * This source file is part of the Atlantis Little Helper program.
 * Copyright (C) 2001 Maxim Shariy.
 *
 * Atlantis Little Helper is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Atlantis Little Helper is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Atlantis Little Helper; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "stdhdr.h"

#include "wx/listctrl.h"

#include "cstr.h"
#include "collection.h"
#include "cfgfile.h"
#include "files.h"
#include "atlaparser.h"
#include "consts.h"
#include "consts_ah.h"
#include "hash.h"

#include "ahapp.h"
#include "ahframe.h"
#include "listpane.h"
#include "unitpane.h"
#include "utildlgs.h"
#include "unitpanefltr.h"
#include "editpane.h"
#include "mappane.h"


BEGIN_EVENT_TABLE(CUnitPaneFltr, wxListCtrl)
    EVT_LIST_ITEM_SELECTED   (list_units_hex_fltr, CUnitPaneFltr::OnSelected)
    EVT_LIST_COL_CLICK       (list_units_hex_fltr, CUnitPaneFltr::OnColClicked) 
    EVT_LIST_ITEM_RIGHT_CLICK(list_units_hex_fltr, CUnitPaneFltr::OnRClick) 
    EVT_IDLE                 (CUnitPaneFltr::OnIdle)

    EVT_MENU             (menu_Popup_Filter ,  CUnitPaneFltr::OnPopupMenuFilter   )
    EVT_MENU             (menu_Popup_SetSort,  CUnitPaneFltr::OnPopupMenuSetSort   )


END_EVENT_TABLE()



//--------------------------------------------------------------------------

CUnitPaneFltr::CUnitPaneFltr(wxWindow *parent, wxWindowID id)
              :CUnitPane(parent, id)
{
    m_ColClickedFltr = -1;
    m_IsUpdating     = FALSE;
}

//--------------------------------------------------------------------------

typedef enum                      {GT=0,GE,   EQ,   LE,   LT,  NE, NOP} eCompareOp;

void CUnitPaneFltr::Update(CUnitFilterDlg * pFilter)
{
    CStr             Property[SIMPLE_FLTR_COUNT];
    CStr             Compare [SIMPLE_FLTR_COUNT];
    CStr             sValue  [SIMPLE_FLTR_COUNT];
    long             lValue  [SIMPLE_FLTR_COUNT];
    int              i,k, unit;
    CStr             ConfigKey;
    eCompareOp       CompareOp[SIMPLE_FLTR_COUNT];
    CUnit          * pUnit;
    BOOL             ok;
    EValueType       type;
    const void     * value;
    CStr             TrackingGroup;
    CLongSortColl    Tracking(32);
    const char     * p;
    CStr             S;


    for (i=0; i<SIMPLE_FLTR_COUNT; i++)
    {
        if (pFilter)
        {
            Property[i] = pFilter->m_cbProperty[i]->GetValue();
            Compare [i] = pFilter->m_cbCompare [i]->GetValue();
            sValue  [i] = pFilter->m_tcValue   [i]->GetValue();
    
//            ConfigKey.Format("%s%d", SZ_KEY_UNIT_FLTR_PROPERTY, i);  gpApp->SetConfig(m_sConfigSection.GetData(), ConfigKey.GetData(), Property[i].GetData());
//            ConfigKey.Format("%s%d", SZ_KEY_UNIT_FLTR_COMPARE , i);  gpApp->SetConfig(m_sConfigSection.GetData(), ConfigKey.GetData(), Compare [i].GetData());
//            ConfigKey.Format("%s%d", SZ_KEY_UNIT_FLTR_VALUE   , i);  gpApp->SetConfig(m_sConfigSection.GetData(), ConfigKey.GetData(), sValue  [i].GetData());
        }
        else
        {
            CStr sConfSet;
            sConfSet = gpApp->GetConfig(m_sConfigSection.GetData(), SZ_KEY_FLTR_SET);
            
            ConfigKey.Format("%s%d", SZ_KEY_UNIT_FLTR_PROPERTY, i);  Property[i] = gpApp->GetConfig(sConfSet.GetData(), ConfigKey.GetData());
            ConfigKey.Format("%s%d", SZ_KEY_UNIT_FLTR_COMPARE , i);  Compare [i] = gpApp->GetConfig(sConfSet.GetData(), ConfigKey.GetData());
            ConfigKey.Format("%s%d", SZ_KEY_UNIT_FLTR_VALUE   , i);  sValue  [i] = gpApp->GetConfig(sConfSet.GetData(), ConfigKey.GetData());
        }

        Property[i].TrimRight(TRIM_ALL);    Property[i].TrimLeft(TRIM_ALL);
        Compare [i].TrimRight(TRIM_ALL);    Compare [i].TrimLeft(TRIM_ALL);
        sValue  [i].TrimRight(TRIM_ALL);    sValue  [i].TrimLeft(TRIM_ALL);

        CompareOp[i] = NOP;
        for (k=GT; k<NOP; k++)
            if (0==stricmp(FILTER_OPERATION[k], Compare[i].GetData()))
            {
                CompareOp[i] = (eCompareOp)k;
                break;
            }
        lValue[i] = atol(sValue[i].GetData());
    }

    // prepare tracking group
    if (pFilter)
    {
        TrackingGroup = pFilter->m_TrackingGroup;
        gpApp->SetConfig(m_sConfigSection.GetData(), SZ_KEY_UNIT_FLTR_TRACKING, TrackingGroup.GetData());
    }
    else
        TrackingGroup = gpApp->GetConfig(m_sConfigSection.GetData(), SZ_KEY_UNIT_FLTR_TRACKING);
    
    TrackingGroup.TrimRight(TRIM_ALL);
    if (!TrackingGroup.IsEmpty())
    {
        p = gpApp->GetConfig(SZ_SECT_UNIT_TRACKING, TrackingGroup.GetData());
        while (p && *p)
        {
            p = S.GetToken(p, ',');
            Tracking.Insert((void*)atol(S.GetData()));
        }
    }
    
    
    InsertUnitInit();

    for (unit=0; unit<gpApp->m_pAtlantis->m_Units.Count(); unit++)
    {
        ok    = TRUE;
        pUnit = (CUnit*)gpApp->m_pAtlantis->m_Units.At(unit);

        if (!TrackingGroup.IsEmpty())
            ok = Tracking.Search((void*)pUnit->Id, i);
        else
            for (i=0; i<SIMPLE_FLTR_COUNT; i++)
            {
                if (!Property[i].IsEmpty() && (NOP!=CompareOp[i]))
                {
                    if ( pUnit->GetProperty(Property[i].GetData(), type, value, eNormal))
                    {
                        switch (type)
                        {
                        case eLong:
                            switch (CompareOp[i])
                            {
                            case GT: ok = ((long)value >  lValue[i]); break;
                            case GE: ok = ((long)value >= lValue[i]); break;
                            case EQ: ok = ((long)value == lValue[i]); break;
                            case LE: ok = ((long)value <= lValue[i]); break;
                            case LT: ok = ((long)value <  lValue[i]); break;
                            case NE: ok = ((long)value != lValue[i]); break;
                            default: break;
                            }
                            break;
    
                        case eCharPtr:
                            switch (CompareOp[i])
                            {
                            case GT: ok = (stricmp((const char *)value, sValue[i].GetData()) >  0); break;
                            case GE: ok = (stricmp((const char *)value, sValue[i].GetData()) >= 0); break;
                            case EQ: ok = (stricmp((const char *)value, sValue[i].GetData()) == 0); break;
                            case LE: ok = (stricmp((const char *)value, sValue[i].GetData()) <= 0); break;
                            case LT: ok = (stricmp((const char *)value, sValue[i].GetData()) <  0); break;
                            case NE: ok = (stricmp((const char *)value, sValue[i].GetData()) != 0); break;
                            default: break;                                      
                            }
                            break;
    
                        default:
                            ok = FALSE;
                        }
                    }
                    else
                        ok = FALSE;
                }
    
                if (!ok)
                    break;
            }
        if (ok)
            InsertUnit(pUnit);
    }

    InsertUnitDone();
}

//--------------------------------------------------------------------------

void CUnitPaneFltr::InsertUnitInit()
{
    m_IsUpdating = TRUE;
    m_pUnits->DeleteAll();
}

//--------------------------------------------------------------------------

void CUnitPaneFltr::InsertUnit(CUnit * pUnit)
{
    m_pUnits->AtInsert(m_pUnits->Count(), pUnit);
}

//--------------------------------------------------------------------------

void CUnitPaneFltr::InsertUnitDone()
{
    m_pUnits->SetSortMode(m_SortKey, 3);
    SetData(sel_by_no, 0, TRUE);
    m_IsUpdating = FALSE;
}

//--------------------------------------------------------------------------

void CUnitPaneFltr::Sort()
{
    CListPane::Sort();
}

//--------------------------------------------------------------------------

void CUnitPaneFltr::OnSelected(wxListEvent& event)
{
    CUnit       * pUnit     = GetUnit(event.m_itemIndex);

    if (m_IsUpdating)
        return;
    gpApp->SelectUnit(pUnit);
}


//--------------------------------------------------------------------------

void CUnitPaneFltr::OnColClicked(wxListEvent& event)
{
    wxMenu  menu;

    m_ColClickedFltr = event.m_col;
    menu.Append(menu_Popup_SetSort  , "Set sort order");
    menu.Append(menu_Popup_Filter   , "Filter"        );

    PopupMenu( &menu, event.GetPoint().x, event.GetPoint().y);

}

//--------------------------------------------------------------------------

void CUnitPaneFltr::OnIdle(wxIdleEvent& event)
{
    CUnitPane::OnIdle(event);
}

//--------------------------------------------------------------------------

void CUnitPaneFltr::OnPopupMenuSetSort  (wxCommandEvent& event)
{
    m_ColClicked = m_ColClickedFltr;
}

//--------------------------------------------------------------------------

void CUnitPaneFltr::OnRClick(wxListEvent& event)
{
    wxMenu  menu;

    menu.Append(menu_Popup_Filter  , "Filter"        );

    PopupMenu( &menu, event.GetPoint().x, event.GetPoint().y);
}

//--------------------------------------------------------------------------

void CUnitPaneFltr::OnPopupMenuFilter  (wxCommandEvent& event) 
{
    CUnitFilterDlg dlg(m_pParent, m_sConfigSection.GetData());

    if (wxID_OK == dlg.ShowModal())
        Update(&dlg);
}

