/*
 * This source file is part of the Atlantis Little Helper program.
 * Copyright (C) 2001 Maxim Shariy.
 *
 * Atlantis Little Helper is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Atlantis Little Helper is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Atlantis Little Helper; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "stdhdr.h"

#include "wx/listctrl.h"
#include "wx/dialog.h"


#include "cstr.h"
#include "collection.h"
#include "cfgfile.h"
#include "files.h"
#include "atlaparser.h"
#include "consts.h"
#include "consts_ah.h"
#include "hash.h"
#include "ahapp.h"
#include "ahframe.h"
#include "utildlgs.h"


//==========================================================================

BEGIN_EVENT_TABLE(CShowOneDescriptionDlg, CResizableDlg)
    EVT_BUTTON        (-1, CShowOneDescriptionDlg::OnButton)
END_EVENT_TABLE()

CShowOneDescriptionDlg::CShowOneDescriptionDlg(wxWindow * parent, const char * title, const char * description)
                       :CResizableDlg( parent, title, SZ_SECT_WND_DESCR_ONE)
{
//    int i;

    wxBoxSizer * topsizer = new wxBoxSizer( wxVERTICAL );
    wxButton   * pBtnDone = new wxButton  (this, wxID_CANCEL, "Done" );
    wxButton   * pBtnSave = new wxButton  (this, wxID_OK    , "Save as" );
    wxTextCtrl * pText    = new wxTextCtrl(this, -1, description, wxDefaultPosition, wxDefaultSize,
                                           wxTE_MULTILINE | wxTE_READONLY | wxHSCROLL ); 
    const char * p;
    int          count = 0;
    CStr         S;

    topsizer->Add(  pText  ,
                    1,            // make vertically  stretchable
                    wxEXPAND );//| wxALL,    // make horizontally stretchable  //   and make border all around
                    //4 );          // set border width to 10
//    topsizer->Add(  pBtnDone,
//                    0,           // make horizontally unstretchable
//                    wxALL | wxALIGN_CENTER,       // make border all around (implicit top alignment)
//                    4 );        // set border width to 10  
//    topsizer->Add(  pBtnSave,
//                    0,           // make horizontally unstretchable
//                    wxALL | wxALIGN_CENTER,       // make border all around (implicit top alignment)
//                    4 );        // set border width to 10  

                    
    wxBoxSizer *button_sizer = new wxBoxSizer( wxHORIZONTAL );  
    
    button_sizer->Add(  pBtnSave,
                        0,           // make horizontally unstretchable
                        wxALL,       // make border all around (implicit top alignment)
                        10 );        // set border width to 10  
    button_sizer->Add(  pBtnDone,
                        0,           // make horizontally unstretchable
                        wxALL,       // make border all around (implicit top alignment)
                        10 );        // set border width to 10  
    
    topsizer->Add( button_sizer,
                   0,                // make vertically unstretchable
                   wxALIGN_CENTER ); // no border and centre horizontally
                    
                    
                    
                    
    SetAutoLayout( TRUE );     // tell dialog to use sizer
    SetSizer( topsizer );      // actually set the sizer
    topsizer->Fit( this );            // set size to minimum size as calculated by the sizer
    topsizer->SetSizeHints( this );   // set size hints to honour mininum size}

    pText->SetFocus();
    pBtnDone ->SetDefault();

    CResizableDlg::SetSize();

    pText->SetFont(*gpApp->m_Fonts[FONT_VIEW_DLG]);
    m_descr = description;

    p = description;
    while (p && *p && count++ < 3)
    {
        p = S.GetToken(p, '(', TRIM_ALL);
        p = S.GetToken(p, ')', TRIM_ALL);
        if (gpApp->SelectLand(S.GetData()))
            break;
    }

}


//--------------------------------------------------------------------------

void CShowOneDescriptionDlg::OnButton(wxCommandEvent& event)
{
    if (event.GetId()==wxID_OK)
    {
        int         err;
        wxString CurrentDir = wxGetCwd();
        if (!m_descr)
            return;
        wxFileDialog dialog(GetParent(), 
                            "Save current text", 
                            "", 
                            "", 
                            SZ_ALL_FILES, 
                            wxSAVE | wxHIDE_READONLY | wxOVERWRITE_PROMPT );
        err = dialog.ShowModal();
        wxSetWorkingDirectory(CurrentDir);
        
        if (wxID_OK == err)
        {
            CFileWriter F;
            if (F.Open(dialog.GetPath().c_str()))
            {
                F.WriteBuf(m_descr, strlen(m_descr));
                
                F.Close();
            }
            else
                wxMessageBox("Can not open file");
            
        }
    
    }
    else if (event.GetId()==wxID_CANCEL)
    {
        StoreSize();
        EndModal(wxID_CANCEL);
    }
}

//==========================================================================

BEGIN_EVENT_TABLE(CShowDescriptionListDlg, CResizableDlg)
    EVT_BUTTON        (-1, CShowDescriptionListDlg::OnButton)
    EVT_LISTBOX       (-1, CShowDescriptionListDlg::OnLBSelect)
    EVT_LISTBOX_DCLICK(-1, CShowDescriptionListDlg::OnLBDClick)
END_EVENT_TABLE()



CShowDescriptionListDlg::CShowDescriptionListDlg(wxWindow * parent, const char * title, CCollection * items)
                        :CResizableDlg( parent, title, SZ_SECT_WND_DESCR_LIST)
{
    long i;
    CBaseObject * pObj;
    wxButton    * pBtnView;
    wxButton    * pBtnStoreSize;
    
    
    m_pItems = items;
    m_pList = new wxListBox(this, -1, wxDefaultPosition, wxDefaultSize,
                               0, NULL, wxLB_SINGLE | wxLB_NEEDED_SB );

    for (i=0; i<items->Count(); i++)
    {
        pObj = (CBaseObject * )items->At(i);
        m_pList->Append(pObj->Name.GetData(), (void *)i);
    }


    wxBoxSizer *topsizer = new wxBoxSizer( wxVERTICAL );

    pBtnView         = new wxButton  (this, wxID_OK    , "View" );
    pBtnStoreSize    = new wxButton  (this, wxID_CANCEL, "Done" );

    topsizer->Add(  m_pList  ,
                    1,                  // make vertically stretchable
                    wxEXPAND | wxALL,   // make horizontally stretchable //   and make border all around
                    4 );                // set border width to 10

    wxBoxSizer *button_sizer = new wxBoxSizer( wxHORIZONTAL );  
    
    button_sizer->Add(  pBtnView,
                        0,           // make horizontally unstretchable
                        wxALL,       // make border all around (implicit top alignment)
                        10 );        // set border width to 10  
    button_sizer->Add(  pBtnStoreSize,
                        0,           // make horizontally unstretchable
                        wxALL,       // make border all around (implicit top alignment)
                        10 );        // set border width to 10  
    
    topsizer->Add( button_sizer,
                   0,                // make vertically unstretchable
                   wxALIGN_CENTER ); // no border and centre horizontally

    SetAutoLayout( TRUE );     // tell dialog to use sizer
    SetSizer( topsizer );      // actually set the sizer
    topsizer->Fit( this );            // set size to minimum size as calculated by the sizer
    topsizer->SetSizeHints( this );   // set size hints to honour mininum size}

    m_pList->SetSelection(0);
    m_pList->SetFocus();
    pBtnView->SetDefault();

    CResizableDlg::SetSize();
}

//--------------------------------------------------------------------------

void CShowDescriptionListDlg::OnButton(wxCommandEvent& event)
{
    if (event.GetId()==wxID_OK)
        OnLBDClick(event);
    else if (event.GetId()==wxID_CANCEL)
    {
        StoreSize();
        EndModal(wxID_CANCEL);
    }
}

//--------------------------------------------------------------------------

void CShowDescriptionListDlg::OnLBDClick(wxCommandEvent & event)
{
    CBaseObject  * pObj;

    pObj = (CBaseObject*)m_pItems->At((long)m_pList->GetClientData(m_pList->GetSelection()));
    if (pObj)
    {
        CShowOneDescriptionDlg dlg(GetParent(), pObj->Name.GetData(), pObj->Description.GetData());
        dlg.ShowModal();
        m_pList->SetFocus();
    }
}

//--------------------------------------------------------------------------

void CShowDescriptionListDlg::OnLBSelect(wxCommandEvent & event)
{
    CBaseObject  * pObj;
    CStr           S;
    const char   * p;

    pObj = (CBaseObject*)m_pItems->At((long)m_pList->GetClientData(m_pList->GetSelection()));
    if (pObj)
    {
        p = pObj->Name.GetData();
        while (p && *p)
        {
            p = S.GetToken(p, '(', TRIM_ALL);
            p = S.GetToken(p, ')', TRIM_ALL);
            if (gpApp->SelectLand(S.GetData()))
                break;
        }
    }
}

//==========================================================================

BEGIN_EVENT_TABLE(CExportMagesCSVDlg, CResizableDlg)
    EVT_BUTTON        (-1, CExportMagesCSVDlg::OnButton)
END_EVENT_TABLE()

int CExportMagesCSVDlg::borderwidth = 4;

CExportMagesCSVDlg::CExportMagesCSVDlg(wxWindow * parent, const char * fname)
                   :CResizableDlg( parent, "Export mages", SZ_SECT_WND_EXP_MAGES_CSV)
{
    wxButton    * pBtn;
    wxStaticText* pStatic;
    const char  * p;
    wxString      FName = fname;

    wxBoxSizer  * topsizer = new wxBoxSizer( wxVERTICAL );
    
    // -------------
    
    wxBoxSizer  * horsizer = new wxBoxSizer( wxHORIZONTAL );  
    
    pStatic  = new wxStaticText(this, -1, "Separator :", wxDefaultPosition, wxDefaultSize);
    horsizer->Add(pStatic, 0, wxALL, borderwidth );   


    m_pSeparator = new wxComboBox(this, -1, "", wxDefaultPosition, wxDefaultSize, 0, NULL, wxCB_DROPDOWN);
    m_pSeparator->Append(",");
    m_pSeparator->Append(";");
    p = SkipSpaces(gpApp->GetConfig(m_sConfigSection.GetData(), SZ_KEY_SEPARATOR));
    if (p && *p)
        m_pSeparator->SetValue(p);
    else
        m_pSeparator->SetSelection(0);
    horsizer->Add(m_pSeparator, 1, wxALL, borderwidth );   

    pStatic  = new wxStaticText(this, -1, "Orientation :", wxDefaultPosition, wxDefaultSize);
    horsizer->Add(pStatic, 0, wxALL, borderwidth );   


    m_pOrientation = new wxComboBox(this, -1, "", wxDefaultPosition, wxDefaultSize, 0, NULL, wxCB_DROPDOWN );
    m_pOrientation->Append(SZ_VERTICAL);
    m_pOrientation->Append(SZ_HORIZONTAL);
    p = SkipSpaces(gpApp->GetConfig(m_sConfigSection.GetData(), SZ_KEY_ORIENTATION));
    if (p && *p)
        m_pOrientation->SetValue(p);
    else
        m_pOrientation->SetSelection(0);
    horsizer->Add(m_pOrientation, 1, wxALL, borderwidth );   

    topsizer->Add( horsizer, 1, wxALIGN_CENTER | wxGROW ); 

    // -------------

    horsizer = new wxBoxSizer( wxHORIZONTAL );  

    pStatic  = new wxStaticText(this, -1, "Export to file :", wxDefaultPosition, wxDefaultSize);
    horsizer->Add(pStatic, 0, wxALL, borderwidth );   

    m_pFileName = new wxTextCtrl(this, -1, FName);
    horsizer->Add(m_pFileName, 1, wxALL, borderwidth );   
    
    pBtn = new wxButton  (this, wxID_SETUP , "Browse" );
    horsizer->Add(pBtn, 0, wxALL, borderwidth );   
    

    topsizer->Add( horsizer, 1, wxALIGN_CENTER | wxGROW  ); 

    // -------------
    
    horsizer = new wxBoxSizer( wxHORIZONTAL );  

    pBtn = new wxButton  (this, wxID_OK    , "Ok" );
    horsizer->Add(pBtn, 0, wxALL, borderwidth );   

    pBtn = new wxButton  (this, wxID_CANCEL    , "Cancel" );
    horsizer->Add(pBtn, 0, wxALL, borderwidth );   

    topsizer->Add( horsizer, 1, wxALIGN_CENTER | wxGROW  ); 

    // -------------

    SetAutoLayout( TRUE );     // tell dialog to use sizer
    SetSizer( topsizer );      // actually set the sizer
    topsizer->Fit( this );            // set size to minimum size as calculated by the sizer
    topsizer->SetSizeHints( this );   // set size hints to honour mininum size}

    CResizableDlg::SetSize();
}


void CExportMagesCSVDlg::OnButton(wxCommandEvent& event)
{
    wxString S;

    switch (event.GetId())
    {
    case wxID_OK:
        S = m_pSeparator->GetValue();
        gpApp->SetConfig(m_sConfigSection.GetData(), SZ_KEY_SEPARATOR, S);
        S = m_pOrientation->GetValue();
        gpApp->SetConfig(m_sConfigSection.GetData(), SZ_KEY_ORIENTATION, S);
        StoreSize();
        EndModal(wxID_OK);
        break;
        
    case wxID_CANCEL:
        StoreSize();
        EndModal(wxID_CANCEL);
        break;

    case wxID_SETUP:  // browse button
        {
            int         err;
            wxString CurrentDir = wxGetCwd();
            wxFileDialog dialog(GetParent(), 
                                "Save mages info", 
                                "", 
                                m_pFileName->GetValue(), 
                                SZ_CSV_FILES, 
                                wxSAVE | wxHIDE_READONLY | wxOVERWRITE_PROMPT );
            err = dialog.ShowModal();
            wxSetWorkingDirectory(CurrentDir);
        
            if (wxID_OK == err)
                m_pFileName->SetValue( dialog.GetPath() );
        }

        break;
    }
}

//==========================================================================

#define ID_BTN_TRACK     wxID_HIGHEST + 10
#define ID_CB_SET_NAME   wxID_HIGHEST + 11
#define ID_BTN_HELP      wxID_HIGHEST + 12

const char * FILTER_OPERATION[] = {">", ">=", "=", "<=", "<", "<>"};


BEGIN_EVENT_TABLE(CUnitFilterDlg, wxDialog)
    EVT_BUTTON(-1, CUnitFilterDlg::OnButton)
    EVT_COMBOBOX           (ID_CB_SET_NAME, CUnitFilterDlg::OnSetNameSelect)
    EVT_TEXT               (ID_CB_SET_NAME, CUnitFilterDlg::OnSetNameChange)
END_EVENT_TABLE()


//--------------------------------------------------------------------------


CUnitFilterDlg::CUnitFilterDlg(wxWindow *parent, const char * szConfigSection)
               :CResizableDlg( parent, "Unit Locator's filter", SZ_SECT_WND_UNITS_FLTR_DLG)
{
    wxBoxSizer * topsizer;
    wxBoxSizer * sizer   ;
    int          count;
    CStr         sConfSet;
    wxStaticText * stSetName;

    m_IsSaving           = FALSE;
    m_lastselect         = 0;
    m_sControllingConfig = szConfigSection;
    
    
    topsizer = new wxBoxSizer( wxVERTICAL );

    m_btnSet        = new wxButton(this, wxID_OK     , "Set"    );
    m_btnRemove     = new wxButton(this, wxID_NO     , "Clear"  );
    m_btnCancel     = new wxButton(this, wxID_CANCEL , "Cancel" );
    m_btnTracking   = new wxButton(this, ID_BTN_TRACK, "Tracking" );
    m_btnHelp       = new wxButton(this, ID_BTN_HELP , "Help" );
    m_cbSetName     = new wxComboBox(this, ID_CB_SET_NAME);
    stSetName       = new wxStaticText (this, -1, "Filter name:");

    sizer    = new wxBoxSizer( wxHORIZONTAL );
    sizer->Add(stSetName       , 1, wxALIGN_CENTER | wxALL , 5);
    sizer->Add(m_cbSetName     , 3, wxALIGN_CENTER | wxALL , 5);
    topsizer->Add(sizer        , 0, wxALIGN_CENTER | wxALL | wxGROW, 5 ); 
        
    
    for (count=0; count < SIMPLE_FLTR_COUNT; count++)
    {
        m_cbProperty[count] = new wxComboBox(this, -1); //, "", wxDefaultPosition, wxDefaultSize, 0, NULL, wxCB_DROPDOWN | wxCB_READONLY );
        m_cbCompare [count] = new wxComboBox(this, -1); //, "", wxDefaultPosition, wxDefaultSize, 0, NULL, wxCB_DROPDOWN | wxCB_READONLY );
        m_tcValue   [count] = new wxTextCtrl(this, -1);

        if (count>0)
        {
            sizer    = new wxBoxSizer( wxHORIZONTAL );
            sizer->Add(new wxStaticText(this, -1, "AND"), 0, wxALIGN_LEFT | wxALL, 5);
            topsizer->Add(sizer, 0, wxALIGN_CENTER ); 
        }

        sizer    = new wxBoxSizer( wxHORIZONTAL );
        sizer->Add(m_cbProperty[count], 0, wxALIGN_CENTER | wxALL, 5);
        sizer->Add(m_cbCompare [count], 0, wxALIGN_CENTER | wxALL, 5);
        sizer->Add(m_tcValue   [count], 0, wxALIGN_CENTER | wxALL, 5);
        topsizer->Add(sizer, 0, wxALIGN_CENTER ); 
    }

    sizer    = new wxBoxSizer( wxHORIZONTAL );
    sizer->Add(m_btnSet     , 0, wxALIGN_CENTER | wxALL, 5);
    sizer->Add(m_btnRemove  , 0, wxALIGN_CENTER | wxALL, 5);
    sizer->Add(m_btnTracking, 0, wxALIGN_CENTER | wxALL, 5);
    sizer->Add(m_btnCancel  , 0, wxALIGN_CENTER | wxALL, 5);
    sizer->Add(m_btnHelp    , 0, wxALIGN_CENTER | wxALL, 5);
    
    
    topsizer->Add(sizer, 0, wxALIGN_CENTER ); 

    SetAutoLayout( TRUE );     // tell dialog to use sizer
    SetSizer( topsizer );      // actually set the sizer
    topsizer->Fit( this );            // set size to minimum size as calculated by the sizer
    topsizer->SetSizeHints( this );   // set size hints to honour mininum size}

    m_cbProperty[0]->SetFocus();
    m_btnSet->SetDefault();

    
    sConfSet = gpApp->GetConfig(szConfigSection, SZ_KEY_FLTR_SET);
    sConfSet.TrimRight(TRIM_ALL);
    if (sConfSet.IsEmpty())
        sConfSet << SZ_SECT_UNIT_FILTER << "Default";
    LoadSetCombo(sConfSet.GetData());
    Init();
    Load(sConfSet.GetData());

    CResizableDlg::SetSize();
}

//--------------------------------------------------------------------------

void CUnitFilterDlg::LoadSetCombo(const char * setselect)
{
    const char * setsection;
    int          setnameoffs;
    int          x=0, i=0;
    
    m_IsSaving = TRUE;
    
    setnameoffs = strlen(SZ_SECT_UNIT_FILTER);
    setsection  = gpApp->GetNextSectionName(CONFIG_FILE_CONFIG, SZ_SECT_UNIT_FILTER);
    while (setsection)
    {
        if (0!=strnicmp(setsection, SZ_SECT_UNIT_FILTER, setnameoffs))
            break;
        m_cbSetName->Append(&setsection[setnameoffs]);
        if (0==stricmp(setsection, setselect))
            x = i;
        i++;
        setsection = gpApp->GetNextSectionName(CONFIG_FILE_CONFIG, setsection);
    }
    if (0==i)
        m_cbSetName->Append(&setselect[setnameoffs]);    
    m_cbSetName->SetSelection(x);
    
    m_IsSaving = FALSE;
}

//--------------------------------------------------------------------------

void CUnitFilterDlg::Init()
{
    int          count;
    int          i;
    const char * item;
    int          postlen;
    CStr         S;

    postlen = strlen(PRP_SKILL_DAYS_POSTFIX);
    for (count=0; count < SIMPLE_FLTR_COUNT; count++)
    {
        m_cbProperty[count]->Append("");
        m_cbCompare [count]->Append("");
    
        for (i=0; i<gpApp->m_pAtlantis->m_UnitPropertyNames.Count(); i++)
        {
            item = (const char *) gpApp->m_pAtlantis->m_UnitPropertyNames.At(i);
            
            // do not show 'skill days' property
            S = item;
            if (S.FindSubStrR(PRP_SKILL_DAYS_POSTFIX) != S.GetLength()-postlen) 
            {
                m_cbProperty[count]->Append(item);
            }
        }

        for (i=0; (unsigned)i<sizeof(FILTER_OPERATION)/sizeof(*FILTER_OPERATION); i++)
        {
            item = FILTER_OPERATION[i];
            m_cbCompare[count]->Append(item);
        }
    }
}

//--------------------------------------------------------------------------

void CUnitFilterDlg::Load(const char * szConfigSection)
{
    int          count;
    int          i;
    CStr         ConfigKey;
    const char * selvalue;
    int          selidx;

    m_sCurrentSection = szConfigSection;

    for (count=0; count < SIMPLE_FLTR_COUNT; count++)
    {
        ConfigKey.Format("%s%d", SZ_KEY_UNIT_FLTR_PROPERTY, count);  
        selvalue = gpApp->GetConfig(szConfigSection, ConfigKey.GetData());
        selidx   = 0;
        for (i=0; i<m_cbProperty[count]->Number(); i++)
            if (0==stricmp(m_cbProperty[count]->GetString(i), selvalue))
            {
                selidx = i;
                break;
            }
        m_cbProperty[count]->SetSelection(selidx);
        
        ConfigKey.Format("%s%d", SZ_KEY_UNIT_FLTR_COMPARE , count);  
        m_cbCompare [count]->SetValue(gpApp->GetConfig(szConfigSection, ConfigKey.GetData()));
        selvalue = gpApp->GetConfig(szConfigSection, ConfigKey.GetData());
        selidx   = 0;
        for (i=0; i<m_cbCompare[count]->Number(); i++)
            if (0==stricmp(m_cbCompare[count]->GetString(i), selvalue))
            {
                selidx = i;
                break;
            }
        m_cbCompare[count]->SetSelection(selidx);

        ConfigKey.Format("%s%d", SZ_KEY_UNIT_FLTR_VALUE   , count);  
        m_tcValue   [count]->SetValue(gpApp->GetConfig(szConfigSection, ConfigKey.GetData()));
    }
}

//--------------------------------------------------------------------------

BOOL CUnitFilterDlg::IsValid()
{
    CStr         S1, S2;
    int          count;
    BOOL         isvalid = FALSE;
    
    for (count=0; count < SIMPLE_FLTR_COUNT; count++)
    {
        S1 = m_cbProperty[count]->GetValue();   S1.TrimRight(TRIM_ALL);
        S2 = m_cbCompare[count]->GetValue();    S2.TrimRight(TRIM_ALL);
        if (!S1.IsEmpty() && !S2.IsEmpty())
            isvalid = TRUE;
    }
    return isvalid;
}

//--------------------------------------------------------------------------

void CUnitFilterDlg::Save()
{
    int          count;
    CStr         ConfigKey;
    CStr         SetName;
    BOOL         found = FALSE;
    
    m_IsSaving = TRUE;
    
    gpApp->RemoveSection(m_sCurrentSection.GetData());
    
    if (IsValid())
    {
        SetName = m_sCurrentSection;
        SetName.DelSubStr(0, strlen(SZ_SECT_UNIT_FILTER));

        for (count=0; count<m_cbSetName->Number(); count++)
            if (0==stricmp(m_cbSetName->GetString(count), SetName.GetData()))
            {
                found = TRUE;
                break;
            }
        if (!found)
            m_cbSetName->Append(SetName.GetData());    
        
                
        for (count=0; count < SIMPLE_FLTR_COUNT; count++)
        {
            ConfigKey.Format("%s%d", SZ_KEY_UNIT_FLTR_PROPERTY, count);  
            gpApp->SetConfig(m_sCurrentSection.GetData(), ConfigKey.GetData(), m_cbProperty[count]->GetValue());
    
            ConfigKey.Format("%s%d", SZ_KEY_UNIT_FLTR_COMPARE , count);  
            gpApp->SetConfig(m_sCurrentSection.GetData(), ConfigKey.GetData(), m_cbCompare[count]->GetValue());
    
            ConfigKey.Format("%s%d", SZ_KEY_UNIT_FLTR_VALUE   , count);  
            gpApp->SetConfig(m_sCurrentSection.GetData(), ConfigKey.GetData(), m_tcValue[count]->GetValue());
        }
    }
    
    m_IsSaving = FALSE;
}

//--------------------------------------------------------------------------

void CUnitFilterDlg::Reload(const char * setname)
{
    CStr  Sect;

    if (m_IsSaving)
        return;
        
    Save();
    Sect << SZ_SECT_UNIT_FILTER << setname;
    Load(Sect.GetData());
}

//--------------------------------------------------------------------------

void  CUnitFilterDlg::OnSetNameChange(wxCommandEvent& event)
{
    if (m_lastselect < time(NULL))
        Reload(m_cbSetName->GetValue()); // returns the old value on windoze when value is selected
}

//--------------------------------------------------------------------------

void  CUnitFilterDlg::OnSetNameSelect(wxCommandEvent& event)
{
    Reload(m_cbSetName->GetString(m_cbSetName->GetSelection())); 
    m_lastselect = time(NULL);
}

//--------------------------------------------------------------------------


void CUnitFilterDlg::OnButton(wxCommandEvent& event)
{
    wxObject * object = event.GetEventObject();
    int        i;

    m_TrackingGroup.Empty();
    if (object == m_btnRemove)
    {
        for (i=0; i < SIMPLE_FLTR_COUNT; i++)
        {
            m_cbProperty[i]->SetValue("");
            m_cbCompare [i]->SetValue("");
            m_tcValue   [i]->SetValue("");
        }
    }
    else if (object == m_btnTracking)
    {
        int          sectidx;
        CStr         S;
        const char * szName;
        const char * szValue;
    
        sectidx = gpApp->GetSectionFirst(SZ_SECT_UNIT_TRACKING, szName, szValue);
        while (sectidx >= 0)
        {
            if (!S.IsEmpty())
                S << ",";
            S << szName;
            sectidx = gpApp->GetSectionNext(sectidx, SZ_SECT_UNIT_TRACKING, szName, szValue);
        }
        if (S.IsEmpty())
            S = "Default";
        
        CComboboxDlg dlg(this, "Show a tracking group", "Select a group to load units from.", S.GetData());
        if (wxID_OK == dlg.ShowModal())
        {
            m_TrackingGroup = dlg.m_Choice;
            
            StoreSize();
            EndModal(wxID_OK);
        }
    }
    else if (object == m_btnSet)
    {
        Save();
        gpApp->SetConfig(m_sControllingConfig.GetData(), SZ_KEY_FLTR_SET, m_sCurrentSection.GetData());
        StoreSize();
        EndModal(wxID_OK);
    }
    else if (object == m_btnCancel)
    {
        StoreSize();
        EndModal(wxID_CANCEL);
    }
    else if (object == m_btnHelp)
    {
        wxMessageBox("\n"
                     "- Filter names can not be modified.\n"
                     "- To add a filter type in the name for it and set conditions.\n"
                     "- To delete a filter remove conditions from it."
                     "\n"
                     "- Tracking will display a group of units. Add a unit to the group by\n"
                     "  right-cliking on the unit in the unit list."
                     );
    }
}

//==========================================================================

BEGIN_EVENT_TABLE(CHexExportDlg, wxDialog)
    EVT_BUTTON(-1, CHexExportDlg::OnButton)
END_EVENT_TABLE()


//--------------------------------------------------------------------------


CHexExportDlg::CHexExportDlg(wxWindow *parent)
              :CResizableDlg( parent, "Hex export options", SZ_SECT_WND_EXP_HEXES, wxDEFAULT_DIALOG_STYLE)
{
    wxBoxSizer * topsizer;
    wxBoxSizer * sizer   ;
    wxBoxSizer * colsizer;
    wxStaticBox* box;
    CStr         ConfigKey;
    CStr         S;

    m_btnOk           = new wxButton     (this, wxID_OK    , "Ok"    );
    m_btnCancel       = new wxButton     (this, wxID_CANCEL, "Cancel" );
    m_btnBrowse       = new wxButton     (this, -1,          "Browse" );
    m_tcFName         = new wxTextCtrl   (this, -1, "", wxDefaultPosition, wxSize(150, -1));

    
    
    m_rbHexNew        = new wxRadioButton(this, -1, "New"     , wxDefaultPosition, wxDefaultSize, wxRB_GROUP);
    m_rbHexCurrent    = new wxRadioButton(this, -1, "Current" , wxDefaultPosition, wxDefaultSize, 0);
    m_rbHexSelected   = new wxRadioButton(this, -1, "Selected", wxDefaultPosition, wxDefaultSize, 0);
    
    m_rbFileOverwrite = new wxRadioButton(this, -1, "Overwrite it" ,wxDefaultPosition, wxDefaultSize, wxRB_GROUP);
    m_rbFileAppend    = new wxRadioButton(this, -1, "Append to it" , wxDefaultPosition, wxDefaultSize, 0);

    m_chbInclStructs  = new wxCheckBox   (this, -1, "Structures");
    m_chbInclUnits    = new wxCheckBox   (this, -1, "Units");
    m_chbInclResources= new wxCheckBox   (this, -1, "Resources");
    m_chbInclTurnNoAcl= new wxCheckBox   (this, -1, "Turn No a la Atlaclient");

    
    topsizer = new wxBoxSizer( wxVERTICAL );

    sizer     = new wxBoxSizer( wxHORIZONTAL );
    sizer->Add(new wxStaticText(this, -1, "Export to file"), 0, wxALIGN_LEFT | wxALL, 5);
    sizer->Add(m_tcFName  , 0, wxALIGN_CENTER | wxALL, 5);
    sizer->Add(m_btnBrowse, 0, wxALIGN_CENTER | wxALL, 5);
    topsizer->Add(sizer, 0, wxALIGN_CENTER | wxALL, 10); 

    

    sizer    = new wxBoxSizer( wxHORIZONTAL );

    box      = new wxStaticBox(this, -1, "If file exists:");
    colsizer = new wxStaticBoxSizer(box, wxVERTICAL );
    colsizer->Add(m_rbFileOverwrite , 0, wxALIGN_LEFT | wxALL, 2);
    colsizer->Add(m_rbFileAppend    , 0, wxALIGN_LEFT | wxALL, 2);
    sizer->Add(colsizer, 0, wxALIGN_LEFT | wxALL, 5 ); 

    box      = new wxStaticBox(this, -1, "Hex(es):");
    colsizer = new wxStaticBoxSizer(box, wxVERTICAL );
    colsizer->Add(m_rbHexNew     , 0, wxALIGN_LEFT | wxALL, 2);
    colsizer->Add(m_rbHexCurrent , 0, wxALIGN_LEFT | wxALL, 2);
    colsizer->Add(m_rbHexSelected, 0, wxALIGN_LEFT | wxALL, 2);
    sizer->Add(colsizer, 0, wxALIGN_LEFT | wxALL, 5 ); 

    box      = new wxStaticBox(this, -1, "Include:");
    colsizer = new wxStaticBoxSizer(box, wxVERTICAL );
    colsizer->Add(m_chbInclStructs  , 0, wxALIGN_LEFT | wxALL, 2);
    colsizer->Add(m_chbInclUnits    , 0, wxALIGN_LEFT | wxALL, 2);
    colsizer->Add(m_chbInclResources, 0, wxALIGN_LEFT | wxALL, 2);
    colsizer->Add(m_chbInclTurnNoAcl, 0, wxALIGN_LEFT | wxALL, 2);
    sizer->Add(colsizer, 0, wxALIGN_LEFT | wxALL, 5 ); 

    topsizer->Add(sizer, 0, wxALIGN_CENTER ); 
    


    sizer    = new wxBoxSizer( wxHORIZONTAL );
    sizer->Add(m_btnOk     , 0, wxALIGN_CENTER | wxALL, 5);
    sizer->Add(m_btnCancel , 0, wxALIGN_CENTER | wxALL, 5);
    topsizer->Add(sizer, 0, wxALIGN_CENTER | wxALL, 10 ); 
    



    SetAutoLayout( TRUE );     // tell dialog to use sizer
    SetSizer( topsizer );      // actually set the sizer
    topsizer->Fit( this );            // set size to minimum size as calculated by the sizer
    topsizer->SetSizeHints( this );   // set size hints to honour mininum size}

    m_tcFName->SetFocus();
    m_btnOk->SetDefault();


    CResizableDlg::SetSize();
}


//--------------------------------------------------------------------------

void CHexExportDlg::OnButton(wxCommandEvent& event)
{
    wxObject * object = event.GetEventObject();
        
    if (object == m_btnBrowse)
    {
        int          err;
        wxString     CurrentDir = wxGetCwd();
        wxFileDialog dialog(GetParent(), 
                            "Export hexes to", 
                            "", 
                            m_tcFName->GetValue(), 
                            SZ_ALL_FILES, 
                            wxSAVE | wxHIDE_READONLY );
        err = dialog.ShowModal();
        wxSetWorkingDirectory(CurrentDir);
    
        if (wxID_OK == err)
            m_tcFName->SetValue( dialog.GetPath() );
    }
    else if (object==m_btnOk)
    {
        StoreSize();
        EndModal(wxID_OK);
    }
    else if (object==m_btnCancel)
    {
        StoreSize();
        EndModal(wxID_CANCEL);
    }
}

//==========================================================================



BEGIN_EVENT_TABLE(CComboboxDlg, wxDialog)
    EVT_BUTTON(-1, CComboboxDlg::OnButton)
END_EVENT_TABLE()

//--------------------------------------------------------------------------

CComboboxDlg::CComboboxDlg(wxWindow *parent, const char * szTitle, const char * szMessage, const char * szChoices)
             :wxDialog(parent, -1, szTitle, wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE)
{
    wxBoxSizer * topsizer;
    wxBoxSizer * sizer   ;
    CStr         S;
    wxButton   * btnOk;
    wxButton   * btnCancel;

    btnOk       = new wxButton     (this, wxID_OK    , "Ok"    );
    btnCancel   = new wxButton     (this, wxID_CANCEL, "Cancel" );
    m_cbChoices = new wxComboBox(this, -1, "", wxDefaultPosition, wxDefaultSize, 0, NULL, wxCB_DROPDOWN);
    
    
    topsizer = new wxBoxSizer( wxVERTICAL );

    topsizer->Add(new wxStaticText(this, -1, szMessage), 0, wxALIGN_LEFT | wxALL, 5);
    topsizer->Add(m_cbChoices                          , 0, wxALIGN_CENTER | wxALL | wxGROW, 5);
    
    sizer     = new wxBoxSizer( wxHORIZONTAL );
    sizer->Add(btnOk    , 0, wxALIGN_CENTER | wxALL, 5);
    sizer->Add(btnCancel, 0, wxALIGN_CENTER | wxALL, 5);
    topsizer->Add(sizer,   0, wxALIGN_CENTER | wxALL, 10); 

    while (szChoices && *szChoices)
    {
        szChoices = S.GetToken(szChoices, ',');
        m_cbChoices->Append(S.GetData());    
    }
    
    SetAutoLayout( TRUE );     // tell dialog to use sizer
    SetSizer( topsizer );      // actually set the sizer
    topsizer->Fit( this );            // set size to minimum size as calculated by the sizer
    topsizer->SetSizeHints( this );   // set size hints to honour mininum size}

    m_cbChoices->SetFocus();
    btnOk->SetDefault();
    
    Centre( wxBOTH | wxCENTER_FRAME);
}


//--------------------------------------------------------------------------

void CComboboxDlg::OnButton(wxCommandEvent& event)
{
    m_Choice = m_cbChoices->GetValue();
    event.Skip();
}

//--------------------------------------------------------------------------


