/*
 * This source file is part of the Atlantis Little Helper program.
 * Copyright (C) 2001 Maxim Shariy.
 *
 * Atlantis Little Helper is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Atlantis Little Helper is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Atlantis Little Helper; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __AH_UTIL_DIALOGS_INCL__
#define __AH_UTIL_DIALOGS_INCL__



//--------------------------------------------------------------------------

class CShowOneDescriptionDlg : public CResizableDlg
{
public:
    CShowOneDescriptionDlg(wxWindow * parent, const char * title, const char * description);
    void OnButton(wxCommandEvent& event);
private:
    const char * m_descr;
    DECLARE_EVENT_TABLE()
};


//--------------------------------------------------------------------------


class CShowDescriptionListDlg : public CResizableDlg
{
public:
    CShowDescriptionListDlg(wxWindow * parent, const char * title, CCollection * items);

private:
    void OnButton(wxCommandEvent& event);
    void OnLBDClick(wxCommandEvent & event);
    void OnLBSelect(wxCommandEvent & event);
    wxListBox   * m_pList;
    CCollection * m_pItems;

    DECLARE_EVENT_TABLE()
};


//--------------------------------------------------------------------------


class CExportMagesCSVDlg : public CResizableDlg
{
public:
    CExportMagesCSVDlg(wxWindow * parent, const char * fname);
    wxComboBox  * m_pSeparator;
    wxComboBox  * m_pOrientation;
    wxTextCtrl  * m_pFileName;

private:
    void OnButton(wxCommandEvent& event);
    static int borderwidth;

    DECLARE_EVENT_TABLE()
};


//--------------------------------------------------------------------------

#define SIMPLE_FLTR_COUNT 3

extern const char * FILTER_OPERATION[];
 
 
class CUnitFilterDlg : public CResizableDlg
{
public:
    CUnitFilterDlg(wxWindow *parent, const char * szConfigSection);


    wxComboBox * m_cbProperty[SIMPLE_FLTR_COUNT];
    wxComboBox * m_cbCompare [SIMPLE_FLTR_COUNT];
    wxTextCtrl * m_tcValue   [SIMPLE_FLTR_COUNT];
    wxComboBox * m_cbSetName;

    CStr         m_TrackingGroup;

private:
    void Init();
    void Save();
    BOOL IsValid();
    void Load        (const char * szConfigSection);
    void LoadSetCombo(const char * setselect);
    void Reload      (const char * setname);
    
    void OnButton(wxCommandEvent& event);
    void OnSetNameChange(wxCommandEvent& event);
    void OnSetNameSelect(wxCommandEvent& event);

    CStr         m_sControllingConfig;
    CStr         m_sCurrentSection;
    BOOL         m_IsSaving;
    time_t       m_lastselect;    
    wxButton   * m_btnSet   ;
    wxButton   * m_btnRemove;
    wxButton   * m_btnCancel;
    wxButton   * m_btnTracking;
    wxButton   * m_btnHelp;


    DECLARE_EVENT_TABLE()
};
 
//--------------------------------------------------------------------------

class CHexExportDlg : public CResizableDlg
{
public:
    CHexExportDlg(wxWindow *parent);

    void OnButton(wxCommandEvent& event);

    wxTextCtrl * m_tcFName;

    wxRadioButton * m_rbHexNew;
    wxRadioButton * m_rbHexCurrent;
    wxRadioButton * m_rbHexSelected;

    wxRadioButton * m_rbFileOverwrite;
    wxRadioButton * m_rbFileAppend;
    
    wxCheckBox    * m_chbInclStructs;
    wxCheckBox    * m_chbInclUnits;
    wxCheckBox    * m_chbInclTurnNoAcl;
    wxCheckBox    * m_chbInclResources;

private:
    wxButton   * m_btnOk    ;
    wxButton   * m_btnCancel;
    wxButton   * m_btnBrowse;
    
    DECLARE_EVENT_TABLE()
};

//--------------------------------------------------------------------------

class CComboboxDlg : public wxDialog
{
public:
    CComboboxDlg(wxWindow *parent, const char * szTitle, const char * szMessage, const char * szChoices);

    CStr m_Choice;

private:
    void OnButton(wxCommandEvent& event);
    
    wxComboBox    * m_cbChoices;
        
    DECLARE_EVENT_TABLE()
};

//--------------------------------------------------------------------------


#endif


